<?php
/**
 * 2007-2021 PrestaShop.
 *
 * NOTICE OF LICENSE
 *
 * This source file is subject to the Academic Free License (AFL 3.0)
 * that is bundled with this package in the file LICENSE.txt.
 * It is also available through the world-wide-web at this URL:
 * http://opensource.org/licenses/afl-3.0.php
 * If you did not receive a copy of the license and are unable to
 * obtain it through the world-wide-web, please send an email
 * to license@prestashop.com so we can send you a copy immediately.
 *
 * DISCLAIMER
 *
 * Do not edit or add to this file if you wish to upgrade PrestaShop to newer
 * versions in the future. If you wish to customize PrestaShop for your
 * needs please refer to http://www.prestashop.com for more information.
 *
 *  @author    PrestaShop SA <contact@prestashop.com>
 *  @copyright 2007-2021 PrestaShop SA
 *  @license   http://opensource.org/licenses/afl-3.0.php  Academic Free License (AFL 3.0)
 *  International Registered Trademark & Property of PrestaShop SA
 */

if (!defined('_PS_VERSION_')) {
    // module validation
    exit;
}

class DfxComment extends ObjectModel
{
    public $id;
    public $id_dfxblog_comment;
    public $active = 0;
    public $comment;
    public $id_dfxblog_post;
    public $id_customer;
    public $customer_name;
    public $customer_email;
    public $date_add;
    public $date_upd;

    public static $definition = array(
        'table' => 'dfxblog_comment',
        'primary' => 'id_dfxblog_comment',
        'multilang' => false,
        'multilang_shop' => false,
        'fields' => array(
            'active' => array('type' => self::TYPE_BOOL, 'validate' => 'isBool'),
            'id_dfxblog_post' => array('type' => self::TYPE_INT, 'validate' => 'isUnsignedInt'),
            'id_customer' => array('type' => self::TYPE_INT, 'validate' => 'isUnsignedInt'),
            'customer_name' => array('type' => self::TYPE_STRING, 'size' => 64, 'required' => true),
            'customer_email' => array('type' => self::TYPE_STRING, 'validate' => 'isEmail', 'size' => 128, 'required' => true),
            'comment' => array('type' => self::TYPE_HTML, 'validate' => 'isCleanHtml', 'required' => true),
            'date_add' => array('type' => self::TYPE_DATE, 'validate' => 'isDate'),
            'date_upd' => array('type' => self::TYPE_DATE, 'validate' => 'isDate'),
        ),
    );

    /* admin function */
    public function delete()
    {
        $ret = parent::delete();

        Hook::exec('actionDfxBlogUpdate', array('post' => $this));

        return $ret;
    }

    public function update($null_values = false)
    {
        $ret = parent::update($null_values);

        Hook::exec('actionDfxBlogUpdate', array('post' => $this));

        return $ret;
    }

    public function add($autodate = true, $null_values = false)
    {
        $ret = parent::add($autodate, $null_values);

        Hook::exec('actionDfxBlogUpdate', array('post' => $this));

        return $ret;
    }

    /* front-end function */
    public static function getLastCommentByCustomer($id_dfxblog_post, $id_customer)
    {
        $results = Db::getInstance()->getValue(
            'SELECT c.`date_add`
			FROM `'._DB_PREFIX_.'dfxblog_comment` c
			WHERE c.`id_dfxblog_post` = '.(int) $id_dfxblog_post.'
			AND c.`id_customer` = '.(int) $id_customer.'
			ORDER BY c.`date_add` DESC'
        );

        return $results;
    }

    public static function getCurrentComments($limit = 3)
    {
        $sql = 'SELECT *
		FROM `'._DB_PREFIX_.'dfxblog_comment` c
		WHERE c.`active` = 1
		ORDER BY c.`date_add` DESC
		LIMIT '.(int) $limit;

        $results = Db::getInstance()->executeS($sql);

        return $results;
    }
}
