<?php
/**
 * 2007-2021 PrestaShop.
 *
 * NOTICE OF LICENSE
 *
 * This source file is subject to the Academic Free License (AFL 3.0)
 * that is bundled with this package in the file LICENSE.txt.
 * It is also available through the world-wide-web at this URL:
 * http://opensource.org/licenses/afl-3.0.php
 * If you did not receive a copy of the license and are unable to
 * obtain it through the world-wide-web, please send an email
 * to license@prestashop.com so we can send you a copy immediately.
 *
 * DISCLAIMER
 *
 * Do not edit or add to this file if you wish to upgrade PrestaShop to newer
 * versions in the future. If you wish to customize PrestaShop for your
 * needs please refer to http://www.prestashop.com for more information.
 *
 *  @author    PrestaShop SA <contact@prestashop.com>
 *  @copyright 2007-2021 PrestaShop SA
 *  @license   http://opensource.org/licenses/afl-3.0.php  Academic Free License (AFL 3.0)
 *  International Registered Trademark & Property of PrestaShop SA
 */

if (!defined('_PS_VERSION_')) {
    // module validation
    exit;
}

class DfxPost extends ObjectModel
{
    public $id;
    public $id_dfxblog_post;
    public $id_dfxblog_category;
    public $name;
    public $image = '';
    public $thumb = '';
    public $active = 1;
    public $description_short;
    public $description;
    public $allow_comment = 1;
    public $allow_views = 1;
    public $allow_likes = 1;
    public $showauthor = 1;
    public $posted_by;
    public $related_posts;
    public $related_products;
    public $link_rewrite;
    public $meta_title;
    public $meta_keywords;
    public $meta_description;
    public $tags;
    public $date_add;
    public $date_upd;
    public $likes;
    public $views;
    public $id_image = false;
    public $parent_category = false;
    public $category;
    public $category_url;
    

    public static $definition = array(
        'table' => 'dfxblog_post',
        'primary' => 'id_dfxblog_post',
        'multilang' => true,
        'multilang_shop' => false,
        'fields' => array(
            'id_dfxblog_category' => array('type' => self::TYPE_INT, 'validate' => 'isUnsignedInt'),
            'allow_comment' => array('type' => self::TYPE_BOOL, 'validate' => 'isUnsignedInt'),
            'allow_likes' => array('type' => self::TYPE_BOOL, 'validate' => 'isUnsignedInt'),
            'allow_views' => array('type' => self::TYPE_BOOL, 'validate' => 'isUnsignedInt'),
            'active' => array('type' => self::TYPE_BOOL, 'validate' => 'isBool', 'required' => true),
            'posted_by' => array('type' => self::TYPE_STRING, 'size' => 100),
            'image' => array('type' => self::TYPE_STRING, 'validate' => 'isCatalogName'),
            'thumb' => array('type' => self::TYPE_STRING, 'lang' => false,),
            'related_posts' => array('type' => self::TYPE_STRING, 'size' => 255),
            'related_products' => array('type' => self::TYPE_STRING, 'size' => 100),
            'date_add' => array('type' => self::TYPE_DATE, 'validate' => 'isDate'),
            'date_upd' => array('type' => self::TYPE_DATE, 'validate' => 'isDate'),
            'name' => array('type' => self::TYPE_STRING, 'lang' => true, 'validate' => 'isCatalogName',
            'required' => true, 'size' => 128),
            'link_rewrite' => array('type' => self::TYPE_STRING, 'lang' => true, 'validate' => 'isLinkRewrite',
            'required' => true, 'size' => 128),
            'description_short' => array('type' => self::TYPE_HTML, 'lang' => true, 'validate' => 'isCleanHtml'),
            'description' => array('type' => self::TYPE_HTML, 'lang' => true, 'validate' => 'isCleanHtml'),
            'meta_title' => array('type' => self::TYPE_STRING, 'lang' => true, 'validate' => 'isGenericName',
            'size' => 128),
            'meta_description' => array('type' => self::TYPE_STRING, 'lang' => true, 'validate' => 'isGenericName',
            'size' => 255),
            'meta_keywords' => array('type' => self::TYPE_STRING, 'lang' => true, 'validate' => 'isGenericName',
            'size' => 255),
            'likes' => array('type' => self::TYPE_INT, 'validate' => 'isUnsignedInt'),
            'views' => array('type' => self::TYPE_INT, 'validate' => 'isUnsignedInt'),
            'tags' => array('type' => self::TYPE_STRING, 'lang' => true, 'validate' => 'isGenericName'),
        ),
    );

    /* admin function */
    public function __construct($id_dfxblog_post = null, $id_lang = null)
    {
        parent::__construct($id_dfxblog_post, $id_lang);
        $folder = Image::getImgFolderStatic($this->id);
        $category = new DfxCategory($this->id_dfxblog_category, $id_lang);
        $this->category = $category->name;
        $this->category_rewrite = $category->link_rewrite;

        $id_lang = (int) Context::getContext()->language->id;

        if (is_array($this->link_rewrite)) {
            $this->url = self::getLink($this->id_dfxblog_post, $this->link_rewrite[$id_lang], $id_lang);
            $this->category_url = DfxCategory::getLink($category->id_dfxblog_category, $category->link_rewrite[$id_lang], $id_lang);
        } else {
            $this->url = self::getLink($this->id_dfxblog_post, $this->link_rewrite, $id_lang);
            $this->category_url = DfxCategory::getLink($category->id_dfxblog_category, $category->link_rewrite, $id_lang);
        }
        
        if ($category->id_parent > 0) {
            $parent = new DfxCategory($category->id_parent, $id_lang);
            $this->parent_category = $parent;
        }
        
        
        if ($this->related_products) {
            $this->related_products = Tools::unSerialize($this->related_products);
        }
        
        if ($this->related_posts) {
            $this->related_posts = Tools::unSerialize($this->related_posts);
        }
        
        $this->id_image = ($this->id &&
            file_exists(_DFXPRO_IMG_DIR_.$folder.$this->id.'.'.$this->image_format)) ?
            (int) $this->id : false;
    }

    public function getObjectLink($id_lang)
    {
        if (is_array($this->link_rewrite)) {
            $objectlink = self::getLink($this->id_dfxblog_post, $this->link_rewrite[$id_lang], $id_lang);
        } else {
            $objectlink = self::getLink($this->id_dfxblog_post, $this->link_rewrite, $id_lang);
        }
        return $objectlink;
    }

    public static function getLink($id_dfxblog_post, $rewrite, $id_lang = null, $id_shop = null)
    {
        $url = dfxblog::myRealUrl();
        $id_lang = Context::getContext()->language->id;

        $dispatcher = Dispatcher::getInstance();
        
        $params = array();
        $params['id'] = $id_dfxblog_post;
        $params['rewrite'] = $rewrite;

        $dispatcher = Dispatcher::getInstance();

        return $url.$dispatcher->createUrl('dfx_post_rule', $id_lang, $params);
    }


    public function updateCategories($category_ids)
    {
        if (empty($category_ids)) {
            return true;
        }

        Db::getInstance()->delete(
            'dfxblog_post',
            'id_dfxblog_post = '.(int) $this->id.
            ' AND id_dfxblog_category NOT IN ('.implode(',', array_map('intval', $category_ids)).')'
        );

        $data = array();
        foreach ($category_ids as $category_id) {
            $data[] = array(
                'id_dfxblog_category' => (int) $category_id,
                'id_dfxblog_post' => (int) $this->id,
            );
        }

        Db::getInstance()->insert('dfxblog_post', $data, false, true, Db::INSERT_IGNORE);

        return true;
    }

    public function delete()
    {
        $this->deleteImg();

        $ret = parent::delete();

        return $ret;
    }
    
    public function save($null_values = false, $autodate = true)
    {
        return (int) $this->id > 0 ? $this->update($null_values) : $this->add($autodate, $null_values);
    }

    public function update($null_values = false)
    {
        if ($this->related_products) {
            $this->related_products = serialize($this->related_products);
        }
        
        if ($this->related_posts) {
            $this->related_posts = serialize($this->related_posts);
        }
        
        parent::update($null_values);

        return true;
    }
    

    public function add($autodate = true, $null_values = false)
    {
        if ($this->related_products) {
            $this->related_products = serialize($this->related_products);
        }
        
        if ($this->related_posts) {
            $this->related_posts = serialize($this->related_posts);
        }
        
        parent::add($autodate, $null_values);
        return true;
    }

    public static function getImgFolderStatic($id = null)
    {
        if (!is_numeric($id)) {
            return false;
        }

        $folders = str_split((string) $id);

        return implode('/', $folders).'/';
    }

    public function deleteImg()
    {
        if (!$this->id) {
            return false;
        }

        if ($this->id_image) {
            $folder = Image::getImgFolderStatic($this->id);
            $image_path = _DFXPRO_IMG_DIR_.$folder.$this->id.'.'.$this->image_format;

            if (file_exists($image_path)) {
                unlink($image_path);

                // delete thumbnails
                $image_resize_path = array(
                    _DFXPRO_IMG_DIR_.$folder.$this->id.'-small.'.$this->image_format,
                    _DFXPRO_IMG_DIR_.$folder.$this->id.'-list.'.$this->image_format,
                    _DFXPRO_IMG_DIR_.$folder.$this->id.'-large.'.$this->image_format,
                );
                foreach ($image_resize_path as $path) {
                    if (file_exists($path)) {
                        unlink($path);
                    }
                }
            }
        }

        return true;
    }

    public static function changeRating($way = 'up', $id_dfxblog_post)
    {
        if ($way == 'up') {
            $sql = 'UPDATE `'._DB_PREFIX_.'dfxblog_post` SET `likes` = `likes` + 1 WHERE id_dfxblog_post = '.$id_dfxblog_post;
        } elseif ($way == 'down') {
            $sql = 'UPDATE `'._DB_PREFIX_.'dfxblog_post` SET `likes` = `likes` - 1 WHERE id_dfxblog_post = '.$id_dfxblog_post;
        } else {
            return;
        }
        
        $result = Db::getInstance()->Execute($sql);

        $sql = 'SELECT `likes` FROM `'._DB_PREFIX_.'dfxblog_post` WHERE id_dfxblog_post = '.$id_dfxblog_post;

        $res = Db::getInstance()->ExecuteS($sql);

        return $res;
    }
    
    public function increaseViewsNb()
    {
        $sql = 'UPDATE `'._DB_PREFIX_.'dfxblog_post` SET `views` = `views` + 1 WHERE id_dfxblog_post = '.$this->id_dfxblog_post;
       
        $result = Db::getInstance()->Execute($sql);

        return $result;
    }
    

    public static function getPostCategoryFull($id_post = '')
    {
        $ret = array();
        $row = Db::getInstance(_PS_USE_SQL_SLAVE_)->executeS(
            'SELECT cp.`id_dfxblog_category`
			FROM `'._DB_PREFIX_.'dfxblog_post` cp
			WHERE cp.`id_dfxblog_post` = '.(int) $id_post
        );

        foreach ($row as $val) {
            $ret[] = $val['id_dfxblog_category'];
        }

        return $ret;
    }

    public function getRelatedPostsLight()
    {
        $posts = array();

        if ($this->related_posts != '') {
            $sql = 'SELECT p.`id_dfxblog_post`, pl.`name`
			FROM `'._DB_PREFIX_.'dfxblog_post` p
			LEFT JOIN `'._DB_PREFIX_.'dfxblog_post_lang` pl ON (pl.`id_dfxblog_post` = p.`id_dfxblog_post` AND 
            pl.`id_lang` = '.(int) Context::getContext()->language->id.')
			WHERE p.`id_dfxblog_post` IN ('.implode(array_map('intval', $this->related_posts), ',').')';

            $items = Db::getInstance()->executeS($sql);

            foreach ($items as $item) {
                $posts[$item['id_dfxblog_post']] = trim($item['name']);
            }
        }

        return $posts;
    }

    public function getProductsAutocompleteInfo($id_lang = null)
    {
        if (!$id_lang) {
            $id_lang = Context::getContext()->language->id;
        }

        $products = array();

        if (!empty($this->related_products)) {
            $query = 'SELECT p.`id_product`, p.`reference`, pl.name
                FROM `'._DB_PREFIX_.'product` p
                LEFT JOIN `'._DB_PREFIX_.'product_lang` pl ON (pl.`id_product` = p.`id_product` AND pl.`id_lang` = '.(int) $id_lang.')
                WHERE p.`id_product` IN ('.implode(array_map('intval', $this->related_products), ',').')';

            $rows = Db::getInstance(_PS_USE_SQL_SLAVE_)->executeS($query);

            foreach ($rows as $row) {
                $products[$row['id_product']] = trim($row['name']).(!empty($row['reference']) ? ' (ref: '.$row['reference'].')' : '');
            }
        }

        return $products;
    }
    
    public static function getProductsByArrayId($array_product_id = null, $id_lang = null)
    {
        if (empty($array_product_id)) {
            return false;
        }
        if (!$id_lang) {
            $id_lang = Context::getContext()->language->id;
        }

        $sql = new DbQuery();
        $sql->select(
            'p.*, product_shop.*, stock.out_of_stock, IFNULL(stock.quantity, 0) as quantity, pl.`description`,
            pl.`description_short`, pl.`link_rewrite`, pl.`meta_description`, pl.`meta_keywords`, 
            pl.`meta_title`, pl.`name`, MAX(image_shop.`id_image`) id_image, il.`legend`, 
            m.`name` AS manufacturer_name,
            DATEDIFF(
                product_shop.`date_add`,
                DATE_SUB(
                    NOW(),
                    INTERVAL '.(Validate::isUnsignedInt(Configuration::get('PS_NB_DAYS_NEW_PRODUCT')) ?
                    Configuration::get('PS_NB_DAYS_NEW_PRODUCT') : 20).' DAY
                )
            ) > 0 AS new'
        );

        $sql->from('product', 'p');
        $sql->join(Shop::addSqlAssociation('product', 'p'));
        $sql->leftJoin('product_lang', 'pl', 'p.`id_product` = pl.`id_product` AND pl.`id_lang` = '.(int) $id_lang.Shop::addSqlRestrictionOnLang('pl'));
        $sql->leftJoin('image', 'i', 'i.`id_product` = p.`id_product`');
        $sql->join(Shop::addSqlAssociation('image', 'i', false, 'image_shop.cover=1'));
        $sql->leftJoin('image_lang', 'il', 'i.`id_image` = il.`id_image` AND il.`id_lang` = '.(int) $id_lang);
        $sql->leftJoin('manufacturer', 'm', 'm.`id_manufacturer` = p.`id_manufacturer`');

        $sql->where('p.`id_product` IN ('.implode(array_map('intval', $array_product_id), ',').')');
        $sql->where('product_shop.`active` = 1');

        $sql->groupBy('product_shop.id_product');

        if (Combination::isFeatureActive()) {
            $sql->select('MAX(product_attribute_shop.id_product_attribute) id_product_attribute');
            $sql->leftOuterJoin('product_attribute', 'pa', 'p.`id_product` = pa.`id_product`');
            $sql->join(Shop::addSqlAssociation('product_attribute', 'pa', false, 'product_attribute_shop.default_on = 1'));
        }
        $sql->join(Product::sqlStock('p', Combination::isFeatureActive() ? 'product_attribute_shop' : 0));

        $result = Db::getInstance(_PS_USE_SQL_SLAVE_)->executeS($sql);

        if (!$result) {
            return false;
        }

        return Product::getProductsProperties((int) $id_lang, $result);
    }

    /* front-end function */
    public static function getInCategories($id_dfxblog_post, $id_lang = null, $active = true)
    {
        if (!isset($id_lang)) {
            $id_lang = (int) Context::getContext()->language->id;
        }

        $sql = 'SELECT c.`id_dfxblog_category`, cl.`name`, cl.`link_rewrite`
				FROM `'._DB_PREFIX_.'dfxblog_category` c
				LEFT JOIN `'._DB_PREFIX_.'dfxblog_post` cp
					ON c.`id_dfxblog_category` = cp.`id_dfxblog_category`
				LEFT JOIN `'._DB_PREFIX_.'dfxblog_category_lang` cl
					ON (c.`id_dfxblog_category` = cl.`id_dfxblog_category`
					AND cl.`id_lang` = '.(int) $id_lang.')
				WHERE cp.`id_dfxblog_post` = '.(int) $id_dfxblog_post
            .($active ? ' AND c.`active` = 1' : '')
            .' GROUP BY c.id_dfxblog_category';

        $result = Db::getInstance(_PS_USE_SQL_SLAVE_)->executeS($sql);

        if ($result) {
            $dfxlink = new DfxLink();

            foreach ($result as &$row) {
                $row['link'] = DfxCategory::getLink($row['id_dfxblog_category'], $row['link_rewrite'], $id_lang);
            }
        }

        return $result;
    }

    public static function getCommentNumber($id_dfxblog_post)
    {
        $result = (int) Db::getInstance(_PS_USE_SQL_SLAVE_)->getValue(
            'SELECT COUNT(`id_dfxblog_comment`) AS "nbc"
            FROM `'._DB_PREFIX_.'dfxblog_comment` pc
            WHERE `id_dfxblog_post` = '.(int) $id_dfxblog_post.' AND `active` = 1'
        );

        return $result;
    }
    

    public function getRelatedPosts($id_lang)
    {
        if (!isset($id_lang)) {
            $id_lang = (int) Context::getContext()->language->id;
        }

        $result = false;

        if ($this->related_posts != '') {
            $sql = 'SELECT p.`id_dfxblog_post`, pl.name, pl.`link_rewrite`
			FROM `'._DB_PREFIX_.'dfxblog_post` p
			LEFT JOIN `'._DB_PREFIX_.'dfxblog_post_lang` pl ON (pl.`id_dfxblog_post` = p.`id_dfxblog_post` AND 
            pl.`id_lang` = '.$id_lang.')
			WHERE p.`id_dfxblog_post` IN ('.implode(array_map('intval', $this->related_posts), ',').')';

            $result = Db::getInstance()->executeS($sql);
            
            if ($result) {
                foreach ($result as &$row) {
                    $row['url'] = self::getLink($row['id_dfxblog_post'], $row['link_rewrite'], $id_lang);
                }
            }
        }
        

        return self::getPostsInfo($result, $id_lang);
    }

    public function getComments($active)
    {
        $comments = false;

        $sql = 'SELECT *
			FROM `'._DB_PREFIX_.'dfxblog_comment` c
			WHERE c.`id_dfxblog_post` = '.(int) $this->id.'
			'.($active ? 'AND `active` = 1' : '').'
			ORDER BY c.`date_add` ASC';

        $results = Db::getInstance()->executeS($sql);

        if ($results) {
            $comments = $results;
        }

        return $comments;
    }

    public static function getLatestPosts($id_lang, $limit = 3)
    {
        if (!isset($id_lang)) {
            $id_lang = (int) Context::getContext()->language->id;
        }

        $sql = 'SELECT p.*, pl.name, pl.description_short, pl.`link_rewrite`
			FROM `'._DB_PREFIX_.'dfxblog_post` p
			LEFT JOIN `'._DB_PREFIX_.'dfxblog_post_lang` pl ON (pl.`id_dfxblog_post` = p.`id_dfxblog_post` AND 
            pl.`id_lang` = '.$id_lang.')
			WHERE p.`active` = 1
			ORDER BY p.`date_add` DESC
			LIMIT '.$limit;

        $result = Db::getInstance()->executeS($sql);
        
        if ($result) {
            foreach ($result as &$row) {
                $row['url'] = self::getLink($row['id_dfxblog_post'], $row['link_rewrite'], $id_lang);
            }
        }
            
        return self::getPostsInfo($result, $id_lang);
    }

    public static function getAllPosts(
        $id_lang,
        $p,
        $n,
        $get_total = false,
        $order_by = null,
        $order_way = null,
        $active = true
    ) {
        if ($p < 1) {
            $p = 1;
        }

        if (empty($order_by)) {
            $order_by = 'date_add';
        } else {
            $order_by = Tools::strtolower($order_by);
        }

        if (empty($order_way)) {
            $order_way = 'DESC';
        }
        if ($order_by == 'id_dfxblog_post' ||
            $order_by == 'date_add' ||
            $order_by == 'date_upd' ||
            $order_by == 'position'
        ) {
            $order_by_prefix = 'p';
        } elseif ($order_by == 'name') {
            $order_by_prefix = 'pl';
        }

        if (!Validate::isBool($active) || !Validate::isOrderBy($order_by) || !Validate::isOrderWay($order_way)) {
            die(Tools::displayError());
        }

        /* Return only the number of post */
        if ($get_total) {
            $sql = 'SELECT COUNT(p.`id_dfxblog_post`) AS total
					FROM `'._DB_PREFIX_.'dfxblog_post` p'.
                ($active ? ' WHERE p.`active` = 1' : '');

            return (int) Db::getInstance(_PS_USE_SQL_SLAVE_)->getValue($sql);
        }

        $sql = 'SELECT p.*, pl.`name`, pl.`description_short`, pl.`description`,
					pl.`link_rewrite`, pl.`tags`
				FROM `'._DB_PREFIX_.'dfxblog_post` p
				LEFT JOIN `'._DB_PREFIX_.'dfxblog_post_lang` pl
					ON (p.`id_dfxblog_post` = pl.`id_dfxblog_post`
					AND pl.`id_lang` = '.(int) $id_lang.')'
            .($active ? ' WHERE p.`active` = 1' : '')
            .' GROUP BY p.id_dfxblog_post';

        $sql .= ' ORDER BY '.(isset($order_by_prefix) ?
        $order_by_prefix.'.' : '').'`'.pSQL($order_by).'` '.pSQL($order_way).'
			LIMIT '.(((int) $p - 1) * (int) $n).','.(int) $n;

        $result = Db::getInstance(_PS_USE_SQL_SLAVE_)->executeS($sql);
        
        if ($result) {
            foreach ($result as &$row) {
                $row['related_products'] = Tools::unSerialize($row['related_products']);
                $category_rewrite = DfxCategory::getRewriteByCategory($row['id_dfxblog_category'], $id_lang);
                $category_obj = new DfxCategory($row['id_dfxblog_category'], $id_lang);
                $category_url = DfxCategory::getLink($category_obj->id, $category_obj->link_rewrite, $id_lang);
                $row['url'] = self::getLink($row['id_dfxblog_post'], $row['link_rewrite'], $id_lang);
                $row['category'] = $category_obj->name;
                $row['category_rewrite'] = $category_obj->link_rewrite;
                $row['category_url'] = $category_url;
            }
        }
        

        return self::getPostsInfo($result, $id_lang);
    }

    public static function getArchives()
    {
        $sql = 'SELECT Month(`date_add`) as month, Year(`date_add`) as year
			FROM `'._DB_PREFIX_.'dfxblog_post`
			WHERE `active` = 1
			GROUP BY month, year
			ORDER BY month DESC, year DESC';

        $result = Db::getInstance(_PS_USE_SQL_SLAVE_)->executeS($sql);

        return $result;
    }
    
        
    public static function getListBlogs($id_category, $id_lang, $page_number, $nb_products, $order_by, $order_way, $condition = array(), $is_active = false, $id_shop = null)
    {
        // module validation
        !is_null($nb_products) ? true : $nb_products = 10;
        !is_null($page_number) ? true : $page_number = 0;

        if (!$id_shop) {
            $context = Context::getContext();
            $id_shop = $context->shop->id;
        }

        if (empty($id_lang)) {
            $id_lang = (int)Configuration::get('PS_LANG_DEFAULT');
        }
        if ($page_number < 1) {
            $page_number = 1;
        }
        if ($nb_products < 1) {
            $nb_products = 10;
        }
        if (empty($order_by) || $order_by == 'position') {
            $order_by = 'date_add';
        }
        if (empty($order_way)) {
            $order_way = 'DESC';
        }
        if ($order_by == 'id_dfxblog_post' || $order_by == 'date_add' || $order_by == 'date_upd') {
            $order_by_prefix = 'c';
        } elseif ($order_by == 'title') {
            $order_by_prefix = 'c';
        }
        if (!Validate::isOrderBy($order_by) || !Validate::isOrderWay($order_way)) {
            die(Tools::displayError());
        }
        if (strpos($order_by, '.') > 0) {
            $order_by = explode('.', $order_by);
            $order_by_prefix = $order_by[0];
            $order_by = $order_by[1];
        }
        
        $where = '';

        if ($id_category) {
            // validate module
            $where .= ' AND c.id_dfxblog_category='.(int)$id_category;
        }
        
        if (isset($condition['type'])) {
            switch ($condition['type']) {
                case 'tag':
                    $tmp = explode(',', $condition['tag']);

                    if (!empty($tmp) && count($tmp) > 1) {
                        $t = array();
                        foreach ($tmp as $tag) {
                            // validate module
                            $t[] = 'l.tags LIKE "%'.pSQL(trim($tag)).'%"';
                        }
                        $where .= ' AND ('.implode(' OR ', $t).') ';
                    } else {
                        // validate module
                        $where .= ' AND l.tags LIKE "%'.pSQL($condition['tag']).'%"';
                    }
                    break;
                case 'samecat':
                    $where .= ' AND c.id_dfxblog_post!='.(int)$condition['id_dfxblog_post'];
                    break;
            }
        }

        if ($is_active) {
            // validate module
            $where .= ' AND c.active=1';
        }

        $query = '
		SELECT c.*, l.*, l.meta_title as title, blc.link_rewrite as category_link_rewrite , blc.name as category_title
		FROM  '._DB_PREFIX_.'dfxblog_post c
		LEFT JOIN '._DB_PREFIX_.'dfxblog_post_lang l
		 ON (c.id_dfxblog_post = l.id_dfxblog_post) and  l.id_lang='.(int)$id_lang
         .' LEFT JOIN '._DB_PREFIX_.'dfxblog_post_shop s
                 ON  (c.id_dfxblog_post = s.id_dfxblog_post) and s.id_shop='.(int)$id_shop
                .' LEFT JOIN '._DB_PREFIX_.'dfxblog_category bc ON  bc.id_dfxblog_category = c.id_dfxblog_category '
                .' LEFT JOIN '._DB_PREFIX_.'dfxblog_category_lang blc
                 ON blc.id_dfxblog_category=bc.id_dfxblog_category and blc.id_lang='.(int)$id_lang
                 .' '.Shop::addSqlAssociation('post', 'c').'
                WHERE l.id_lang = '.(int)$id_lang.$where.'
		GROUP BY c.id_dfxblog_post
		 ';

        $query .= 'ORDER BY '.(isset($order_by_prefix) ? pSQL($order_by_prefix).'.' : '').pSQL($order_by).' '
                .pSQL($order_way).' LIMIT '.(int)(($page_number - 1) * $nb_products).', '.(int)$nb_products;

        $data = Db::getInstance(_PS_USE_SQL_SLAVE_)->executeS($query);
        
        if ($data) {
            foreach ($data as &$row) {
                $category_rewrite = DfxCategory::getRewriteByCategory($row['id_dfxblog_category'], $id_lang);
                $category_obj = new DfxCategory($row['id_dfxblog_category'], $id_lang);
                $category_url = DfxCategory::getLink($category_obj->id, $category_obj->link_rewrite, $id_lang);
                $row['url'] = self::getLink($row['id_dfxblog_post'], $row['link_rewrite'], $id_lang);
                $row['category'] = $category_obj->name;
                $row['category_rewrite'] = $category_obj->link_rewrite;
                $row['category_url'] = $category_url;
            }
        }
        
        return $data;
    }
    
    public static function getPostsByArchive(
        $month,
        $year,
        $id_lang,
        $p,
        $n,
        $get_total = false,
        $order_by = null,
        $order_way = null,
        $active = true
    ) {
        if ($p < 1) {
            $p = 1;
        }

        if (empty($order_by)) {
            $order_by = 'date_add';
        } else {
            $order_by = Tools::strtolower($order_by);
        }

        if (empty($order_way)) {
            $order_way = 'DESC';
        }
        if ($order_by == 'id_dfxblog_post' ||
            $order_by == 'date_add' ||
            $order_by == 'date_upd' ||
            $order_by == 'position'
        ) {
            $order_by_prefix = 'p';
        } elseif ($order_by == 'name') {
            $order_by_prefix = 'pl';
        }

        if (!Validate::isBool($active) || !Validate::isOrderBy($order_by) || !Validate::isOrderWay($order_way)) {
            die(Tools::displayError());
        }

        /* Return only the number of post */
        if ($get_total) {
            $sql = 'SELECT COUNT(p.`id_dfxblog_post`) AS total
					FROM `'._DB_PREFIX_.'dfxblog_post` p
					WHERE Month(`date_add`) = '.(int) $month.' AND Year(`date_add`) = '.(int) $year.
                ($active ? ' AND p.`active` = 1' : '');

            return (int) Db::getInstance(_PS_USE_SQL_SLAVE_)->getValue($sql);
        }

        $sql = 'SELECT p.*, pl.`name`, pl.`description_short`,
					pl.`link_rewrite`, pl.`tags`
				FROM `'._DB_PREFIX_.'dfxblog_post` p
				LEFT JOIN `'._DB_PREFIX_.'dfxblog_post_lang` pl
					ON (p.`id_dfxblog_post` = pl.`id_dfxblog_post`
					AND pl.`id_lang` = '.(int) $id_lang.')
				WHERE Month(`date_add`) = '.(int) $month.' AND Year(`date_add`) = '.(int) $year.
            ($active ? ' AND p.`active` = 1' : '')
            .' GROUP BY p.id_dfxblog_post';

        $sql .= ' ORDER BY '.(isset($order_by_prefix) ?
        $order_by_prefix.'.' : '').'`'.pSQL($order_by).'` '.pSQL($order_way).'
			LIMIT '.(((int) $p - 1) * (int) $n).','.(int) $n;

        $result = Db::getInstance(_PS_USE_SQL_SLAVE_)->executeS($sql);
    
        if ($result) {
            foreach ($result as &$row) {
                $row['url'] = self::getLink($row['id_dfxblog_post'], $row['link_rewrite'], $id_lang);
            }
        }
            
        return self::getPostsInfo($result, $id_lang);
    }

    public function getPreviousPost($id_lang)
    {
        if (!isset($id_lang)) {
            $id_lang = (int) Context::getContext()->language->id;
        }

        $post = false;

        $sql = 'SELECT p.`id_dfxblog_post`, pl.name, pl.`link_rewrite`, p.`date_add`
			FROM `'._DB_PREFIX_.'dfxblog_post` p
			LEFT JOIN `'._DB_PREFIX_.'dfxblog_post_lang` pl ON (pl.`id_dfxblog_post` = p.`id_dfxblog_post` AND 
            pl.`id_lang` = '.$id_lang.')
			WHERE p.`date_add` < \''.$this->date_add.'\'
			ORDER BY p.`date_add` DESC';

        $row = Db::getInstance(_PS_USE_SQL_SLAVE_)->getRow($sql);

        if ($row) {
            $post = $this->getPostInfo($row, $id_lang);
        }

        return $post;
    }

    public function getNextPost($id_lang)
    {
        if (!isset($id_lang)) {
            $id_lang = (int) Context::getContext()->language->id;
        }

        $post = false;

        $sql = 'SELECT p.`id_dfxblog_post`, pl.name, pl.`link_rewrite`, p.`date_add`
			FROM `'._DB_PREFIX_.'dfxblog_post` p
			LEFT JOIN `'._DB_PREFIX_.'dfxblog_post_lang` pl ON (pl.`id_dfxblog_post` = p.`id_dfxblog_post` AND 
            pl.`id_lang` = '.$id_lang.')
			WHERE p.`date_add` > \''.$this->date_add.'\'
			ORDER BY p.`date_add`';

        $row = Db::getInstance(_PS_USE_SQL_SLAVE_)->getRow($sql);

        if ($row) {
            $post = $this->getPostInfo($row, $id_lang);
        }

        return $post;
    }

    public static function getPostsInfo($rows, $id_lang)
    {
        $dfxposts = array();

        if (is_array($rows)) {
            foreach ($rows as $row) {
                $dfxposts[] = self::getPostInfo($row, $id_lang, '-list');
            }
        }

        return $dfxposts;
    }

    public static function getPostInfo($row, $id_lang)
    {
        $dfxlink = new DfxLink();
        $folder = Image::getImgFolderStatic($row['id_dfxblog_post']);
        $row['link'] = self::getLink($row['id_dfxblog_post'], $row['link_rewrite'], $id_lang);
        $row['bloglink'] = $dfxlink->getBlogLink($id_lang);
        $row['id_image'] = file_exists(
            _DFXPRO_IMG_DIR_.$folder.$row['id_dfxblog_post'].'.jpg'
        ) ? (int) $row['id_dfxblog_post'] : false;
        $row['categories'] = self::getInCategories($row['id_dfxblog_post'], $id_lang);
        $row['num_comment'] = self::getCommentNumber($row['id_dfxblog_post']);

        return $row;
    }
}
