<?php
/**
 * 2007-2021 PrestaShop.
 *
 * NOTICE OF LICENSE
 *
 * This source file is subject to the Academic Free License (AFL 3.0)
 * that is bundled with this package in the file LICENSE.txt.
 * It is also available through the world-wide-web at this URL:
 * http://opensource.org/licenses/afl-3.0.php
 * If you did not receive a copy of the license and are unable to
 * obtain it through the world-wide-web, please send an email
 * to license@prestashop.com so we can send you a copy immediately.
 *
 * DISCLAIMER
 *
 * Do not edit or add to this file if you wish to upgrade PrestaShop to newer
 * versions in the future. If you wish to customize PrestaShop for your
 * needs please refer to http://www.prestashop.com for more information.
 *
 *  @author    PrestaShop SA <contact@prestashop.com>
 *  @copyright 2007-2021 PrestaShop SA
 *  @license   http://opensource.org/licenses/afl-3.0.php  Academic Free License (AFL 3.0)
 *  International Registered Trademark & Property of PrestaShop SA
 */

require_once _PS_MODULE_DIR_.'dfxblog/loader.php';

class AdminDfxCategoryController extends ModuleAdminController
{
    protected $category = null;
    public $imageType = 'jpg';
    
    public function __construct()
    {
        $this->table = 'dfxblog_category';
        $this->className = 'DfxCategory';
        $this->id_dfxblog_category = true;
        $this->multishop_context = Shop::CONTEXT_ALL;
        $this->lang = true;
        
        
        $this->addRowAction('edit');
        $this->addRowAction('view');
        $this->addRowAction('delete');
        
        parent::__construct();
        $this->bootstrap = true;
        $this->explicitSelect = true;
        $this->allow_export = true;

        $this->context = Context::getContext();

        $this->fieldImageSettings = array(
            'name' => 'image',
            'dir' => _DFXCAT_,
        );
        
        
        $this->fields_list = array(
            'id_dfxblog_category' => array(
                'title' => $this->l('ID'),
                'align' => 'center',
                'class' => 'fixed-width-xs',
                'orderby' => false,
            ),
            'name' => array(
                'title' => $this->l('Name'),
                'orderby' => false,
            ),
            'position' => array(
                'title' => $this->l('Position'),
                'class' => 'fixed-width-sm',
                'align' => 'center',
                'orderby' => false,
            ),
            'active' => array(
                'title' => $this->l('Displayed'),
                'active' => 'status',
                'type' => 'bool',
                'class' => 'fixed-width-xs',
                'align' => 'center',
                'orderby' => false,
            ),
        );

        $this->bulk_actions = array('delete' => array('text' => $this->l('Delete selected')));
    }

    public function init()
    {
        parent::init();
        $this->_select = 'id_parent, level_depth';

        if (($id_dfxblog_category = Tools::getvalue('id_dfxblog_category')) && $this->action != 'select_delete') {
            $this->category = new DfxCategory($id_dfxblog_category);
        } else {
            $this->category = new DfxCategory();
        }
    }

    public function renderForm()
    {
        $dfxcategory = new DfxCategory();
        $id_dfxblog_category = (int) Tools::getvalue('id_dfxblog_category');

        if ($id_dfxblog_category) {
            $dfxcategory = new DfxCategory($id_dfxblog_category);
        }

        $image = _DFXCAT_IMG_DIR_.$dfxcategory->id.'.'.$this->imageType;
        $image_url = file_exists($image) ? __PS_BASE_URI__.'img/'.$this->fieldImageSettings['dir'].$dfxcategory->id.'.'.$this->imageType : false;
        $image_size = $image_url ? filesize($image) / 1000 : false;
        

        $id_lang = (int) Configuration::get('PS_LANG_DEFAULT');
        $categorys = DfxCategory::getCategories($id_lang, false, true);
        $cat_options = $this->createCategoryTreeOptions($categorys);

        array_unshift($categorys, array('id' => 0, 'name' => '-- Root --'));


        $this->fields_form = array(
            'tinymce' => true,
            'legend' => array(
                'title' => $this->l('Blog Category'),
                'icon' => 'icon-book',
            ),
            'input' => array(
                array(
                    'type' => 'text',
                    'label' => $this->l('Name:'),
                    'name' => 'name',
                    'lang' => true,
                    'size' => 48,
                    'required' => true,
                    'class' => 'copy2friendlyUrl',
                    'hint' => $this->l('Invalid characters:').' <>;=#{}',
                ),
                array(
                    'type' => 'switch',
                    'label' => $this->l('Displayed:'),
                    'name' => 'active',
                    'required' => false,
                    'class' => 't',
                    'is_bool' => true,
                    'values' => array(
                        array(
                            'id' => 'active_on',
                            'value' => 1,
                            'label' => $this->l('Enabled'),
                        ),
                        array(
                            'id' => 'active_off',
                            'value' => 0,
                            'label' => $this->l('Disabled'),
                        ),
                    ),
                ),
                 array(
                    'type' => 'select',
                    'label' => $this->l('Parent category:'),
                    'name' => 'id_parent',
                    'required' => true,
                    'options' => array(
                        'query' => $categorys,
                        'id' => 'id',
                        'name' => 'name',
                    ),
                ),
                array(
                    'type' => 'textarea',
                    'label' => $this->l('Description:'),
                    'name' => 'description',
                    'autoload_rte' => true,
                    'lang' => true,
                    'rows' => 10,
                    'cols' => 100,
                    //'hint' => $this->l('Invalid characters:').' <>;=#{}'
                ),
                array(
                    'type' => 'file',
                    'label' => $this->l('Image:'),
                    'name' => 'image',
                    'display_image' => true,
                    'image' => $image_url ? '<img src="'.$image_url.'" alt="" class="imgm img-thumbnail" 
                    style="max-width: 100%;" />' : false,
                    'size' => $image_size,
                    'delete_url' => self::$currentIndex.'&'.$this->identifier.'='.$dfxcategory->id.'&token='.
                    $this->token.'&deleteImageCategory=1',
                    'desc' => $this->l('Upload a category image from your computer.'),
                ),
                array(
                    'type' => 'text',
                    'label' => $this->l('Meta title:'),
                    'name' => 'meta_title',
                    'lang' => true,
                    //'hint' => $this->l('Forbidden characters:').' <>;=#{}'
                ),
                array(
                    'type' => 'text',
                    'label' => $this->l('Meta description:'),
                    'name' => 'meta_description',
                    'lang' => true,
                    //'hint' => $this->l('Forbidden characters:').' <>;=#{}'
                ),
                array(
                    'type' => 'tags',
                    'label' => $this->l('Meta keywords:'),
                    'name' => 'meta_keywords',
                    'lang' => true,
                    'desc' => $this->l('To add "tags," click in the field, write something, and then press "Enter."'),
                    //'hint' => $this->l('Forbidden characters:').' <>;=#{}'
                ),
                array(
                    'type' => 'text',
                    'label' => $this->l('Friendly URL:'),
                    'name' => 'link_rewrite',
                    'lang' => true,
                    'required' => true,
                    'desc' => $this->l('Only letters and the minus (-) character are allowed.'),
                ),
                array(
                    'type' => 'text',
                    'label' => $this->l('Position:'),
                    'name' => 'position',
                ),
            ),
            'submit' => array(
                'title' => $this->l('Save'),
            ),
        );

        $this->tpl_form_vars['PS_ALLOW_ACCENTED_CHARS_URL'] = (int) Configuration::get('PS_ALLOW_ACCENTED_CHARS_URL');

        return parent::renderForm();
    }

    public function setMedia($isNewTheme = false)
    {
        parent::setMedia($isNewTheme);
        $this->addJqueryUi('ui.widget');
        $this->addJqueryPlugin('tagify');
    }

    public static function getDescriptionClean($description)
    {
        return Tools::getDescriptionClean($description);
    }

    public function initPageHeaderToolbar()
    {
        parent::initPageHeaderToolbar();

        if ($this->display != 'edit' && $this->display != 'add') {
            $this->page_header_toolbar_btn['new_dfxblog_category'] = array(
                'href' => self::$currentIndex.'&adddfxblog_category&token='.$this->token,
                'desc' => $this->l('Add New Category', null, null, false),
                'icon' => 'process-icon-new',
            );
        }
    }

    public function renderList()
    {
        if (isset($this->_filter) && trim($this->_filter) == '') {
            $this->_filter = $this->original_filter;
        }
        return parent::renderList();
    }

    public function getList(
        $id_lang,
        $order_by = null,
        $order_way = null,
        $start = 0,
        $limit = null,
        $id_lang_shop = false
    ) {
        $order_by = 'id_parent';
        $order_way = 'ASC';
        $id_lang_shop = false;
        parent::getList($id_lang, $order_by, $order_way, $start, $limit, $id_lang_shop);
        $this->_list = $this->advanceListCategories(0);
    }

    private function advanceListCategories($id_parent = 0)
    {
        $result = array();
        foreach ($this->_list as $category) {
            if ($category['id_parent'] == $id_parent) {
                $category['name'] = str_repeat('|- - ', $category['level_depth']).$category['name'];
                $result[] = $category;
                $result = array_merge($result, $this->advanceListCategories($category['id_dfxblog_category']));
            }
        }

        return $result;
    }

    private function createCategoryTreeOptions($categorys = array(), $pos = 0)
    {
        $options = array();
        if (isset($categorys[$pos])) {
            foreach ($categorys[$pos] as $category) {
                $options[] = array(
                    'id' => $category['id_dfxblog_category'],
                    'name' => str_repeat('|- - ', $category['level_depth']).$category['name'],
                );
                $options = array_merge($options, $this->createCategoryTreeOptions(
                    $categorys,
                    $category['id_dfxblog_category']
                ));
            }
        }

        return $options;
    }

    protected function categoryImage($id)
    {
        $dir = $this->fieldImageSettings['dir'];
        $folder = Image::getImgFolderStatic($id);
        $ret = $this->uploadImage($id, $this->fieldImageSettings['name'], $dir.$folder);

        // create thumbnail
        if ($ret) {
            $settings = unserialize(Configuration::get('DFX_BLOG_SETTINGS'));
            $category_img_size = isset($settings['category_img']) ?
            $settings['category_img'] : array('width' => 850, 'height' => 250);

            ImageManager::resize(
                _PS_IMG_DIR_.$dir.$folder.$id.'.'.$this->imageType,
                _PS_IMG_DIR_.$dir.$folder.$id.'.'.$this->imageType,
                (int) $category_img_size['width'],
                (int) $category_img_size['height']
            );
        }

        return $ret;
    }

    protected function uploadImage($id, $name, $dir, $ext = false, $width = null, $height = null)
    {
        return parent::uploadImage($id, $name, $dir, $ext, $width, $height);
    }


    
    public function postProcess()
    {
        if (Tools::isSubmit('viewdfxblog_category')
            && ($id_dfxblog_category = (int)Tools::getValue('id_dfxblog_category'))
            && ($dfxCategory = new DfxCategory($id_dfxblog_category, $this->context->language->id))
            && Validate::isLoadedObject($dfxCategory)) {
            $redir = $dfxCategory->getObjectLink($this->context->language->id);
            Tools::redirectAdmin($redir);
        }
        
        if (Tools::isSubmit('deleteImageCategory')) {
            $id = Tools::getValue('id_dfxblog_category');
            $dfxcategory = new DfxCategory($id);
            
            $dfxcategory->deleteImg();

            Tools::redirectAdmin(
                self::$currentIndex.'&id_dfxblog_category='.$id.'&updatedfxblog_category&token='.
                Tools::getAdminTokenLite('AdminDfxCategory').'&conf=7'
            );
        }

        return parent::postProcess();
    }
}
