<?php
/**
 * 2007-2021 PrestaShop.
 *
 * NOTICE OF LICENSE
 *
 * This source file is subject to the Academic Free License (AFL 3.0)
 * that is bundled with this package in the file LICENSE.txt.
 * It is also available through the world-wide-web at this URL:
 * http://opensource.org/licenses/afl-3.0.php
 * If you did not receive a copy of the license and are unable to
 * obtain it through the world-wide-web, please send an email
 * to license@prestashop.com so we can send you a copy immediately.
 *
 * DISCLAIMER
 *
 * Do not edit or add to this file if you wish to upgrade PrestaShop to newer
 * versions in the future. If you wish to customize PrestaShop for your
 * needs please refer to http://www.prestashop.com for more information.
 *
 *  @author    PrestaShop SA <contact@prestashop.com>
 *  @copyright 2007-2021 PrestaShop SA
 *  @license   http://opensource.org/licenses/afl-3.0.php  Academic Free License (AFL 3.0)
 *  International Registered Trademark & Property of PrestaShop SA
 */

require_once _PS_MODULE_DIR_.'dfxblog/loader.php';

class AdminDfxPostController extends ModuleAdminController
{
    protected $post = null;
    public $imageType = 'jpg';
    protected $position_identifier = 'id_dfxblog_post';
    
    public function __construct()
    {
        $this->table = 'dfxblog_post';
        $this->className = 'DfxPost';
        $this->identifier = 'id_dfxblog_post';
        $this->multishop_context = Shop::CONTEXT_ALL;
        $this->lang = true;
        
        $this->addRowAction('edit');
        $this->addRowAction('view');
        $this->addRowAction('delete');
        
        parent::__construct();
        $this->bootstrap = true;
        $this->explicitSelect = true;
        $this->allow_export = true;
        
        $this->context = Context::getContext();

        $this->_defaultOrderBy = 'date_add';
        $this->_defaultOrderWay = 'DESC';
        
        $this->fieldImageSettings = array(
            'name' => 'image',
            'dir' => _DFXPRO_,
        );

        $this->fields_list = array(
            'id_dfxblog_post' => array(
                'title' => $this->l('ID'),
                'align' => 'center',
                'class' => 'fixed-width-xs',
            ),
            'name' => array(
                'title' => $this->l('Name'),
            ),
            'description_short' => array(
                'title' => $this->l('Short Description'),
                'maxlength' => 90,
                'callback' => 'getDescriptionClean',
                'orderby' => false,
            ),
            'views' => array(
                'title' => $this->l('Views'),
                'width' => 30,
                'align' => 'center',
                'search' => false,
            ),
            'likes' => array(
                'title' => $this->l('Likes'),
                'width' => 30,
                'align' => 'center',
                'search' => false,
            ),
            'date_add' => array(
                'title' => $this->l('Date Add'),
                'align' => 'text-right',
                'class' => 'fixed-width-sm',
                'type' => 'datetime',
            ),
            'active' => array(
                'title' => $this->l('Displayed'),
                'active' => 'status',
                'align' => 'center',
                'type' => 'bool',
                'class' => 'fixed-width-xs',
                'orderby' => false,
            ),
        );

        $this->bulk_actions = array('delete' => array('text' => $this->l('Delete selected')));
        
        if (!Tools::getValue('id_dfxblog_post', 0)) {
            $this->informations[] = $this->l('You can view blog here: ').'<a href="'.DfxLink::getBlogLink().'" title="" class="_blank">'.DfxLink::getBlogLink().'</a>';
        }
    }

    public function init()
    {
        parent::init();

        if (($id_dfxblog_post = Tools::getvalue('id_dfxblog_post')) && $this->action != 'select_delete') {
            $this->post = new DfxPost($id_dfxblog_post);
        } else {
            $this->post = new DfxPost();
        }
    }

    public function renderForm()
    {
        $dfxpost = new DfxPost();
        $id_dfxblog_post = (int) Tools::getvalue('id_dfxblog_post');

        if ($id_dfxblog_post) {
            $dfxpost = new DfxPost($id_dfxblog_post);
        }
        
        $this->buildFolder($this->fieldImageSettings['dir'], $dfxpost->id);

        $folder = Image::getImgFolderStatic($dfxpost->id);
        $image = _DFXPRO_IMG_DIR_.$folder.$dfxpost->id.'.'.$this->imageType;
        $image_url = file_exists($image) ? __PS_BASE_URI__.'img/'.$this->fieldImageSettings['dir'].$folder.$dfxpost->id.'.'.$this->imageType : false;
        $image_size = $image_url ? filesize($image) / 1000 : false;

        $id_lang = (int) Configuration::get('PS_LANG_DEFAULT');
        $available_categories = DfxCategory::getCategories($id_lang, false, true);

        foreach ($available_categories as &$category) {
            if ($category['is_child']) {
                $category['name'] = '&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;'.$category['name'];
            }
        }
        
        $this->fields_form = array(
            'tinymce' => true,
            'legend' => array(
                'title' => $this->l('Blog Post'),
                'icon' => 'icon-bookmark',
            ),
            'input' => array(
                array(
                    'type' => 'hidden',
                    'name' => 'post_type',
                ),
                array(
                        'type' => 'hidden',
                        'name' => 'id_dfxblog_post',
                    ),
                array(
                    'type' => 'text',
                    'label' => $this->l('Name:'),
                    'name' => 'name',
                    'lang' => true,
                    'size' => 48,
                    'required' => true,
                    'class' => 'copy2friendlyUrl',
                ),
                array(
                    'type' => 'text',
                    'label' => $this->l('Friendly URL:'),
                    'name' => 'link_rewrite',
                    'lang' => true,
                    'required' => true,
                    'desc' => $this->l('Only letters and the minus (-) character are allowed.'),
                ),
                array(
                    'type' => 'select',
                    'label' => $this->l('Category:'),
                    'name' => 'id_dfxblog_category',
                    'required' => true,
                    'options' => array(
                        'id' => 'id',
                        'query' => $available_categories,
                        'name' => 'name'
                        )
                ),
                array(
                    'type' => 'switch',
                    'label' => $this->l('Displayed:'),
                    'name' => 'active',
                    'required' => false,
                    'class' => 't',
                    'is_bool' => true,
                    'values' => array(
                        array(
                            'id' => 'active_on',
                            'value' => 1,
                            'label' => $this->l('Enabled'),
                        ),
                        array(
                            'id' => 'active_off',
                            'value' => 0,
                            'label' => $this->l('Disabled'),
                        ),
                    ),
                ),
                array(
                    'type' => 'textarea',
                    'label' => $this->l('Short Description:'),
                    'name' => 'description_short',
                    'autoload_rte' => true,
                    'lang' => true,
                    'rows' => 10,
                    'cols' => 100,
                ),
                array(
                    'type' => 'textarea',
                    'label' => $this->l('Description:'),
                    'name' => 'description',
                    'autoload_rte' => true,
                    'lang' => true,
                    'rows' => 10,
                    'cols' => 100,
                ),
                array(
                    'type' => 'file',
                    'label' => $this->l('Image:'),
                    'name' => 'image',
                    'display_image' => true,
                    'image' => $image_url ? '<img src="'.$image_url.'" alt="" class="imgm img-thumbnail" 
                    style="max-width: 100%;" />' : false,
                    'size' => $image_size,
                    'delete_url' => self::$currentIndex.'&'.$this->identifier.'='.$this->post->id.'&token='.
                    $this->token.'&deleteImagePost=1',
                    'desc' => $this->l('Upload a post image from your computer.'),
                ),
                array(
                    'type' => 'text',
                    'label' => $this->l('Posted By:'),
                    'name' => 'posted_by',
                ),
                array(
                    'type' => 'switch',
                    'label' => $this->l('Allow comment:'),
                    'name' => 'allow_comment',
                    'required' => false,
                    'class' => 't',
                    'is_bool' => true,
                    'values' => array(
                        array(
                            'id' => 'comment_on',
                            'value' => 1,
                            'label' => $this->l('Enabled'),
                        ),
                        array(
                            'id' => 'comment_off',
                            'value' => 0,
                            'label' => $this->l('Disabled'),
                        ),
                    ),
                ),
                array(
                    'type' => 'switch',
                    'label' => $this->l('Allow Views:'),
                    'name' => 'allow_views',
                    'required' => false,
                    'class' => 't',
                    'is_bool' => true,
                    'values' => array(
                        array(
                            'id' => 'views_on',
                            'value' => 1,
                            'label' => $this->l('Enabled'),
                        ),
                        array(
                            'id' => 'views_off',
                            'value' => 0,
                            'label' => $this->l('Disabled'),
                        ),
                    ),
                ),
                array(
                    'type' => 'switch',
                    'label' => $this->l('Allow Likes:'),
                    'name' => 'allow_likes',
                    'required' => false,
                    'class' => 't',
                    'is_bool' => true,
                    'values' => array(
                        array(
                            'id' => 'likes_on',
                            'value' => 1,
                            'label' => $this->l('Enabled'),
                        ),
                        array(
                            'id' => 'likes_off',
                            'value' => 0,
                            'label' => $this->l('Disabled'),
                        ),
                    ),
                ),
                array(
                    'type' => 'tags',
                    'label' => $this->l('Tags:'),
                    'name' => 'tags',
                    'lang' => true,
                    'desc' => $this->l('To add "tags," click in the field, write something, and then press "Enter."'),
                ),
                array(
                    'type' => 'related_post',
                    'label' => $this->l('Related Posts:'),
                    'name' => 'related_posts',
                    'ajax_path' => $this->module->getPath().'ajax_posts_list.php',
                    'desc' => $this->l('Begin typing the First Letters of the Post Name, then select the Post 
                    from the Drop-down List.'),
                ),
                array(
                    'type' => 'product_autocomplete',
                    'label' => $this->l('Related Products:'),
                    'name' => 'related_products',
                    'ajax_path' => $this->module->getPath().'ajax_products_list.php',
                    'desc' => $this->l('Begin typing the First Letters of the Product Name, then select the 
                    Product from the Drop-down List.'),
                ),
                array(
                    'type' => 'text',
                    'label' => $this->l('Meta title:'),
                    'name' => 'meta_title',
                    'lang' => true,
                ),
                array(
                    'type' => 'text',
                    'label' => $this->l('Meta description:'),
                    'name' => 'meta_description',
                    'lang' => true,
                ),
                array(
                    'type' => 'text',
                    'label' => $this->l('Meta keywords:'),
                    'name' => 'meta_keywords',
                    'lang' => true,
                    'desc' => $this->l('Keywords for HTML header, separated by commas.'),
                ),
                array(
                    'type' => 'datetime',
                    'label' => $this->l('Date Add:'),
                    'name' => 'date_add',
                ),
            ),
            'submit' => array(
                'title' => $this->l('Save'),
            ),
        );

        $this->tpl_form_vars['PS_ALLOW_ACCENTED_CHARS_URL'] = (int) Configuration::get(
            'PS_ALLOW_ACCENTED_CHARS_URL'
        );

        $this->fields_value = array(
            'id_dfxblog_category' => $dfxpost->id_dfxblog_category,
            'id_dfxblog_post' => $id_dfxblog_post,
            'related_posts' => $dfxpost->getRelatedPostsLight(),
            'related_products' => $dfxpost->getProductsAutocompleteInfo($this->context->language->id),

        );

        if (!(int) $dfxpost->id_dfxblog_post) {
            $this->fields_value['posted_by'] = $this->context->employee->firstname.' '.$this->context->employee->lastname;
        }

        return parent::renderForm();
    }

    public function setMedia($isNewTheme = false)
    {
        parent::setMedia($isNewTheme);
        $this->addJqueryUi('ui.widget');
        $this->addJqueryPlugin('tagify');
        $this->addJqueryPlugin('autocomplete');
    }

    public static function getDescriptionClean($description)
    {
        return Tools::getDescriptionClean($description);
    }

    public function initPageHeaderToolbar()
    {
        parent::initPageHeaderToolbar();

        if ($this->display != 'edit' && $this->display != 'add') {
            $this->page_header_toolbar_btn['new_dfxblog_post'] = array(
                'href' => self::$currentIndex.'&adddfxblog_post&token='.$this->token,
                'desc' => $this->l('Add New Post', null, null, false),
                'icon' => 'process-icon-new',
            );
        }
    }

    public function renderList()
    {
        if (isset($this->_filter) && trim($this->_filter) == '') {
            $this->_filter = $this->original_filter;
        }
        return parent::renderList();
    }

    protected function postImage($id)
    {
        $dir = $this->fieldImageSettings['dir'];
        $this->createImgFolder($dir);
        $this->buildFolder($dir, $id);
        $folder = Image::getImgFolderStatic($id);
        $ret = $this->uploadImage($id, $this->fieldImageSettings['name'], $dir.$folder);

        // create thumbnails
        if ($ret) {
            $settings = unserialize(Configuration::get('DFX_BLOG_SETTINGS'));

            $post_img_size = array(
                'list' => isset($settings['post_img_list']) ?
                $settings['post_img_list'] : array('width' => 240, 'height' => 160),
                'large' => isset($settings['post_img_large']) ?
                $settings['post_img_large'] : array('width' => 750, 'height' => 500),
                'small' => isset($settings['post_img_small']) ?
                $settings['post_img_small'] : array('width' => 60, 'height' => 40),
            );

            foreach ($post_img_size as $type => $size) {
                ImageManager::resize(
                    _PS_IMG_DIR_.$dir.$folder.$id.'.'.$this->imageType,
                    _PS_IMG_DIR_.$dir.$folder.$id.'-'.$type.'.'.$this->imageType,
                    (int) $size['width'],
                    (int) $size['height']
                );
            }
        }

        return $ret;
    }

    protected function uploadImage($id, $name, $dir, $ext = false, $width = null, $height = null)
    {
        $this->createImgFolder($dir);
        
        return parent::uploadImage($id, $name, $dir, $ext, $width, $height);
    }
    public static function buildFolder($dir, $id)
    {
        if (!is_dir(_PS_IMG_DIR_.$dir.'/'.$id)) {
            // validate module
            mkdir(_PS_IMG_DIR_.$dir.'/'.$id, 755, true);
        }
    }

    public function createImgFolder($dir)
    {
        if (!file_exists(_PS_IMG_DIR_.$dir)) {
            $success = @mkdir(_PS_IMG_DIR_.$dir, 0775, true);
            $chmod = @chmod(_PS_IMG_DIR_.$dir, 0775);

            // Create an index.php file in the new folder
            if (($success || $chmod) &&
                !file_exists(_PS_IMG_DIR_.$dir.'index.php') &&
                file_exists(_PS_IMG_DIR_.'index.php')
            ) {
                return @copy(_PS_IMG_DIR_.'index.php', _PS_IMG_DIR_.$dir.'index.php');
            }
        }

        return true;
    }

    public function postProcess()
    {
        if (Tools::isSubmit('deleteImagePost')) {
            $id = Tools::getValue('id_dfxblog_post');
            $dfxpost = new DfxPost($id);

            $dfxpost->deleteImg();

            Tools::redirectAdmin(
                self::$currentIndex.'&id_dfxblog_post='.$id.
                '&updatedfxblog_post&token='.Tools::getAdminTokenLite('AdminDfxPost').'&conf=7'
            );
        }
        
        if (Tools::isSubmit('viewdfxblog_post')
            && ($id_dfxblog_post = (int)Tools::getValue('id_dfxblog_post'))
            && ($DfxPost = new DfxPost($id_dfxblog_post, $this->context->language->id))
            && Validate::isLoadedObject($DfxPost)) {
            $redir = $DfxPost->getObjectLink($this->context->language->id);
            Tools::redirectAdmin($redir);
        }

        return parent::postProcess();
    }
        
    public function processSave()
    {
        if ($this->id_object) {
            $this->object = $this->loadObject(true);
            $object = $this->processUpdate();
        } else {
            $object = $this->processAdd();
        }
        
        $dfxpost = new DfxPost();
        $id_dfxblog_post = (int) Tools::getvalue('id_dfxblog_post');

        if ($id_dfxblog_post) {
            $dfxpost = new DfxPost($id_dfxblog_post);
        }

        $dfxpost->id_dfxblog_category = null;
        if (Tools::getValue('id_dfxblog_category')) {
            $dfxpost->id_dfxblog_category = Tools::getValue('id_dfxblog_category');
        }
        
        // save related products
        $dfxpost->related_products = null;
        if (Tools::getValue('related_products')) {
            $dfxpost->related_products = Tools::getValue('related_products');
        }

        // save related post
        $dfxpost->related_posts  = null;
        if (Tools::getValue('related_posts')) {
            $dfxpost->related_posts = Tools::getValue('related_posts');
        }
        

        $object = $dfxpost->save();
          
        
        return $object;
    }
}
