<?php
/**
 * 2007-2021 PrestaShop.
 *
 * NOTICE OF LICENSE
 *
 * This source file is subject to the Academic Free License (AFL 3.0)
 * that is bundled with this package in the file LICENSE.txt.
 * It is also available through the world-wide-web at this URL:
 * http://opensource.org/licenses/afl-3.0.php
 * If you did not receive a copy of the license and are unable to
 * obtain it through the world-wide-web, please send an email
 * to license@prestashop.com so we can send you a copy immediately.
 *
 * DISCLAIMER
 *
 * Do not edit or add to this file if you wish to upgrade PrestaShop to newer
 * versions in the future. If you wish to customize PrestaShop for your
 * needs please refer to http://www.prestashop.com for more information.
 *
 *  @author    PrestaShop SA <contact@prestashop.com>
 *  @copyright 2007-2021 PrestaShop SA
 *  @license   http://opensource.org/licenses/afl-3.0.php  Academic Free License (AFL 3.0)
 *  International Registered Trademark & Property of PrestaShop SA
 */

require_once _PS_MODULE_DIR_.'dfxblog/loader.php';

class AdminDfxSettingController extends ModuleAdminController
{
    private $settings;
    protected $start_time = 0;
    protected $max_execution_time = 7200;

    public function __construct()
    {
        $this->bootstrap = true;
        $this->className = 'Configuration';
        $this->multishop_context = Shop::CONTEXT_ALL;

        parent::__construct();

        $this->settings = unserialize(Configuration::get('DFX_BLOG_SETTINGS'));
        foreach ($this->module->blog_config as $key => $value) {
            if (!isset($this->settings[$key])) {
                $this->settings[$key] = $value;
            }
        }
    }

    public function initFieldSidebar()
    {
        $this->fields_form[1]['form'] = array(
            'legend' => array(
                'title' => $this->l('Sidebar Blocks'),
                'icon' => 'icon-book',
            ),
            'input' => array(
                array(
                    'type' => 'switch',
                    'label' => $this->l('Blog Categories:'),
                    'name' => 'module_category',
                    'class' => 't',
                    'is_bool' => true,
                    'values' => array(
                        array(
                            'id' => 'category_on',
                            'value' => 1,
                            'label' => $this->l('Enable'),
                        ),
                        array(
                            'id' => 'category_off',
                            'value' => 0,
                            'label' => $this->l('Disable'),
                        ),
                    ),
                ),
                array(
                    'type' => 'switch',
                    'label' => $this->l('Blog Archives:'),
                    'name' => 'module_archive',
                    'class' => 't',
                    'is_bool' => true,
                    'values' => array(
                        array(
                            'id' => 'archive_on',
                            'value' => 1,
                            'label' => $this->l('Enable'),
                        ),
                        array(
                            'id' => 'archive_off',
                            'value' => 0,
                            'label' => $this->l('Disable'),
                        ),
                    ),
                ),
                array(
                    'type' => 'switch',
                    'label' => $this->l('Blog Latest Posts:'),
                    'name' => 'module_latestpost',
                    'class' => 't',
                    'is_bool' => true,
                    'values' => array(
                        array(
                            'id' => 'latestpost_on',
                            'value' => 1,
                            'label' => $this->l('Enable'),
                        ),
                        array(
                            'id' => 'latestpost_off',
                            'value' => 0,
                            'label' => $this->l('Disable'),
                        ),
                    ),
                ),
                array(
                    'type' => 'switch',
                    'label' => $this->l('Blog Current Comments:'),
                    'name' => 'module_currentcomment',
                    'class' => 't',
                    'is_bool' => true,
                    'values' => array(
                        array(
                            'id' => 'currentcomment_on',
                            'value' => 1,
                            'label' => $this->l('Enable'),
                        ),
                        array(
                            'id' => 'currentcomment_off',
                            'value' => 0,
                            'label' => $this->l('Disable'),
                        ),
                    ),
                ),
            ),
            'submit' => array(
                'title' => $this->l('Save'),
            ),
        );

        $this->fields_value['module_category'] = $this->settings['module_category'];
        $this->fields_value['module_archive'] = $this->settings['module_archive'];
        $this->fields_value['module_latestpost'] = $this->settings['module_latestpost'];
        $this->fields_value['module_currentcomment'] = $this->settings['module_currentcomment'];
    }

    public function initFieldPost()
    {
        $this->fields_form[2]['form'] = array(
            'legend' => array(
                'title' => $this->l('Post'),
                'icon' => 'icon-bookmark',
            ),
            'input' => array(
            
                array(
                    'type' => 'text',
                    'label' => $this->l('Post per page:'),
                    'name' => 'post_per_page',
                    'desc' => $this->l('Number of post displayed per page. Default is 10.'),
                ),
                array(
                    'type' => 'switch',
                    'label' => $this->l('Allow comment:'),
                    'name' => 'allow_comment',
                    'class' => 't',
                    'is_bool' => true,
                    'values' => array(
                        array(
                            'id' => 'comment_on',
                            'value' => 1,
                            'label' => $this->l('Yes'),
                        ),
                        array(
                            'id' => 'comment_off',
                            'value' => 0,
                            'label' => $this->l('No'),
                        ),
                    ),
                    'desc' => $this->l('Allows reader comments.'),
                ),
                array(
                    'type' => 'switch',
                    'label' => $this->l('Allow Views:'),
                    'name' => 'allow_views',
                    'required' => false,
                    'class' => 't',
                    'is_bool' => true,
                    'values' => array(
                        array(
                            'id' => 'views_on',
                            'value' => 1,
                            'label' => $this->l('Yes'),
                        ),
                        array(
                            'id' => 'views_off',
                            'value' => 0,
                            'label' => $this->l('No'),
                        ),
                    ),
                ),
                array(
                    'type' => 'switch',
                    'label' => $this->l('Allow Likes:'),
                    'name' => 'allow_likes',
                    'required' => false,
                    'class' => 't',
                    'is_bool' => true,
                    'values' => array(
                        array(
                            'id' => 'likes_on',
                            'value' => 1,
                            'label' => $this->l('Yes'),
                        ),
                        array(
                            'id' => 'likes_off',
                            'value' => 0,
                            'label' => $this->l('No'),
                        ),
                    ),
                ),
                array(
                    'type' => 'switch',
                    'label' => $this->l('Comment validate:'),
                    'name' => 'comment_validate',
                    'class' => 't',
                    'is_bool' => true,
                    'values' => array(
                        array(
                            'id' => 'validate_on',
                            'value' => 1,
                            'label' => $this->l('Yes'),
                        ),
                        array(
                            'id' => 'validate_off',
                            'value' => 0,
                            'label' => $this->l('No'),
                        ),
                    ),
                    'desc' => $this->l('All comments must be validated.'),
                ),
                array(
                    'type' => 'switch',
                    'label' => $this->l('Related Products:'),
                    'name' => 'enable_relatedproduct',
                    'class' => 't',
                    'is_bool' => true,
                    'values' => array(
                        array(
                            'id' => 'related_on',
                            'value' => 1,
                            'label' => $this->l('Yes'),
                        ),
                        array(
                            'id' => 'related_off',
                            'value' => 0,
                            'label' => $this->l('No'),
                        ),
                    ),
                ),
                array(
                    'type' => 'text',
                    'label' => $this->l('Time comment:'),
                    'name' => 'time_comment',
                    'desc' => $this->l('Minimum time between 2 comments from the same user. Default is 60 seconds.'),
                ),
            ),
            'submit' => array(
                'title' => $this->l('Save'),
            ),
        );


        $this->fields_value['post_per_page'] = $this->settings['post_per_page'];
        $this->fields_value['allow_comment'] = $this->settings['allow_comment'];
        $this->fields_value['allow_views'] = $this->settings['allow_views'];
        $this->fields_value['allow_likes'] = $this->settings['allow_likes'];
        $this->fields_value['comment_validate'] = $this->settings['comment_validate'];
        $this->fields_value['time_comment'] = $this->settings['time_comment'];
		$this->fields_value['enable_relatedproduct'] = $this->settings['enable_relatedproduct'];
    }

    public function initFieldImageSize()
    {
        $this->fields_form[3]['form'] = array(
            'legend' => array(
                'title' => $this->l('Blog Image Size'),
                'icon' => 'icon-picture',
            ),
            'input' => array(
                array(
                    'type' => 'image_size',
                    'label' => $this->l('Post Image Small Size'),
                    'name' => 'post_img_small',
                    'size' => 4,
                ),
                array(
                    'type' => 'image_size',
                    'label' => $this->l('Post Image List Size'),
                    'name' => 'post_img_list',
                    'size' => 4,
                ),
                array(
                    'type' => 'image_size',
                    'label' => $this->l('Post Image Large Size'),
                    'name' => 'post_img_large',
                    'size' => 4,
                ),
                array(
                    'type' => 'image_size',
                    'label' => $this->l('Blog Category Image Size'),
                    'name' => 'category_img',
                    'size' => 4,
                ),
            ),
            'submit' => array(
                'title' => $this->l('Save'),
            ),
        );

        $this->fields_form[4]['form'] = array(
            'legend' => array(
                'title' => $this->l('Regenerate thumbnails'),
                'icon' => 'icon-picture',
            ),
            'description' => $this->l('Regenerates thumbnails for all existing images').'<br />'.$this->l('Please 
            be patient. This can take several minutes.').'<br />'.$this->l('Be careful! Manually uploaded 
            thumbnails will be erased and replaced by automatically generated thumbnails.'),
            'input' => array(
                array(
                    'type' => 'switch',
                    'label' => $this->l('Post Images:'),
                    'name' => 'postImage',
                    'class' => 't',
                    'is_bool' => true,
                    'values' => array(
                        array(
                            'id' => 'postImage_on',
                            'value' => 1,
                            'label' => $this->l('Yes'),
                        ),
                        array(
                            'id' => 'postImage_off',
                            'value' => 0,
                            'label' => $this->l('No'),
                        ),
                    ),
                    'desc' => $this->l('Regenerate All of Post Image Size.'),
                ),
                array(
                    'type' => 'switch',
                    'label' => $this->l('Category Images:'),
                    'name' => 'categoryImage',
                    'class' => 't',
                    'is_bool' => true,
                    'values' => array(
                        array(
                            'id' => 'categoryImage_on',
                            'value' => 1,
                            'label' => $this->l('Yes'),
                        ),
                        array(
                            'id' => 'categoryImage_off',
                            'value' => 0,
                            'label' => $this->l('No'),
                        ),
                    ),
                    'desc' => $this->l('Regenerate All of Category Images.'),
                ),
                array(
                    'type' => 'regenerate_image',
                    'label' => '',
                    'name' => 'submitRegenerateimage',
                ),
            ),
        );

        $this->fields_value['post_img_list'] = $this->settings['post_img_list'];
        $this->fields_value['post_img_large'] = $this->settings['post_img_large'];
        $this->fields_value['post_img_small'] = $this->settings['post_img_small'];
        $this->fields_value['category_img'] = $this->settings['category_img'];
        $this->fields_value['postImage'] = 1;
        $this->fields_value['categoryImage'] = 1;
    }

    public function renderForm()
    {
        $this->initFieldSidebar();
        $this->initFieldPost();
        $this->initFieldImageSize();

        $this->fields_form = array_values($this->fields_form);

        $this->multiple_fieldsets = true;

        return parent::renderForm();
    }

    public function initContent()
    {
        $this->initToolbar();
        $this->initPageHeaderToolbar();
        $this->display = '';
        $this->content .= $this->renderForm();

        $this->context->smarty->assign(array(
            'content' => $this->content,
            'url_post' => self::$currentIndex.'&token='.$this->token,
            'show_page_header_toolbar' => $this->show_page_header_toolbar,
            'page_header_toolbar_title' => $this->page_header_toolbar_title,
            'page_header_toolbar_btn' => $this->page_header_toolbar_btn,
            'title' => $this->page_header_toolbar_title,
            'toolbar_btn' => $this->page_header_toolbar_btn,
        ));
    }

    public function initToolbar()
    {
        $this->toolbar_btn['save'] = array(
            'href' => '#',
            'desc' => $this->l('Save'),
        );
    }

    public function postProcess()
    {
        if (Tools::isSubmit('submitRegenerateimage')) {
            if ($this->regenerateNewImages(Tools::getValue('categoryImage'), Tools::getValue('postImage'))) {
                Tools::redirectAdmin(self::$currentIndex.'&token='.Tools::getValue('token').'&conf=4');
            }
        }

        if (Tools::isSubmit('submitAddconfiguration')) {
            $config = array();

           
            
            $config['layout'] = Tools::getValue('layout');
            $config['module_category'] = Tools::getValue('module_category');
            $config['module_archive'] = Tools::getValue('module_archive');
            $config['module_latestpost'] = Tools::getValue('module_latestpost');
            $config['module_currentcomment'] = Tools::getValue('module_currentcomment');
            $config['enable_relatedproduct'] = Tools::getValue('enable_relatedproduct');
            $config['post_per_page'] = Tools::getValue('post_per_page');
            $config['allow_comment'] = Tools::getValue('allow_comment');
            $config['allow_views'] = Tools::getValue('allow_views');
            $config['allow_likes'] = Tools::getValue('allow_likes');
            $config['comment_validate'] = Tools::getValue('comment_validate');
            $config['time_comment'] = Tools::getValue('time_comment');
            $config['post_img_small'] = Tools::getValue('post_img_small');
            $config['post_img_list'] = Tools::getValue('post_img_list');
            $config['post_img_large'] = Tools::getValue('post_img_large');
            $config['category_img'] = Tools::getValue('category_img');
            $config['postImage'] = Tools::getValue('postImage');
            $config['categoryImage'] = Tools::getValue('categoryImage');
            
            Configuration::updateValue('DFX_BLOG_SETTINGS', serialize($config));
            Hook::exec('actionDfxBlogUpdate');

            Tools::redirectAdmin(self::$currentIndex.'&token='.Tools::getValue('token').'&conf=4');
        }
        return parent::postProcess();
    }

    protected function regenerateNewImages($categoryImage = true, $postImage = true)
    {
        $this->start_time = time();
        ini_set('max_execution_time', $this->max_execution_time);
        $this->max_execution_time = (int) ini_get('max_execution_time');

        // regenerate category image
        $c_return = false;
        if ($categoryImage) {
            $c_return = $this->regenerateCategoryImages();
            if ($c_return === true) {
                if (!count($this->errors)) {
                    $this->errors[] = sprintf(Tools::displayError('Cannot write blog category images. Please 
                    check the folder\'s writing permissions '._DFXCAT_IMG_DIR_));
                }
            }
        }

        // regenerate post images
        $p_return = false;
        if ($postImage) {
            $p_return = $this->regeneratePostImages();
            if ($p_return === true) {
                if (!count($this->errors)) {
                    $this->errors[] = sprintf(Tools::displayError('Cannot write blog post images. 
                    Please check the folder\'s writing permissions '._DFXPRO_IMG_DIR_));
                }
            }
        }

        if ($c_return == 'timeout' || $p_return == 'timeout') {
            $this->errors[] = Tools::displayError('Only part of the images have been regenerated. The server 
            timed out before finishing.');
        }

        return (count($this->errors) > 0 ? false : true);
    }

    protected function regenerateCategoryImages()
    {
        $dir = _DFXCAT_IMG_DIR_;
        $name = 'category';
        $width = $this->settings['category_img']['width'];
        $height = $this->settings['category_img']['height'];

        if (!is_dir($dir)) {
            return false;
        }

        $errors = false;

        foreach (scandir($dir) as $image) {
            if (preg_match('/^[0-9]*\.jpg$/', $image)) {
                if (!file_exists($dir.$image) || !filesize($dir.$image)) {
                    $errors = true;
                    $this->errors[] = sprintf(
                        Tools::displayError('Source file does not exist or is empty (%s)'),
                        $dir.$image
                    );
                } elseif (!ImageManager::resize(
                    $dir.$image,
                    $dir.Tools::substr($image, 0, -4).'-'.Tools::stripslashes($name).'.jpg',
                    (int) $width,
                    (int) $height
                )) {
                    $errors = true;
                    $this->errors[] = sprintf(
                        Tools::displayError(
                            'Failed to resize image file (%s)'
                        ),
                        $dir.$image
                    );
                }
            }

            if (time() - $this->start_time > $this->max_execution_time - 4) {
                return 'timeout';
            }
        }

        return $errors;
    }

    protected function regeneratePostImages()
    {
        $dir = _DFXPRO_IMG_DIR_;
        $type = array(
            array(
                'name' => 'large',
                'width' => $this->settings['post_img_large']['width'],
                'height' => $this->settings['post_img_large']['height'],
            ),
            array(
                'name' => 'list',
                'width' => $this->settings['post_img_list']['width'],
                'height' => $this->settings['post_img_list']['height'],
            ),
            array(
                'name' => 'small',
                'width' => $this->settings['post_img_small']['width'],
                'height' => $this->settings['post_img_small']['height'],
            ),
        );

        if (!is_dir($dir)) {
            return false;
        }

        $errors = false;

        $sql = 'SELECT p.`id_dfxblog_post` FROM `'._DB_PREFIX_.'dfxblog_post` p';

        $result = Db::getInstance(_PS_USE_SQL_SLAVE_)->executeS($sql);

        $temppost = new DfxPost();

        foreach ($result as $row) {
			$folder = Image::getImgFolderStatic($row['id_dfxblog_post']);
            $pdir = $dir.$folder;
            $image = $row['id_dfxblog_post'].'.jpg';

            if (file_exists($pdir.$image) && filesize($pdir.$image)) {
                foreach ($type as $imageType) {
                    if (!ImageManager::resize(
                        $pdir.$image,
                        $pdir.Tools::substr($image, 0, -4).'-'.Tools::stripslashes($imageType['name']).'.jpg',
                        (int) $imageType['width'],
                        (int) $imageType['height']
                    )) {
                        $errors = true;
                        $this->errors[] = sprintf(
                            Tools::displayError(
                                'Failed to resize image file (%s)'
                            ),
                            $pdir.$image
                        );
                    }
                }
            }

            if (time() - $this->start_time > $this->max_execution_time - 4) {
                return 'timeout';
            }
        }

        return $errors;
    }
}
