<?php
/**
 * 2007-2021 PrestaShop.
 *
 * NOTICE OF LICENSE
 *
 * This source file is subject to the Academic Free License (AFL 3.0)
 * that is bundled with this package in the file LICENSE.txt.
 * It is also available through the world-wide-web at this URL:
 * http://opensource.org/licenses/afl-3.0.php
 * If you did not receive a copy of the license and are unable to
 * obtain it through the world-wide-web, please send an email
 * to license@prestashop.com so we can send you a copy immediately.
 *
 * DISCLAIMER
 *
 * Do not edit or add to this file if you wish to upgrade PrestaShop to newer
 * versions in the future. If you wish to customize PrestaShop for your
 * needs please refer to http://www.prestashop.com for more information.
 *
 *  @author    PrestaShop SA <contact@prestashop.com>
 *  @copyright 2007-2021 PrestaShop SA
 *  @license   http://opensource.org/licenses/afl-3.0.php  Academic Free License (AFL 3.0)
 *  International Registered Trademark & Property of PrestaShop SA
 */

use PrestaShop\PrestaShop\Adapter\Image\ImageRetriever;
use PrestaShop\PrestaShop\Adapter\Product\PriceFormatter;
use PrestaShop\PrestaShop\Core\Product\ProductListingPresenter;
use PrestaShop\PrestaShop\Adapter\Product\ProductColorsRetriever;
use PrestaShop\PrestaShop\Adapter\Category\CategoryProductSearchProvider;
use PrestaShop\PrestaShop\Core\Product\Search\ProductSearchContext;
use PrestaShop\PrestaShop\Core\Product\Search\ProductSearchQuery;
use PrestaShop\PrestaShop\Core\Product\Search\SortOrder;
use PrestaShop\PrestaShop\Adapter\BestSales\BestSalesProductSearchProvider;
use PrestaShop\PrestaShop\Adapter\NewProducts\NewProductsProductSearchProvider;

require_once _PS_MODULE_DIR_.'dfxblog/loader.php';

class DfxBlogPostModuleFrontController extends ModuleFrontController
{
    public $php_self;
    protected $post;
    protected $template_path = '';
    protected $dfxlink;
    public $dfxblog_post_name;
    public $dfxblog_post_url;
    public $dfxblog_post_thumbnail;
    public $link_rewrite;
    
    public function __construct()
    {
        parent::__construct();
        $this->context = Context::getContext();
        $this->template_path = _PS_MODULE_DIR_.'dfxblog/views/templates/front/';
        // $this->rewrite = 'aaaa';
    }
    
    public function init()
    {
        parent::init();

        // Get post
        $id_lang = $this->context->language->id;
        $id_post = (int) Tools::getValue('id_dfxblog_post');
        
        $this->post = new DfxPost($id_post, $this->context->language->id);
        $link_rewrite = $this->post->link_rewrite;
        $this->dfxlink = new DfxLink();
        if (!$this->post->id || !$this->post->active) {
            $this->errors[] = Tools::displayError('Missing Post ID');
            $headers = array('HTTP/1.1 404 Not Found', 'Status: 404 Not Found');
            Tools::redirect('404', __PS_BASE_URI__, null, $headers);
        }

        $meta_title = (isset($this->post->meta_title) && $this->post->meta_title != '') ?
        $this->post->meta_title : $this->post->name;
        $meta_description = isset($this->post->meta_description) ? $this->post->meta_description : '';
        $meta_keywords = isset($this->post->meta_keywords) ? $this->post->meta_keywords : '';
        
        $this->dfxblog_post_name = $this->post->name;
        $this->dfxblog_post_url = DfxPost::getLink($this->post->id, $this->post->link_rewrite, $id_lang);
        $this->dfxblog_post_thumbnail = DfxLink::getPostImageLink($this->post->id, 'small');
        
        $this->context->smarty->assign(array(
            'meta_title' => $meta_title,
            'meta_description' => $meta_description,
            'meta_keywords' => $meta_keywords,
        ));
    }
    
    public function getLayout()
    {
        $entity = 'module-dfxblog-'.$this->php_self;
        
        $layout = $this->context->shop->theme->getLayoutRelativePathForPage($entity);
        
        if ($overridden_layout = Hook::exec(
            'overrideLayoutTemplate',
            array(
                'default_layout' => $layout,
                'entity' => $entity,
                'locale' => $this->context->language->locale,
                'controller' => $this,
            )
        )
        ) {
            return $overridden_layout;
        }

        if ((int) Tools::getValue('content_only')) {
            $layout = 'layouts/layout-content-only.tpl';
        }

        return $layout;
    }
    
    protected function comment()
    {
        $post = array();
        $post['customer_name'] = Tools::getValue('customer_name');
        $post['customer_email'] = Tools::getValue('customer_email');
        $post['comment'] = Tools::getValue('comment');
        $post['id_dfxblog_post'] = Tools::getValue('id_dfxblog_post');
         
        // check time comment
        $customer_last_comment = DfxComment::getLastCommentByCustomer(
            Tools::getValue('id_dfxblog_post'),
            $this->context->customer->id
        );
        $time_comment = isset($this->settings['time_comment']) ? (int) $this->settings['time_comment'] : 60;

        if ((strtotime($customer_last_comment) + $time_comment) - time() > 0) {
            $post = array($this->l('You should wait').' '.$this->settings['time_comment'].' '.
            $this->l('seconds before posting a new comment'));
        }
        
        if (!empty($post)) {
            $comment = new DfxComment();

            $error = new stdClass();
            $error->error = true;
            $this->copyFromPost($comment, 'comment', $post);
           
            if ($comment->validateFields(false) && $comment->validateFieldsLang(false)) {
                $comment->save();
                $error->message = $this->l('Thanks for your comment, it will be published soon!');
                $error->error = false;
            } else {
                // validate module
                $error->message = $this->l(
                    'An error occurred while sending the comment. Please recorrect data in fields!',
                    'blog'
                );
            }
            

            die(Tools::jsonEncode($error));
        }
    }
    
    
    public function initContent()
    {
        $this->context->controller->addJqueryPlugin('cooki-plugin');
        $this->context->controller->addJqueryPlugin('cookie-plugin');
        
        $this->php_self = 'post';
        
        parent::initContent();
        $id_lang = $this->context->language->id;
        
        if (Tools::isSubmit('submitcomment')) {
            // validate module
            $this->comment();
        }
        
        $this->post->increaseViewsNb();
        // breadcrumb
        $path = $this->post->name;
        // post
        $this->settings = unserialize(Configuration::get('DFX_BLOG_SETTINGS'));
        $post_categories = DfxPost::getInCategories($this->post->id_dfxblog_post, $id_lang);

        // related posts
        $related_posts = $this->post->getRelatedPosts($id_lang);

        // related products
        $rlt_product = $this->post->related_products;
        $related_products = $this->getRelatedProducts($rlt_product, $id_lang);

        // list comment
        $comment_validate = isset($this->settings['comment_validate']) ? $this->settings['comment_validate'] : 1;
        $comments = $this->post->getComments($comment_validate);
        
        $dfxlink = new dfxLink();
        
    
        // list same categories
        
        $category = new DfxCategory($this->post->id_dfxblog_category, $this->context->language->id);
        $tags = array();
        if ($this->post->tags && $tmp = explode(',', $this->post->tags)) {
            foreach ($tmp as $tag) {
                $tags[] = array(
                    'tag' => $tag,
                    'link' => $dfxlink->getBlogTagLink($tag)
                );
            }
        }

        $limit = 5;
        $samecats = DfxPost::getListBlogs(
            $category->id_dfxblog_category,
            $this->context->language->id,
            0,
            $limit,
            'date_add',
            'DESC',
            array('type' => 'samecat', 'id_dfxblog_post' => $this->post->id_dfxblog_post),
            true
        );
        
        
        foreach ($samecats as $key => $cblog) {
            $cblog['link'] = DfxPost::getLink($cblog['id_dfxblog_post'], $cblog['link_rewrite'], $id_lang);
            $samecats[$key] = $cblog;
        }
        
        $tagrelated = array();

        if ($this->post->tags) {
            $tagrelated = DfxPost::getListBlogs(
                $category->id_dfxblog_category,
                $this->context->language->id,
                0,
                $limit,
                'id_dfxblog_post',
                'DESC',
                array('type' => 'tag', 'tag' => $this->post->tags),
                true
            );
            
            foreach ($tagrelated as $key => $tblog) {
                $tblog['link'] = DfxPost::getLink($tblog['id_dfxblog_post'], $tblog['link_rewrite'], $id_lang);
                $tagrelated[$key] = $tblog;
            }
        }


        // form comment
        $allow_comment = isset($this->settings['allow_comment']) ? $this->settings['allow_comment'] : 1;
        $allow_views = isset($this->settings['allow_views']) ? $this->settings['allow_views'] : 1;
        $allow_likes = isset($this->settings['allow_likes']) ? $this->settings['allow_likes'] : 1;
        $enable_relatedproduct = isset($this->settings['enable_relatedproduct']) ? $this->settings['enable_relatedproduct'] : 1;
        $logged = $this->context->customer->isLogged(true);
        $customer_name = '';
        if ($logged) {
            $customer_name = $this->context->customer->firstname.' '.$this->context->customer->lastname;
        }

        // previous post
        $previous_post = $this->post->getPreviousPost($id_lang);
        // next post
        $next_post = $this->post->getNextPost($id_lang);

        
        $this->context->smarty->assign(array(
            'path' => $path,
            'dfxpost' => $this->post,
            'post_categories' => $post_categories,
            'related_posts' => $related_posts,
            'related_products' => $related_products,
            'previous_post' => $previous_post,
            'next_post' => $next_post,
            'comments' => $comments,
            'samecats' => $samecats,
            'tagrelated' => $tagrelated,
            'allow_comment' => $allow_comment,
            'allow_views' => $allow_views,
            'allow_likes' => $allow_likes,
            'enable_relatedproduct' => $enable_relatedproduct,
            'logged' => $logged,
            'customer_name' => $customer_name,
            'dfxlink' => $this->dfxlink,
            'tags' => $tags,
        ));

        $this->setTemplate('module:dfxblog/views/templates/front/dfxpost.tpl');
    }
    
    
    
    protected function getRelatedProducts($array_product_id, $id_lang)
    {
        $products = DfxPost::getProductsByArrayId($array_product_id, $id_lang);
        $present_products = array();

        if ($products) {
            $assembler = new ProductAssembler($this->context);

            $presenterFactory = new ProductPresenterFactory($this->context);
            $presentationSettings = $presenterFactory->getPresentationSettings();
            $presenter = new ProductListingPresenter(
                new ImageRetriever($this->context->link),
                $this->context->link,
                new PriceFormatter(),
                new ProductColorsRetriever(),
                $this->context->getTranslator()
            );

            foreach ($products as $rawProduct) {
                $present_products[] = $presenter->present(
                    $presentationSettings,
                    $assembler->assembleProduct($rawProduct),
                    $this->context->language
                );
            }
        }

        return $present_products;
    }
    
    public function getBreadcrumbLinks()
    {
        $breadcrumb = parent::getBreadcrumbLinks();

        $id_lang = Context::getContext()->language->id;

        $breadcrumb['links'][] = [
                'title' => $this->l('Blog'),
                'url' => DfxLink::getBlogLink($id_lang)
        ];

        $breadcrumb['links'][] = [
            'title' => $this->post->category,
            'url' => $this->post->category_url
        ];

        $breadcrumb['links'][] = [
            'title' => $this->post->name,
            'url' => $this->post->url
        ];
        return $breadcrumb;
    }
    
    protected function copyFromPost(&$object, $table, $post = array())
    {
        /* Classical fields */
        foreach ($post as $key => $value) {
            if (key_exists($key, $object) && $key != 'id_'.$table) {
                /* Do not take care of password field if empty */
                if ($key == 'passwd' && Tools::getValue('id_'.$table) && empty($value)) {
                    continue;
                }
                if ($key == 'passwd' && !empty($value)) {
                    /* Automatically encrypt password in MD5 */
                    $value = Tools::encrypt($value);
                }
                $object->{$key} = $value;
            }
        }
        
        /* Multilingual fields */
        $rules = call_user_func(array(get_class($object), 'getValidationRules'), get_class($object));
        if (count($rules['validateLang'])) {
            $languages = Language::getLanguages(false);
            foreach ($languages as $language) {
                foreach (array_keys($rules['validateLang']) as $field) {
                    $field_name = $field.'_'.(int)($language['id_lang']);
                    $value = Tools::getValue($field_name);
                    if (isset($value)) {
                        // validate module
                        $object->{$field}[(int)($language['id_lang'])] = $value;
                    }
                }
            }
        }
    }
    
    public function getPost()
    {
        return $this->post;
    }
}
