<?php
/**
* Copyright since 2007 PrestaShop SA and Contributors
* PrestaShop is an International Registered Trademark & Property of PrestaShop SA
*
* NOTICE OF LICENSE
*
* This source file is subject to the Academic Free License 3.0 (AFL-3.0)
* that is bundled with this package in the file LICENSE.md.
* It is also available through the world-wide-web at this URL:
* https://opensource.org/licenses/AFL-3.0
* If you did not receive a copy of the license and are unable to
* obtain it through the world-wide-web, please send an email
* to license@prestashop.com so we can send you a copy immediately.
*
* DISCLAIMER
*
* Do not edit or add to this file if you wish to upgrade PrestaShop to newer
* versions in the future. If you wish to customize PrestaShop for your
* needs please refer to https://devdocs.prestashop.com/ for more information.
*
* @author    PrestaShop SA and Contributors <contact@prestashop.com>
* @copyright Since 2007 PrestaShop SA and Contributors
* @license   https://opensource.org/licenses/AFL-3.0 Academic Free License 3.0 (AFL-3.0)
*/

if (!defined('_PS_VERSION_')) {
    exit;
}

use PrestaShop\PrestaShop\Adapter\Category\CategoryProductSearchProvider;
use PrestaShop\PrestaShop\Core\Product\Search\ProductSearchContext;
use PrestaShop\PrestaShop\Core\Product\Search\ProductSearchQuery;
use PrestaShop\PrestaShop\Core\Product\Search\SortOrder;

use PrestaShop\PrestaShop\Core\Module\WidgetInterface;

include_once _PS_MODULE_DIR_ . 'dfxcategorytab/classes/DfxCategorytabGroup.php';

class DfxCategoryTab extends Module implements WidgetInterface
{
    public $banner_img_folder = 'views/img/';
    public $_html = '';
    public $templateFile;
    public $currentIndex;

    public function __construct()
    {
        $this->name = 'dfxcategorytab';
        $this->tab = 'front_office_features';
        $this->version = '2.1.2';
        $this->author = 'DigitalRedfox';
        $this->need_instance = 0;
        $this->secure_key = Tools::encrypt($this->name);
        $this->bootstrap = true;

        parent::__construct();

        $this->displayName = $this->trans('Dfx Top Categories', array(), 'Modules.DfxCategorytab.Admin');
        $this->description = $this->trans('Shop by category on carousel in Top of the Homepage', array(), 'Modules.DfxCategorytab.Admin');
        $this->ps_versions_compliancy = array('min' => '1.7.1.0', 'max' => _PS_VERSION_);
        
        $this->currentIndex = AdminController::$currentIndex . '&configure=' . $this->name;
        
        
        $this->templateFile = 'module:dfxcategorytab/views/templates/hook/dfxcategorytab.tpl';
    }

    /**
     * @see Module::install()
     */
    public function install()
    {
        /* Adds Module */
        if (parent::install() &&
            $this->registerHook('displayHeader') &&
            $this->registerHook('actionShopDataDuplication') &&
            $this->registerHook('displayHomeTop')
        ) {
            /* Creates tables */
            $res = $this->createTables();

            /* Adds samples */
            if ($res) {
                $this->installSamples();
            }

            return (bool)$res;
        }

        return false;
    }

    /**
     * Adds samples
     */
	protected function installSamples()
    {
        $res = 1;
        include_once dirname(__FILE__).'/sql/sample.php';
        return $res;
    }
    
    /**
     * @see Module::uninstall()
     */
    public function uninstall()
    {
        Configuration::deleteByName('category_title');
        /* Deletes Module */
        if (parent::uninstall()) {
            /* Deletes tables */
            $res = $this->deleteTables();
            return (bool)$res;
        }

        return false;
    }

    /**
     * Creates tables
     */
    public function createTables()
    {
        /* Tab category Shop */
        $res = (bool)Db::getInstance()->execute('
            CREATE TABLE IF NOT EXISTS `'._DB_PREFIX_.'dfxcategory_shop` (
                `id_dfxcategory` int(10) unsigned NOT NULL AUTO_INCREMENT,
                `id_shop` int(10) unsigned NOT NULL,
                PRIMARY KEY (`id_dfxcategory`, `id_shop`)
            ) ENGINE='._MYSQL_ENGINE_.' DEFAULT CHARSET=UTF8;
        ');
        
        /* Tab category configuration */
        $res &= Db::getInstance()->execute('
            CREATE TABLE IF NOT EXISTS `'._DB_PREFIX_.'dfxcategory` (
              `id_dfxcategory` int(10) unsigned NOT NULL AUTO_INCREMENT,
              `position` int(10) unsigned NOT NULL DEFAULT \'0\',
              `active` tinyint(1) unsigned NOT NULL DEFAULT \'0\',
			  `scrollcategories` varchar(50) NOT NULL,
              PRIMARY KEY (`id_dfxcategory`)
            ) ENGINE='._MYSQL_ENGINE_.' DEFAULT CHARSET=UTF8;
        ');
		
		 /* Tab lang configuration */
        $res &= Db::getInstance()->execute('
            CREATE TABLE IF NOT EXISTS `'._DB_PREFIX_.'dfxcategory_lang` (
              `id_dfxcategory` int(10) unsigned NOT NULL,
              `id_lang` int(10) unsigned NOT NULL,
              `offer` varchar(255) NOT NULL,
			  `button` varchar(255) NOT NULL,
			  `banner_image` varchar(128) DEFAULT NULL,
			  `banner_image_w` varchar(255) NOT NULL,
	  		  `banner_image_h` varchar(255) NOT NULL,
              PRIMARY KEY (`id_dfxcategory`,`id_lang`)
            ) ENGINE='._MYSQL_ENGINE_.' DEFAULT CHARSET=UTF8;
        ');
        
        return $res;
    }

    /**
     * deletes tables
     */
    public function deleteTables()
    {
        $dfxcategory = $this->getDfxCategory();
        foreach ($dfxcategory as $ttcat) {
            $to_del = new DfxCategorytabGroup($ttcat['id_dfxcategory']);
            $to_del->delete();
        }

        return Db::getInstance()->execute('
            DROP TABLE IF EXISTS `'._DB_PREFIX_.'dfxcategory_shop`, `'._DB_PREFIX_.'dfxcategory`, `'._DB_PREFIX_.'dfxcategory_lang`;
        ');
    }

    public function hookDisplayHeader($params)
    {
        $this->context->controller->addCSS($this->_path.'views/css/front.css');
        $this->context->controller->registerJavascript('modules-categorytab', 'modules/'.$this->name.'/views/js/front.js', ['position' => 'bottom', 'priority' => 154]);
    }
    public function getContent()
    {
        $this->context->controller->addJS($this->_path.'views/js/adminback.js');
        $this->context->controller->addCSS($this->_path.'views/css/adminback.css');
        
        $this->_html .= $this->headerHTML();
        
        /* Validate & process */
        if (Tools::isSubmit('submitTitle')) {
            $this->_postProcess();
            $this->_html .= $this->renderTitleform().$this->renderList();
            $this->_clearCache('*');
        } elseif (Tools::isSubmit('submitDfxcategory') || Tools::isSubmit('delete_id_dfxcategory') ||
            Tools::isSubmit('changeStatus')
        ) {
            $this->_postProcess();
            $this->_html .= $this->renderTitleform().$this->renderList();
            
            $this->_clearCache('*');
        } elseif ((Tools::isSubmit('delete_tab')) && Tools::isSubmit('id_dfxcategory')) {
            $this->_postProcess();
            $this->_html .= $this->renderDfxCategoryForm();

            $this->_clearCache('*');
        } elseif (Tools::isSubmit('addDfxcategory') || (Tools::isSubmit('id_dfxcategory') && $this->dfxCategoryExists((int)Tools::getValue('id_dfxcategory')))) {
            if (Tools::isSubmit('addDfxcategory')) {
                $mode = 'add';
            } else {
                $mode = 'edit';
            }

            if ($mode == 'add') {
                if (Shop::getContext() != Shop::CONTEXT_GROUP && Shop::getContext() != Shop::CONTEXT_ALL) {
                    $this->_html .= $this->renderDfxCategoryForm();
                } else {
                    $this->_html .= $this->getShopContextError(null, $mode);
                }
            } else {
                $associated_shop_ids = DfxCategorytabGroup::getAssociatedIdsShop((int)Tools::getValue('id_dfxcategory'));
                $context_shop_id = (int)Shop::getContextShopID();

                if ($associated_shop_ids === false) {
                    $this->_html .= $this->getShopAssociationError((int)Tools::getValue('id_dfxcategory'));
                } elseif (Shop::getContext() != Shop::CONTEXT_GROUP && Shop::getContext() != Shop::CONTEXT_ALL && in_array($context_shop_id, $associated_shop_ids)) {
                    if (count($associated_shop_ids) > 1) {
                        $this->_html = $this->getSharedTabcategoryWarning();
                    }
                    $this->_html .= $this->renderDfxCategoryForm();
                } else {
                    $shops_name_list = array();
                    foreach ($associated_shop_ids as $shop_id) {
                        $associated_shop = new Shop((int)$shop_id);
                        $shops_name_list[] = $associated_shop->name;
                    }
                    $this->_html .= $this->getShopContextError($shops_name_list, $mode);
                }
            }
        } else {
            if (Shop::getContext() != Shop::CONTEXT_GROUP && Shop::getContext() != Shop::CONTEXT_ALL) {
                $this->_html .= $this->renderTitleform().$this->renderList();
            }
            $this->_html .= $this->getWarningMultishopHtml().$this->getCurrentShopInfoMsg();
        }

        return $this->_html;
    }

    
    public function _postProcess()
    {
        $errors = array();
        $languages = Language::getLanguages(false);
        $result = array();
        /* Processes Slider */
        if (Tools::isSubmit('submitTitle')) {
            foreach ($languages as $language) {
                $result['category_title'][$language['id_lang']] = Tools::getValue('category_title_'.$language['id_lang']);
            }
            
            Configuration::updateValue('category_title', $result['category_title']);
        } elseif (Tools::isSubmit('submitDfxcategory')) {
            /* Sets ID if needed */
            if (Tools::getValue('id_dfxcategory')) {
                $ttcategory = new DfxCategorytabGroup((int)Tools::getValue('id_dfxcategory'));
                if (!Validate::isLoadedObject($ttcategory)) {
                    $this->_html .= $this->displayError($this->trans('Invalid Tab category ID', array(), 'Modules.DfxCategorytab.Admin'));
                    return false;
                }
            } else {
                $ttcategory = new DfxCategorytabGroup();
            }
            /* Sets position */
            $ttcategory->position = (int)Tools::getValue('position');
            /* Sets active */
            $ttcategory->active = (int)Tools::getValue('active_dfxcategory');
            $ttcategory->scrollcategories = Tools::getValue('scrollcategories');
            /* Uploads image and sets slide */
            foreach ($languages as $language) {
                $ttcategory->offer[$language['id_lang']] = Tools::getValue('offer_'.$language['id_lang']);
				$ttcategory->button[$language['id_lang']] = Tools::getValue('button_'.$language['id_lang']);
			
	
			$type = Tools::strtolower(Tools::substr(strrchr($_FILES['banner_image_'.$language['id_lang']]['name'], '.'), 1));
            $imagesize = @getimagesize($_FILES['banner_image_'.$language['id_lang']]['tmp_name']);
                
            $ttcategory->banner_image_w[$language['id_lang']] = is_numeric(Tools::getValue('banner_image_w_'.$language['id_lang'])) ? (int)Tools::getValue('banner_image_w_'.$language['id_lang']) : '';
            $ttcategory->banner_image_h[$language['id_lang']] = is_numeric(Tools::getValue('banner_image_h_'.$language['id_lang'])) ? (int)Tools::getValue('banner_image_h_'.$language['id_lang']) : '';
                    
            if (isset($_FILES['banner_image_'.$language['id_lang']]) &&
                    isset($_FILES['banner_image_'.$language['id_lang']]['tmp_name']) &&
                    !empty($_FILES['banner_image_'.$language['id_lang']]['tmp_name']) &&
                    !empty($imagesize) &&
                    in_array(Tools::strtolower(Tools::substr(strrchr($imagesize['mime'], '/'), 1)), array( 'jpg', 'gif', 'jpeg', 'png')) &&
                    in_array($type, array('jpg', 'gif', 'jpeg', 'png'))
                ) {
                $temp_name = tempnam(_PS_TMP_IMG_DIR_, 'PS');
                $salt = sha1(microtime());
                if ($error = ImageManager::validateUpload($_FILES['banner_image_'.$language['id_lang']])) {
                    $errors[] = $error;
                } elseif (!$temp_name || !move_uploaded_file($_FILES['banner_image_'.$language['id_lang']]['tmp_name'], $temp_name)) {
                    return false;
                } elseif (!ImageManager::resize($temp_name, dirname(__FILE__).DIRECTORY_SEPARATOR.'views'.DIRECTORY_SEPARATOR.'img'.DIRECTORY_SEPARATOR.$_FILES['banner_image_'.$language['id_lang']]['name'], null, null, $type)) {
                    $errors[] = $this->displayError($this->trans('An error occurred during the image upload process.', array(), 'Admin.Notifications.Error'));
                }
                if (isset($temp_name)) {
                    @unlink($temp_name);
                }
                
                $ttcategory->banner_image[$language['id_lang']] = $_FILES['banner_image_'.$language['id_lang']]['name'];
                $ttcategory->banner_image_w[$language['id_lang']] = $imagesize['0']/1;
                $ttcategory->banner_image_h[$language['id_lang']] = $imagesize['1']/1;
            } elseif (Tools::getValue('banner_image_old_'.$language['id_lang']) != '') {
                $ttcategory->banner_image[$language['id_lang']] = Tools::getValue('banner_image_old_'.$language['id_lang']);
                $ttcategory->banner_image_w[$language['id_lang']] = $imagesize['0']/1;
                $ttcategory->banner_image_h[$language['id_lang']] = $imagesize['1']/1;
            }
           }    
                    

            /* Processes if no errors  */
            if (!$errors) {
                /* Adds */
                if (!Tools::getValue('id_dfxcategory')) {
                    if (!$ttcategory->add()) {
                        $errors[] = $this->displayError($this->trans('The Tab category could not be added.', array(), 'Modules.DfxCategorytab.Admin'));
                    }
                } elseif (!$ttcategory->update()) {
                    $errors[] = $this->displayError($this->trans('The Tab category could not be updated.', array(), 'Modules.DfxCategorytab.Admin'));
                }
                $this->_clearCache('*');
            }

            Tools::redirectAdmin($this->context->link->getAdminLink('AdminModules', true) . '&conf=6&configure=' . $this->name . '&tab_module=' . $this->tab . '&module_name=' . $this->name);
        } elseif (Tools::isSubmit('changeStatus') && Tools::isSubmit('id_dfxcategory')) {
            $ttcategory = new DfxCategorytabGroup((int)Tools::getValue('id_dfxcategory'));
            if ($ttcategory->active == 0) {
                $ttcategory->active = 1;
            } else {
                $ttcategory->active = 0;
            }
            
            
            $res = $ttcategory->update();

            $this->_clearCache('*');
            $this->_html .= ($res ? $this->displayConfirmation($this->trans('Configuration updated', array(), 'Admin.Notifications.Success')) : $this->displayError($this->trans('The configuration could not be updated.', array(), 'Modules.DfxCategorytab.Admin')));
        } elseif (Tools::isSubmit('delete_id_dfxcategory')) {
            $ttcategory = new DfxCategorytabGroup((int)Tools::getValue('delete_id_dfxcategory'));
            $res = $ttcategory->delete();
            $this->_clearCache('*');
            if (!$res) {
                $this->_html .= $this->displayError('Could not delete.');
            } else {
                Tools::redirectAdmin($this->context->link->getAdminLink('AdminModules', true) . '&conf=1&configure=' . $this->name . '&tab_module=' . $this->tab . '&module_name=' . $this->name);
            }
        } 
        /* Display errors if needed */
        if (count($errors)) {
            $this->_html .= $this->displayError(implode('<br />', $errors));
        } elseif (Tools::isSubmit('submitDfxcategory') && Tools::getValue('id_dfxcategory')) {
            Tools::redirectAdmin($this->context->link->getAdminLink('AdminModules', true) . '&conf=4&configure=' . $this->name . '&tab_module=' . $this->tab . '&module_name=' . $this->name);
        } elseif (Tools::isSubmit('submitDfxcategory')) {
            Tools::redirectAdmin($this->context->link->getAdminLink('AdminModules', true) . '&conf=3&configure=' . $this->name . '&tab_module=' . $this->tab . '&module_name=' . $this->name);
        }
    }
    
    public function renderTitleform()
    {
        $fields_form = array(
        'form' => array(
            'legend' => array(
                'title' => $this->trans('Category Title'),
            ),
            'input' => array(
               array(
                        'col' => 7,
                        'type' => 'text',
                        'label' => $this->getTranslator()->trans('Title', array(), 'Admin.Global'),
                        'name' => 'category_title',
                        'lang' => true,
                        'desc' => $this->getTranslator()->trans('Add your Title', array(), 'Admin.Global')
                    )
            ),
            'submit' => array(
                     'title' => $this->getTranslator()->trans('Submit', array(), 'Admin.Actions'),
                     'name' => 'submitTitle',
                )
            ),
        );
        $lang = new Language((int) Configuration::get('PS_LANG_DEFAULT'));

        $helper = new HelperForm();
        $helper->show_toolbar = false;
        $helper->table = $this->table;
        $lang = new Language((int)Configuration::get('PS_LANG_DEFAULT'));
        $helper->default_form_language = $lang->id;
        $helper->allow_employee_form_lang = Configuration::get('PS_BO_ALLOW_EMPLOYEE_FORM_LANG') ? Configuration::get('PS_BO_ALLOW_EMPLOYEE_FORM_LANG') : 0;
        $this->fields_form = array();

        $helper->identifier = $this->identifier;
        $helper->submit_action = 'submitTitle';
        $helper->currentIndex = $this->context->link->getAdminLink('AdminModules', false).'&configure='.$this->name.'&tab_module='.$this->tab.'&module_name='.$this->name;
        $helper->token = Tools::getAdminTokenLite('AdminModules');
        $helper->tpl_vars = array(
            'fields_value' => $this->getConfigFieldsTitleValues(),
            'languages' => $this->context->controller->getLanguages(),
            'id_language' => $this->context->language->id
        );

        return $helper->generateForm(array($fields_form));
    }
    
    public function getConfigFieldsTitleValues()
    {
        $fields = array();

        $languages = Language::getLanguages(false);

        foreach ($languages as $lang) {
            $fields['category_title'][$lang['id_lang']] = Tools::getValue('category_title_'.$lang['id_lang'], Configuration::get('category_title', $lang['id_lang']));
        }

        return $fields;
    }
    
    public function clearCache()
    {
        $this->_clearCache($this->templateFile);
    }

    public function hookActionShopDataDuplication($params)
    {
        Db::getInstance()->execute(
            '
            INSERT IGNORE INTO '._DB_PREFIX_.'dfxcategory_shop (id_dfxcategory, id_shop)
            SELECT id_dfxcategory, '.(int)$params['new_id_shop'].'
            FROM '._DB_PREFIX_.'dfxcategory_shop
            WHERE id_shop = '.(int)$params['old_id_shop']
        );
        
        $this->_clearCache('*');
    }

    public function headerHTML()
    {
        if (Tools::getValue('controller') != 'AdminModules' && Tools::getValue('configure') != $this->name) {
            return;
        }

        $this->context->controller->addJqueryUI('ui.sortable');
        /* Style & js for fieldset 'Tabcategory configuration' */
        $html = '<script type="text/javascript">
            $(function() {
                var $myCategorytabs = $("#ttcategory");
                $myCategorytabs.sortable({
                    opacity: 0.6,
                    cursor: "move",
                    update: function() {
                        var order = $(this).sortable("serialize") + "&action=updateListPosition";
                        $.post("'.$this->context->shop->physical_uri.$this->context->shop->virtual_uri.'modules/'.$this->name.'/ajax_'.$this->name.'.php?secure_key='.$this->secure_key.'", order);
                        }
                    });
                $myCategorytabs.hover(function() {
                    $(this).css("cursor","move");
                    },
                    function() {
                    $(this).css("cursor","auto");
                });

            });
        </script>';

        return $html;
    }

    public function getDfxcategory($active = null)
    {
        $this->context = Context::getContext();
        $id_shop = $this->context->shop->id;
		$id_lang = $this->context->language->id;

        $ttcategory = Db::getInstance(_PS_USE_SQL_SLAVE_)->executeS(
            '
            SELECT hs.`id_dfxcategory` as id_dfxcategory, hss.`position`, hss.`active`, hss.`scrollcategories`, hssl.`offer`, hssl.`button`, hssl.`banner_image`, hssl.`banner_image_w`, hssl.`banner_image_h` FROM '._DB_PREFIX_.'dfxcategory_shop hs
            LEFT JOIN '._DB_PREFIX_.'dfxcategory hss ON (hs.id_dfxcategory = hss.id_dfxcategory)
			LEFT JOIN '._DB_PREFIX_.'dfxcategory_lang hssl ON (hss.id_dfxcategory = hssl.id_dfxcategory)
            WHERE id_shop = '.(int)$id_shop.'
			AND hssl.id_lang = '.(int)$id_lang.
            ($active ? ' AND hss.`active` = 1' : ' ').'
            ORDER BY hss.position ASC, hss.id_dfxcategory ASC'
        );
        
        foreach ($ttcategory as &$ttcat_tab) {
            $ttcat_tab['image_url'] = $this->context->link->getMediaLink(_MODULE_DIR_.$this->name.'/views/img/'.$ttcat_tab['banner_image']);
        }
        
        return $ttcategory;
    }

    public function displayStatus($id_dfxcategory, $active)
    {
        $title = ((int)$active == 0 ? $this->trans('Disabled', array(), 'Admin.Global') : $this->trans('Enabled', array(), 'Admin.Global'));
        $icon = ((int)$active == 0 ? 'icon-remove' : 'icon-check');
        $class = ((int)$active == 0 ? 'btn-danger' : 'btn-success');
        $html = '<a class="btn '.$class.'" href="'.AdminController::$currentIndex.
            '&configure='.$this->name.
                '&token='.Tools::getAdminTokenLite('AdminModules').
                '&changeStatus&id_dfxcategory='.(int)$id_dfxcategory.'" title="'.$title.'"><i class="'.$icon.'"></i> '.$title.'</a>';

        return $html;
    }

    public function dfxCategoryExists($id_dfxcategory)
    {
        $req = 'SELECT hs.`id_dfxcategory` as id_dfxcategory
                FROM `'._DB_PREFIX_.'dfxcategory_shop` hs
                WHERE hs.`id_dfxcategory` = '.(int)$id_dfxcategory;
        $row = Db::getInstance(_PS_USE_SQL_SLAVE_)->getRow($req);

        return ($row);
    }
 
    public function renderDfxCategoryForm()
    {

        $fields_form = array(
            'form' => array(
                'legend' => array(
                    'title' => $this->trans('Category Block Information', array(), 'Modules.DfxCategorytab.Admin'),
                    'icon' => 'icon-cogs'
                ),
                'input' => array(
                    array(
                        'type' => 'file_lang',
                        'label' => $this->getTranslator()->trans('Image', array(), 'Admin.Global'),
                        'name' => 'banner_image',
                        'required' => true,
                        'lang' => true,
                        'desc' => $this->getTranslator()->trans('Maximum image size: %s.', array(ini_get('upload_max_filesize')), 'Admin.Global')
                    ),
                    array(
                        'type' => 'text',
                        'label' => $this->trans('Width', array(), 'Modules.DfxCategorytab.Admin'),
                        'name' => 'banner_image_w',
                        'suffix' => 'pixels',
                        'class' => 'fixed-width-sm',
                        'desc' => $this->getTranslator()->trans('Image width', array(), 'Modules.DfxCategorytab.Admin'),
                        'lang' => true,
                    ),
                    array(
                        'type' => 'text',
                        'label' => $this->trans('Height', array(), 'Modules.DfxCategorytab.Admin'),
                        'name' => 'banner_image_h',
                        'suffix' => 'pixels',
                        'class' => 'fixed-width-sm',
                        'desc' => $this->getTranslator()->trans('Image height', array(), 'Modules.DfxCategorytab.Admin'),
                        'lang' => true,
                    ),
                    array(
                        'type' => 'select_link',
                        'label' => $this->trans('Select Category'),
                        'name' => 'scrollcategories'
                    ),
					array(
                        'type' => 'text',
                        'label' => $this->getTranslator()->trans('Offer', array(), 'Admin.Global'),
                        'name' => 'offer',
                        'lang' => true,
                    ),
					array(
                        'type' => 'text',
                        'label' => $this->getTranslator()->trans('Button Text', array(), 'Admin.Global'),
                        'name' => 'button',
                        'lang' => true,
                    ),
                    array(
                        'type' => 'switch',
                        'label' => $this->trans('Enabled', array(), 'Admin.Global'),
                        'name' => 'active_dfxcategory',
                        'is_bool' => true,
                        'values' => array(
                            array(
                                'id' => 'active_on',
                                'value' => 1,
                                'label' => $this->trans('Yes', array(), 'Admin.Global')
                            ),
                            array(
                                'id' => 'active_off',
                                'value' => 0,
                                'label' => $this->trans('No', array(), 'Admin.Global')
                            )
                        ),
                    ),
                ),
                'submit' => array(
                    'title' => $this->trans('Save', array(), 'Admin.Actions'),
                ),
                'buttons' => array(
                    array(
                        'href' => $this->currentIndex . '&token=' . Tools::getAdminTokenLite('AdminModules'),
                        'title' => $this->trans('Back'),
                        'icon' => 'process-icon-back',
                    ),
                ),
            ),
        );
        
        if (Tools::isSubmit('id_dfxcategory') && $this->dfxCategoryExists((int)Tools::getValue('id_dfxcategory'))) {
			$ttcategory = new DfxCategorytabGroup((int)Tools::getValue('id_dfxcategory'));
            $fields_form['form']['input'][] = array('type' => 'hidden', 'name' => 'id_dfxcategory');
			$fields_form['form']['images'] = $ttcategory->banner_image;
			$has_picture = true;

            foreach (Language::getLanguages(false) as $lang) {
                if (!isset($ttcategory->banner_image[$lang['id_lang']])) {
                    $has_picture &= false;
                }
            }

            if ($has_picture) {
                $fields_form['form']['input'][] = array('type' => 'hidden', 'name' => 'has_picture');
            }
            $cat_value = $ttcategory->scrollcategories;
        } else {
            if ($ttcategory->scrollcategories) {
                $cat_value = $ttcategory->scrollcategories;
            } else {
                $cat_value = 3;
            }
        }
        
        $helper = new HelperForm();
        $helper->show_toolbar = false;
        $helper->table = $this->table;
        $lang = new Language((int)Configuration::get('PS_LANG_DEFAULT'));
        $helper->default_form_language = $lang->id;
        $helper->allow_employee_form_lang = Configuration::get('PS_BO_ALLOW_EMPLOYEE_FORM_LANG') ? Configuration::get('PS_BO_ALLOW_EMPLOYEE_FORM_LANG') : 0;
        $this->fields_form = array();
        $helper->module = $this;
        $helper->identifier = $this->identifier;
        $helper->submit_action = 'submitDfxcategory';
        $helper->currentIndex = $this->context->link->getAdminLink('AdminModules', false).'&configure='.$this->name.'&tab_module='.$this->tab.'&module_name='.$this->name;
        $helper->token = Tools::getAdminTokenLite('AdminModules');
        $language = new Language((int)Configuration::get('PS_LANG_DEFAULT'));
        $helper->tpl_vars = array(
            'base_url' => $this->context->shop->getBaseURL(),
            'language' => array(
                'id_lang' => $language->id,
                'iso_code' => $language->iso_code
            ),
            'fields_value' => $this->getAddFieldsValues(),
            'languages' => $this->context->controller->getLanguages(),
            'id_language' => $this->context->language->id,
            'cat_options' => $this->getAllCatLink(),
            'cat_value' => $cat_value,
			'image_baseurl' => $this->_path.'views/img/'
        );
         
        $helper->override_folder = '/';

    
        return $helper->generateForm(array($fields_form));
    }

    public function getAddFieldsValues()
    {
        $fields = array();
        $id_dfxcategory = (int) Tools::getValue('id_dfxcategory');
       
       if (Tools::isSubmit('id_dfxcategory') && $this->dfxCategoryExists((int)Tools::getValue('id_dfxcategory'))) {
            $ttcategory = new DfxCategorytabGroup((int)Tools::getValue('id_dfxcategory'));
            $fields['id_dfxcategory'] = (int)Tools::getValue('id_dfxcategory', $ttcategory->id);
        } else {
            $ttcategory = new DfxCategorytabGroup();
        }

        $fields['has_picture'] = true;
        $fields['active_dfxcategory'] = Tools::getValue('active_dfxcategory', $ttcategory->active);
        $languages = Language::getLanguages(false);

        foreach ($languages as $lang) {
		$fields['offer'][$lang['id_lang']] = Tools::getValue('offer_'.(int)$lang['id_lang'], $ttcategory->offer[$lang['id_lang']]);
		$fields['button'][$lang['id_lang']] = Tools::getValue('button_'.(int)$lang['id_lang'], $ttcategory->button[$lang['id_lang']]);
		
		$fields['banner_image'][$lang['id_lang']] = Tools::getValue('banner_image_'.(int)$lang['id_lang']);
        $fields['banner_image_w'][$lang['id_lang']] = is_numeric(Tools::getValue('banner_image_w_'.(int)$lang['id_lang'], $ttcategory->banner_image_w[$lang['id_lang']])) ? (int)Tools::getValue('banner_image_w_'.(int)$lang['id_lang'], $ttcategory->banner_image_w[$lang['id_lang']]) : '';
                
        $fields['banner_image_h'][$lang['id_lang']] = is_numeric(Tools::getValue('banner_image_h_'.(int)$lang['id_lang'], $ttcategory->banner_image_h[$lang['id_lang']])) ? (int)Tools::getValue('banner_image_h_'.(int)$lang['id_lang'], $ttcategory->banner_image_h[$lang['id_lang']]) : '';
		}
        $fields['scrollcategories'] = Tools::getValue('scrollcategories', $ttcategory->scrollcategories);
        return $fields;
    }
    
    public function getAllCatLink($id_lang = null, $link = false)
    {
        if (is_null($id_lang)) {
            $id_lang = (int)$this->context->language->id;
        }
        $html = '<optgroup label="'.$this->trans('Category').'">';
        $html .= $this->getCategoryOption(1, $id_lang, false, true, $link);
        $html .= '</optgroup>';
        return $html;
    }
    
    public function getCategoryOption($id_category = 1, $id_lang = false, $id_shop = false, $recursive = true, $link = false)
    {
        $html = '';
        $id_lang = $id_lang ? (int)$id_lang : (int)Context::getContext()->language->id;
        $id_shop = $id_shop ? (int)$id_shop : (int)Context::getContext()->shop->id;
        $category = new Category((int)$id_category, (int)$id_lang, (int)$id_shop);
        if (is_null($category->id)) {
            return;
        }
        if ($recursive) {
            $children = Category::getChildren((int)$id_category, (int)$id_lang, true, (int)$id_shop);
            $spacer = str_repeat('&nbsp;', 3 * (int)$category->level_depth);
        }

        if (!in_array($category->id, array(Configuration::get('PS_HOME_CATEGORY'), Configuration::get('PS_ROOT_CATEGORY')))) {
            if ($link) {
                $html .= '<option value="'.$this->context->link->getCategoryLink($category->id).'">'.(isset($spacer) ? $spacer : '').str_repeat('&nbsp;', 3 * (int)$category->level_depth).$category->name.'</option>';
            } else {
                $html .= '<option value="'.(int)$category->id.'">'.str_repeat('&nbsp;', 3 * (int)$category->level_depth).$category->name.'</option>';
            }
        } elseif ($category->id != Configuration::get('PS_ROOT_CATEGORY')) {
            $html .= '<optgroup label="'.str_repeat('&nbsp;', 3 * (int)$category->level_depth).$category->name.'">';
        }
        if (isset($children) && count($children)) {
            foreach ($children as $child) {
                $html .= $this->getCategoryOption(
                    (int)$child['id_category'],
                    (int)$id_lang,
                    (int)$child['id_shop'],
                    $recursive,
                    $link
                );
            }
        }
        return $html;
    }
        
    public function getWarningMultishopHtml()
    {
        if (Shop::getContext() == Shop::CONTEXT_GROUP || Shop::getContext() == Shop::CONTEXT_ALL) {
            return '<p class="alert alert-warning">' .
            $this->trans('You cannot manage slides items from a "All Shops" or a "Group Shop" context, select directly the shop you want to edit', array(), 'Modules.DfxCategorytab.Admin') .
            '</p>';
        } else {
            return '';
        }
    }

    public function getShopContextError($shop_contextualized_name, $mode)
    {
        if (is_array($shop_contextualized_name)) {
            $shop_contextualized_name = implode('<br/>', $shop_contextualized_name);
        }

        if ($mode == 'edit') {
            return '<p class="alert alert-danger">' .
            $this->trans('You can only edit this Category Tab from the shop(s) context: %s', array($shop_contextualized_name), 'Modules.DfxCategorytab.Admin') .
            '</p>';
        } else {
            return '<p class="alert alert-danger">' .
            $this->trans('You cannot add Category Tab from a "All Shops" or a "Group Shop" context', array(), 'Modules.DfxCategorytab.Admin') .
            '</p>';
        }
    }

    public function getShopAssociationError($id_dfxcategory)
    {
        return '<p class="alert alert-danger">'.
                        $this->trans('Unable to get menu shop association information (id_dfxcategory: %d)', array((int)$id_dfxcategory), 'Modules.DfxCategorytab.Admin') .
                '</p>';
    }


    protected function getCurrentShopInfoMsg()
    {
        $shop_info = null;

        if (Shop::isFeatureActive()) {
            if (Shop::getContext() == Shop::CONTEXT_SHOP) {
                $shop_info = $this->trans('The modifications will be applied to shop: %s', array($this->context->shop->name), 'Modules.DfxCategorytab.Admin');
            } elseif (Shop::getContext() == Shop::CONTEXT_GROUP) {
                $shop_info = $this->trans('The modifications will be applied to this group: %s', array(Shop::getContextShopGroup()->name), 'Modules.DfxCategorytab.Admin');
            } else {
                $shop_info = $this->trans('The modifications will be applied to all shops and shop groups', array(), 'Modules.DfxCategorytab.Admin');
            }

            return '<div class="alert alert-info">'.
                        $shop_info.
                    '</div>';
        } else {
            return '';
        }
    }

    public function getSharedTabcategoryWarning()
    {
        return '<p class="alert alert-warning">'.
                    $this->trans('This slide is shared with other shops! All shops associated to this slide will apply modifications made here', array(), 'Modules.DfxCategorytab.Admin').
                '</p>';
    }
    
    /* Front Office Features */
    public function renderList()
    {
        $dfxcategory = $this->getDfxcategory();
        $id_lang = $this->context->language->id;

        foreach ($dfxcategory as $key => $ttcat) {
            $id_cat = $ttcat['scrollcategories'];
            $category = new Category($id_cat, (int)$id_lang);
            $dfxcategory[$key]['status'] = $this->displayStatus($ttcat['id_dfxcategory'], $ttcat['active']);
            $associated_shop_ids = DfxCategorytabGroup::getAssociatedIdsShop((int)$ttcat['id_dfxcategory']);
            if ($associated_shop_ids && count($associated_shop_ids) > 1) {
                $dfxcategory[$key]['is_shared'] = true;
            } else {
                $dfxcategory[$key]['is_shared'] = false;
            }
            $categoryids = $this->getCategoryByIds($id_cat, (int)$id_lang);
            $prodIds = array();
                                            
            foreach ($categoryids as $keys => $productids) {
                $prodIds[$keys] = $productids['id_product'];
            }
            $dfxcategory[$key]['product_count'] = count($prodIds);
                                
                                
            $dfxcategory[$key]['cat_name'] = $category->name;
        }
                                            
        $this->context->smarty->assign(
            array(
                'link' => $this->context->link,
                'dfxcategory' => $dfxcategory,
                'banner_image_url' => $this->_path.$this->banner_img_folder,
            )
        );

        return $this->display(__FILE__, 'list.tpl');
    }

    public function _clearCache($template, $cache_id = null, $compile_id = null)
    {
        parent::_clearCache($this->templateFile);
    }

    public function renderWidget($hookName = null, array $configuration = array())
    {
	  if (!preg_match('/^displayHeader\d*$/', $hookName)) {
        if (!$this->isCached($this->templateFile, $this->getCacheId($this->name))) {
            $variables = $this->getWidgetVariables($hookName, $configuration);
            if (empty($variables)) {
                return false;
            }
            $this->smarty->assign($variables);
        }

        return $this->fetch($this->templateFile, $this->getCacheId($this->name));
	  }
    }

    public function getWidgetVariables($hookName = null, array $configuration = array())
    {
        $categoryList = $this->getCatblocks();
        
        if (!empty($categoryList)) {
            return $categoryList;
        }
        return false;
    }
    
    public function getCatblocks()
    {
        $id_lang = (int)$this->context->language->id;
        $dfxcategory = $this->getDfxcategory(true);
        
        foreach ($dfxcategory as $key => $ttcat) {
            $id_cat = $ttcat['scrollcategories'];
            
            $categoryids = $this->getCategoryByIds($id_cat, (int)$id_lang);
            $prodIds = array();
                                            
            foreach ($categoryids as $keys => $productids) {
                $prodIds[$keys] = $productids['id_product'];
            }
            $dfxcategory[$key]['product_count'] = count($prodIds);
                                
            $category = new Category($id_cat, (int)$id_lang);
            $dfxcategory[$key]['cat_name'] = $category->name;
            $dfxcategory[$key]['link'] = $this->context->link->getCategoryLink($id_cat, $category->link_rewrite);
        }

        return [
            'dfxcategory' => $dfxcategory,
            'category_title' => Configuration::get('category_title', $this->context->language->id)
        ];
    }
    
    public function getCategoryByIds($id_cat, $id_lang)
    {
        $db = Db::getInstance(_PS_USE_SQL_SLAVE_);
        
        $sql_groups = '';
        if (Group::isFeatureActive()) {
            $groups = FrontController::getCurrentCustomerGroups();
            $sql_groups = 'AND cg.`id_group` '.(count($groups) ? 'IN ('.implode(',', $groups).')' : '= 1');
        }

        $results = $db->executeS('
		SELECT cp.`id_product`
		FROM `'._DB_PREFIX_.'category_product` cp
		'.(Group::isFeatureActive() ? 'INNER JOIN `'._DB_PREFIX_.'category_group` cg ON cp.`id_category` = cg.`id_category`' : '').'
		INNER JOIN `'._DB_PREFIX_.'category` c ON cp.`id_category` = c.`id_category`
        INNER JOIN `'._DB_PREFIX_.'product` p ON cp.`id_product` = p.`id_product`
		'.Shop::addSqlAssociation('product', 'p', false).'
        INNER JOIN `'._DB_PREFIX_.'product_lang` pl ON p.`id_product` = pl.`id_product`
		WHERE c.`active` = 1
		AND product_shop.`active` = 1
		AND product_shop.`active` = 1
		'.($id_cat ? 'AND c.`id_category` = '.$id_cat.'':'').'
        '.($id_lang ? 'AND pl.`id_lang` = '.$id_lang.'':'').'
		AND product_shop.`visibility` IN ("both", "search")
		AND product_shop.indexed = 1
		'.$sql_groups, true, false);
        
        return $results;
    }
}
