<?php
/**
* 2007-2021 PrestaShop
*
* NOTICE OF LICENSE
*
* This source file is subject to the Academic Free License (AFL 3.0)
* that is bundled with this package in the file LICENSE.txt.
* It is also available through the world-wide-web at this URL:
* http://opensource.org/licenses/afl-3.0.php
* If you did not receive a copy of the license and are unable to
* obtain it through the world-wide-web, please send an email
* to license@prestashop.com so we can send you a copy immediately.
*
* DISCLAIMER
*
* Do not edit or add to this file if you wish to upgrade PrestaShop to newer
* versions in the future. If you wish to customize PrestaShop for your
* needs please refer to http://www.prestashop.com for more information.
*
*  @author PrestaShop SA <contact@prestashop.com>
*  @copyright  2007-2021 PrestaShop SA
*  @license    http://opensource.org/licenses/afl-3.0.php  Academic Free License (AFL 3.0)
*  International Registered Trademark & Property of PrestaShop SA
*/
use PrestaShop\PrestaShop\Adapter\Image\ImageRetriever;
use PrestaShop\PrestaShop\Core\Product\ProductListingPresenter;
use PrestaShop\PrestaShop\Adapter\Product\ProductColorsRetriever;
use PrestaShop\PrestaShop\Adapter\Product\PriceFormatter;

if (!defined('_PS_VERSION_')) {
    exit;
}

class DfxCompare extends Module
{
	public $configTit;
    protected $templateFile;
    
    public function __construct()
    {
        $this->name = 'dfxcompare';
        $this->version = '2.2.0';
        $this->author = 'Digitalredfox';
        $this->need_instance = 0;
        $this->bootstrap = true;
        $this->tab = 'front_office_features';
        $this->controllers = array('comparator', 'actions');

        parent::__construct();
        $this->displayName = $this->trans('Dfx Compare products', array(), 'Modules.DfxCompare.Admin');
        $this->description = $this->trans('Allow customers to compare products', array(), 'Modules.DfxCompare.Admin');

        $this->ps_versions_compliancy = array('min' => '1.7', 'max' => _PS_VERSION_);
		$this->defaults = array(
            'dfxcomparenb' => 0,
        );
    }

    public function install()
    {
        return (parent::install()
		&& $this->setDefaults()
            && $this->registerHook('displayHeader')
            && $this->registerHook('displayTop')
            && $this->registerHook('displayAfterProductAddCartBtn')
            && $this->registerHook('displayProductListFunctionalButtons')
            && $this->registerHook('displayBeforeBodyClosingTag')
            && $this->registerHook('displayProductcompare')
			&& $this->registerHook('actionProductAdd')
			&& $this->registerHook('actionProductSave')
			&& $this->registerHook('actionProductUpdate')
			&& $this->registerHook('actionProductDelete')
        );
    }

    public function uninstall()
    {
        return parent::uninstall();
    }
	
	public function setDefaults()
    {
        foreach ($this->defaults as $default => $value) {
            Configuration::updateValue($this->configTit . $default, $value);
        }
        return true;
    }

    public function hookDisplayHeader()
    {
        $this->context->controller->registerJavascript($this->name.'-2', 'modules/'.$this->name.'/views/js/front.js', ['position' => 'bottom', 'priority' => 152]);
        $this->context->controller->addCSS($this->_path.'views/css/front.css');
		$id_lang = (int)Context::getContext()->language->id;
		
		$allcompareIds = array();
        $productsIds = array();
		
        $productsIds = Context::getContext()->cookie->dfxCompare;

        if ($productsIds) {
        $allcompareIds = Tools::jsonDecode($productsIds, true);
		}
		
        Media::addJsDef(
            array(
            'dfxcomparenb' =>  (int) $this->context->cookie->dfxCompareNb,
			'in_compare' => $allcompareIds,
            )
        );
    }
	public function hookActionProductAdd()
    {
        $this->clearCache();
    }

    public function hookActionProductSave()
    {
        $this->clearCache();
    }

    public function hookActionProductUpdate()
    {
        $this->clearCache();
    }

    public function hookActionProductDelete()
    {
        $this->clearCache();
    }
	 protected function clearCache()
    {
        $this->_clearCache('product-buttons.tpl');
    }
    public function hookDisplayTop()
    {
		return $this->fetch('module:'.$this->name.'/views/templates/hook/display-nav.tpl');
    }
	public function hookDisplayAfterProductAddCartBtn($params)
    {
       return $this->addToCompareButton($params);
    }
	public function hookDisplayBeforeBodyClosingTag($params)
    {
        return $this->fetch('module:'.$this->name.'/views/templates/hook/display-modal.tpl');
    }
	public function hookDisplayProductListFunctionalButtons($params)
    {
        return $this->addToCompareButton($params);
    }
	
	public function addToCompareButton($params)
    {
		$productsIds = Context::getContext()->cookie->dfxCompare;

        if ($productsIds) {
        $allcompareIds = Tools::jsonDecode($productsIds, true);

		  if (in_array($params['product']->id_product, $allcompareIds)) {
			$params['product']['in_compare'] = 'compare-added';
          }

		}
			
        $this->smarty->assign(array(
             'id_product_attribute' => $params['product']->id_product_attribute,
             'id_product' => $params['product']->id_product,
			 'product' => $params['product'],
        ));

      return $this->fetch('module:'.$this->name.'/views/templates/hook/product-buttons.tpl');
    }

    public function getFeaturesForComparison($idsArray, $idLang)
    {
        if (!Feature::isFeatureActive()) {
            return false;
        }

        $ids = implode(",", $idsArray);

        if (empty($ids)) {
            return false;
        }

        return Db::getInstance()->executeS('
			SELECT f.*, fl.*
			FROM `'._DB_PREFIX_.'feature` f
			LEFT JOIN `'._DB_PREFIX_.'feature_product` fp
				ON f.`id_feature` = fp.`id_feature`
			LEFT JOIN `'._DB_PREFIX_.'feature_lang` fl
				ON f.`id_feature` = fl.`id_feature`
			WHERE fp.`id_product` IN ('.$ids.')
			AND `id_lang` = '.(int)$idLang.'
			GROUP BY f.`id_feature`
			ORDER BY f.`position` ASC
		');
    }
}
