/**
 * Copyright since 2007 PrestaShop SA and Contributors
 * PrestaShop is an International Registered Trademark & Property of PrestaShop SA
 *
 * NOTICE OF LICENSE
 *
 * This source file is subject to the Academic Free License 3.0 (AFL-3.0)
 * that is bundled with this package in the file LICENSE.md.
 * It is also available through the world-wide-web at this URL:
 * https://opensource.org/licenses/AFL-3.0
 * If you did not receive a copy of the license and are unable to
 * obtain it through the world-wide-web, please send an email
 * to license@prestashop.com so we can send you a copy immediately.
 *
 * DISCLAIMER
 *
 * Do not edit or add to this file if you wish to upgrade PrestaShop to newer
 * versions in the future. If you wish to customize PrestaShop for your
 * needs please refer to https://devdocs.prestashop.com/ for more information.
 *
 * @author    PrestaShop SA and Contributors <contact@prestashop.com>
 * @copyright Since 2007 PrestaShop SA and Contributors
 * @license   https://opensource.org/licenses/AFL-3.0 Academic Free License 3.0 (AFL-3.0)
 */
$(document).ready(function() {
  var $container = $(".portfolio-items");
  $container.isotope({
    itemSelector: "article",
    layoutMode: "fitRows",
    getSortData: {
      name: ".name",
      brand: "[data-brand]",
      idproduct: "[data-id-product] parseInt",
      price: function (itemElem) {
        var price = $(itemElem).find("span.price").text();
        return parseFloat(price.replace(/[^0-9\.-]+/g, ""));
      },
    },
  });
  $container.find(".product-miniature").each(function () {
    $(this).css("visibility", "hidden");
  });
  var $itemsFilter = $(".casual-list-filter");
  $itemsFilter.find("li").first().addClass("selected");

  var dataFilter = $itemsFilter
    .find("li")
    .first()
    .find("a")
    .attr("data-filter-value");
  if (dataFilter != "*") {
    dataFilter = "." + dataFilter;
  }
  $container.find(".product-miniature" + dataFilter).each(function () {
    $(this).css("visibility", "visible");
  });
  $container.isotope({ filter: dataFilter });

  $itemsFilter.on("click", "a", function (e) {
    var selector = $(this).attr("data-filter-value");

    if (selector != "*") {
      selector = "." + selector;
    }
    $container.find(".product-miniature" + selector).each(function () {
      $(this).find("img.js-lazyload").trigger("appear");
      $(this).css("visibility", "visible");
    });
    $container.isotope({ filter: selector });
    $itemsFilter.find("li.selected").removeClass("selected");
    $(this).parent().parent("li").addClass("selected");
    e.preventDefault();
  });

  $(".sort-by-group").on("click", "button", function () {
    var sortbyItem = $(this).attr("data-sort-by");
    var direction = $(this).attr("data-sort-direction");
    var isAscending = direction == "asc";
    var newDirection = isAscending ? "desc" : "asc";

    $container.isotope({ sortBy: sortbyItem, sortAscending: isAscending });
    $(this).attr("data-sort-direction", newDirection);

    var icon = $(this).find(".icon");
    icon.toggleClass("redfox-chevron-up redfox-chevron-down");

    $(".sort-by-group .sort-btn.active").removeClass("active");
    $(this).addClass("active");
  });
});
/* end Portfolio Filter */
