<?php
/**
* 2007-2021 PrestaShop
*
* NOTICE OF LICENSE
*
* This source file is subject to the Academic Free License (AFL 3.0)
* that is bundled with this package in the file LICENSE.txt.
* It is also available through the world-wide-web at this URL:
* http://opensource.org/licenses/afl-3.0.php
* If you did not receive a copy of the license and are unable to
* obtain it through the world-wide-web, please send an email
* to license@prestashop.com so we can send you a copy immediately.
*
* DISCLAIMER
*
* Do not edit or add to this file if you wish to upgrade PrestaShop to newer
* versions in the future. If you wish to customize PrestaShop for your
* needs please refer to http://www.prestashop.com for more information.
*
*  @author PrestaShop SA <contact@prestashop.com>
*  @copyright  2007-2021 PrestaShop SA
*  @license    http://opensource.org/licenses/afl-3.0.php  Academic Free License (AFL 3.0)
*  International Registered Trademark & Property of PrestaShop SA
*/

class DfxmenuClass extends ObjectModel
{
    public $id;
    public $title;
    public $url;
    public $active;
    public $position;
    public $id_shop;
    public $dropwidth = '100%';
    public $menu_image;
    public $menu_image_w = '130';
    public $menu_image_h = '230';
    public $background_position;
    public $background_stretch;
    public $custom_css;
    public $new_window;
	public $menu_type;
	public $categorieslink;
	public $manufacturerlink;
	public $supplierlink;
	public $cmslink;

    /**
     * @see ObjectModel::$definition
     */
    public static $definition = array(
        'table' => 'dfxmenu',
        'primary' => 'id_dfxmenu',
        'multilang' => true,
        'fields' => array(
            'active' => array('type' => self::TYPE_BOOL, 'validate' => 'isBool', 'required' => true),
            'new_window' => array('type' => self::TYPE_BOOL, 'validate' => 'isBool'),
            'position' => array('type' => self::TYPE_INT, 'validate' => 'isunsignedInt', 'required' => true),
            // Lang fields
            
            'title' => array('type' => self::TYPE_STRING, 'lang' => true, 'validate' => 'isCleanHtml', 'required' => true, 'size' => 255),
            'url' => array('type' => self::TYPE_STRING, 'lang' => true, 'validate' => 'isUrl', 'size' => 255),
			'categorieslink' => array('type' => self::TYPE_INT),   
            'manufacturerlink' => array('type' => self::TYPE_INT),
            'supplierlink' => array('type'=>self::TYPE_INT),
            'cmslink' => array('type' => self::TYPE_INT),
			'menu_type' => array('type' => self::TYPE_STRING),
            'dropwidth' => array('type' => self::TYPE_STRING, 'validate' => 'isCleanHtml', 'size' => 255),
            'menu_image_w' => array('type' => self::TYPE_STRING, 'validate' => 'isCleanHtml', 'size' => 255),
            'menu_image_h' => array('type' => self::TYPE_STRING, 'validate' => 'isCleanHtml', 'size' => 255),
            'menu_image' => array('type' => self::TYPE_STRING, 'size' => 128),
            'custom_css' => array('type' => self::TYPE_STRING, 'validate' => 'isCleanHtml', 'size' => 255),
            'background_position' => array('type' => self::TYPE_STRING, 'size' => 50),
            'background_stretch' => array('type' => self::TYPE_STRING, 'size' => 50),
        )
    );

    public function __construct($id_menu = null, $id_lang = null, $id_shop = null, Context $context = null)
    {
        parent::__construct($id_menu, $id_lang, $id_shop);
    }

    public function add($autodate = true, $null_values = false)
    {
        $context = Context::getContext();
        $id_shop = $context->shop->id;

        $res = parent::add($autodate, $null_values);
        $res &= Db::getInstance()->execute(
            '
			INSERT INTO `'._DB_PREFIX_.'dfxmenu_shop` (`id_shop`, `id_dfxmenu`)
			VALUES('.(int)$id_shop.', '.(int)$this->id.')'
        );
        return $res;
    }

    public function delete()
    {
        $res = true;

        $res &= Db::getInstance()->execute(
            '
			DELETE FROM `'._DB_PREFIX_.'dfxmenu_shop`
			WHERE `id_dfxmenu` = '.(int)$this->id
        );

        $res &= parent::delete();
        return $res;
    }
    
    public function rePositions($id_menu)
    {
        return Db::getInstance(_PS_USE_SQL_SLAVE_)->getRow('
			SELECT ms.`position`
			FROM '._DB_PREFIX_.'dfxmenu ms
			WHERE ms.`id_dfxmenu` = '.(int)$id_menu);
    }

    public static function getAssociatedIdsShop($id_menu)
    {
        $result = Db::getInstance(_PS_USE_SQL_SLAVE_)->executeS(
            '
			SELECT hs.`id_shop`
			FROM `'._DB_PREFIX_.'dfxmenu_shop` hs
			WHERE hs.`id_dfxmenu` = '.(int)$id_menu
        );

        if (!is_array($result)) {
            return false;
        }

        $return = array();

        foreach ($result as $id_shop) {
            $return[] = (int)$id_shop['id_shop'];
        }

        return $return;
    }
}
