<?php
/**
* Copyright since 2007 PrestaShop SA and Contributors
* PrestaShop is an International Registered Trademark & Property of PrestaShop SA
*
* NOTICE OF LICENSE
*
* This source file is subject to the Academic Free License 3.0 (AFL-3.0)
* that is bundled with this package in the file LICENSE.md.
* It is also available through the world-wide-web at this URL:
* https://opensource.org/licenses/AFL-3.0
* If you did not receive a copy of the license and are unable to
* obtain it through the world-wide-web, please send an email
* to license@prestashop.com so we can send you a copy immediately.
*
* DISCLAIMER
*
* Do not edit or add to this file if you wish to upgrade PrestaShop to newer
* versions in the future. If you wish to customize PrestaShop for your
* needs please refer to https://devdocs.prestashop.com/ for more information.
*
* @author    PrestaShop SA and Contributors <contact@prestashop.com>
* @copyright Since 2007 PrestaShop SA and Contributors
* @license   https://opensource.org/licenses/AFL-3.0 Academic Free License 3.0 (AFL-3.0)
*/

if (!defined('_PS_VERSION_')) {
    exit;
}

use PrestaShop\PrestaShop\Adapter\Image\ImageRetriever;
use PrestaShop\PrestaShop\Adapter\Product\PriceFormatter;
use PrestaShop\PrestaShop\Core\Product\ProductListingPresenter;
use PrestaShop\PrestaShop\Adapter\Product\ProductColorsRetriever;
use PrestaShop\PrestaShop\Adapter\Category\CategoryProductSearchProvider;
use PrestaShop\PrestaShop\Core\Product\Search\ProductSearchContext;
use PrestaShop\PrestaShop\Core\Product\Search\ProductSearchQuery;
use PrestaShop\PrestaShop\Core\Product\Search\SortOrder;
use PrestaShop\PrestaShop\Core\Module\WidgetInterface;

include_once(_PS_MODULE_DIR_.'dfxmegamenu/classes/DfxmenuClass.php');
include_once(_PS_MODULE_DIR_.'dfxmegamenu/classes/DfxmenuRow.php');
include_once(_PS_MODULE_DIR_.'dfxmegamenu/classes/DfxmenuColumn.php');

class DfxMegamenu extends Module implements WidgetInterface
{
    public $menu_img_folder = 'views/img/';
    public $_html = '';
    public $currentIndex;
    public $spacer_size = '5';
    public $pattern = '/^([A-Z_]*)[0-9]+/';
    public $page_name = '';
	public $confirmUninstall;
	private $templateFile;
    
    public function __construct()
    {
        $this->name = 'dfxmegamenu';
        $this->tab = 'front_office_features';
        $this->version = '6.1.1';
        $this->author = 'Digitalredfox';
        $this->need_instance = 0;
        $this->secure_key = Tools::encrypt($this->name);
        $this->bootstrap = true;

        parent::__construct();

        $this->displayName = $this->trans('Dfx Megamenu', array(), 'Modules.DfxMegamenu.Admin');
        $this->description = $this->trans('Adds an Own Menu to your site.', array(), 'Modules.DfxMegamenu.Admin');
        $this->ps_versions_compliancy = array('min' => '1.7.1.0', 'max' => _PS_VERSION_);
        
		$this->confirmUninstall = $this->trans('Are you sure you want to uninstall?', array(), 'Modules.DfxMegamenu.Admin');
        $this->currentIndex = AdminController::$currentIndex . '&configure=' . $this->name;
    }

    /**
     * @see Module::install()
     */
    public function install()
    {
        /* Adds Module */
        if (parent::install() &&
		   $this->registerHook('addproduct') &&
           $this->registerHook('updateproduct') &&
           $this->registerHook('deleteproduct') &&
           $this->registerHook('actionCategoryAdd') &&
           $this->registerHook('actionCategoryUpdate') &&
           $this->registerHook('actionCategoryDelete') &&
		   $this->registerHook('updateOrderStatus') &&
           $this->registerHook('actionObjectManufacturerAddAfter') &&
           $this->registerHook('actionObjectManufacturerUpdateAfter') &&
           $this->registerHook('actionObjectManufacturerDeleteAfter') &&
           $this->registerHook('displayHeader') &&
           $this->registerHook('actionShopDataDuplication') &&
           $this->registerHook('displayNavFullWidth') &&
           $this->registerHook('displayMegamenuMobile')
        ) {
            /* Creates tables */
            $res = $this->createTables();

            /* Adds samples */
            if ($res) {
                $this->installSamples();
            }

            return (bool)$res;
        }

        return false;
    }

    /**
     * Adds samples
     */
    public function installSamples()
    {
        $res = 1;
        include_once dirname(__FILE__).'/sql/sample.php';
        return $res;
    }

    /**
     * @see Module::uninstall()
     */
    public function uninstall()
    {
        /* Deletes Module */
        if (parent::uninstall()) {
            /* Deletes tables */
            $res = $this->deleteTables();
            return (bool)$res;
        }

        return false;
    }

    /**
     * Creates tables
     */
    public function createTables()
    {
        /* Menu Shop */
        $res = (bool)Db::getInstance()->execute('
            CREATE TABLE IF NOT EXISTS `'._DB_PREFIX_.'dfxmenu_shop` (
                `id_dfxmenu` int(10) unsigned NOT NULL AUTO_INCREMENT,
                `id_shop` int(10) unsigned NOT NULL,
                PRIMARY KEY (`id_dfxmenu`, `id_shop`)
            ) ENGINE='._MYSQL_ENGINE_.' DEFAULT CHARSET=UTF8;
        ');
        
        /* Menu configuration */
        $res &= Db::getInstance()->execute('
            CREATE TABLE IF NOT EXISTS `'._DB_PREFIX_.'dfxmenu` (
              `id_dfxmenu` int(10) unsigned NOT NULL AUTO_INCREMENT,
              `position` int(10) unsigned NOT NULL DEFAULT \'0\',
              `active` tinyint(1) unsigned NOT NULL DEFAULT \'0\',
			  `menu_type` varchar(50) NOT NULL,
			  `cmslink` int(11) DEFAULT NULL,
              `manufacturerlink` int(11) DEFAULT NULL,
              `supplierlink` int(11) DEFAULT NULL,
              `categorieslink` int(11) DEFAULT NULL,
			  `dropwidth` varchar(255) NOT NULL,
			  `custom_css` varchar(255) NOT NULL,
			  `menu_image` varchar(128) DEFAULT NULL,
			  `menu_image_w` varchar(255) NOT NULL,
	  		  `menu_image_h` varchar(255) NOT NULL,
			  `new_window` tinyint(1) NOT NULL,
			  `background_position` varchar(50) DEFAULT NULL,
			  `background_stretch` varchar(50) DEFAULT NULL,
              PRIMARY KEY (`id_dfxmenu`)
            ) ENGINE='._MYSQL_ENGINE_.' DEFAULT CHARSET=UTF8;
        ');
        
        /* Menu lang configuration */
        $res &= Db::getInstance()->execute('
            CREATE TABLE IF NOT EXISTS `'._DB_PREFIX_.'dfxmenu_lang` (
              `id_dfxmenu` int(10) unsigned NOT NULL,
              `id_lang` int(10) unsigned NOT NULL,
              `title` varchar(255) NOT NULL,
              `url` varchar(255) NOT NULL,
              PRIMARY KEY (`id_dfxmenu`,`id_lang`)
            ) ENGINE='._MYSQL_ENGINE_.' DEFAULT CHARSET=UTF8;
        ');
        
        $res &= Db::getInstance()->execute('
            CREATE TABLE IF NOT EXISTS `'._DB_PREFIX_.'dfxmenurow` (
			`id_dfxmenurow` int(10) unsigned NOT NULL AUTO_INCREMENT,
            `id_dfxmenu` int(10) unsigned NOT NULL,
            `active` tinyint(1) unsigned NOT NULL DEFAULT \'0\',
            PRIMARY KEY (`id_dfxmenurow`)
            ) ENGINE='._MYSQL_ENGINE_.' DEFAULT CHARSET=UTF8;
        ');
        
        $res &= Db::getInstance()->execute('
            CREATE TABLE IF NOT EXISTS `'._DB_PREFIX_.'dfxmenurow_shop` (
			`id_dfxmenurow` int(10) unsigned NOT NULL,
              `id_dfxmenu` int(10) unsigned NOT NULL,
              `id_shop` int(10) unsigned NOT NULL,
			  `active` tinyint(1) unsigned NOT NULL DEFAULT \'0\',
              PRIMARY KEY (`id_dfxmenurow`,`id_shop`)
            ) ENGINE='._MYSQL_ENGINE_.' DEFAULT CHARSET=UTF8;
        ');
        
        $res &= Db::getInstance()->execute('
            CREATE TABLE IF NOT EXISTS `'._DB_PREFIX_.'dfxmenucolumn` (
              `id_dfxmenucolumn` int(10) unsigned NOT NULL AUTO_INCREMENT,
			  `id_dfxmenurow` int(10) unsigned NOT NULL,
			  `width` varchar(255) NOT NULL,
			  `position` int(10) unsigned NOT NULL DEFAULT \'0\',
			  `active` tinyint(1) unsigned NOT NULL DEFAULT \'0\',
			  `content_type` varchar(50) NOT NULL,
			  `category_list` varchar(50) NOT NULL,
			  `cat_thumb` tinyint(1) unsigned NOT NULL DEFAULT \'0\',
			  `col_title` tinyint(1) unsigned NOT NULL DEFAULT \'0\',
			  `banner_image` varchar(128) DEFAULT NULL,
			  `banner_image_w` varchar(255) NOT NULL,
			  `banner_image_h` varchar(255) NOT NULL,
			  `products` text DEFAULT NULL,
			  `manufacturers` varchar(50) NOT NULL,
			  `cmslinks` varchar(50) NOT NULL,
			  `prod_count` varchar(255) NOT NULL,
			  `short_product` varchar(50) NOT NULL,
              PRIMARY KEY (`id_dfxmenucolumn`)
            ) ENGINE='._MYSQL_ENGINE_.' DEFAULT CHARSET=UTF8;
        ');
        
        $res &= Db::getInstance()->execute('
            CREATE TABLE IF NOT EXISTS `'._DB_PREFIX_.'dfxmenucolumn_shop` (
             `id_dfxmenucolumn` int(10) unsigned NOT NULL,
	  		 `id_dfxmenurow` int(10) unsigned NOT NULL,
	  		 `id_shop` int(10) unsigned NOT NULL,
			 `active` tinyint(1) unsigned NOT NULL DEFAULT \'0\',
              PRIMARY KEY (`id_dfxmenucolumn`,`id_shop`)
            ) ENGINE='._MYSQL_ENGINE_.' DEFAULT CHARSET=UTF8;
        ');
        
        $res &= Db::getInstance()->execute('
            CREATE TABLE IF NOT EXISTS `'._DB_PREFIX_.'dfxmenucolumn_lang` (
             `id_dfxmenucolumn` int(10) unsigned NOT NULL,
			 `id_dfxmenurow` int(10) unsigned NOT NULL,
	  		 `id_lang` int(10) unsigned NOT NULL,
			 `static_content` text DEFAULT NULL,
			 `title` varchar(255) NOT NULL,
             `url` varchar(255) NOT NULL,
			 `banner_url` varchar(255) NOT NULL,
             PRIMARY KEY (`id_dfxmenucolumn`,`id_lang`)
            ) ENGINE='._MYSQL_ENGINE_.' DEFAULT CHARSET=UTF8;
        ');

        return $res;
    }

    /**
     * deletes tables
     */
    public function deleteTables()
    {
        $menus = $this->getMenus();
        foreach ($menus as $menu) {
            $to_del = new DfxmenuClass($menu['id_dfxmenu']);
            $to_del->delete();
        }

        return Db::getInstance()->execute('
            DROP TABLE IF EXISTS `'._DB_PREFIX_.'dfxmenu_shop`, `'._DB_PREFIX_.'dfxmenu`, `'._DB_PREFIX_.'dfxmenu_lang`, `'._DB_PREFIX_.'dfxmenurow`, 
			`'._DB_PREFIX_.'dfxmenurow_shop`, `'._DB_PREFIX_.'dfxmenucolumn`, `'._DB_PREFIX_.'dfxmenucolumn_shop`, `'._DB_PREFIX_.'dfxmenucolumn_lang`;
        ');
    }

    public function getContent()
    {
        $this->context->controller->addJS($this->_path.'views/js/back.js');
        $this->context->controller->addCSS($this->_path.'views/css/back.css');
        
        /* Validate & process */
        if (Tools::isSubmit('submitMenu') || Tools::isSubmit('delete_id_menu') ||
            Tools::isSubmit('changeStatus')
        ) {
            if ($this->_postValidation()) {
                $this->_postProcess();
                $this->_html .= $this->renderList();
            } else {
                $this->_html .= $this->renderMenuForm();
            }
            $this->_clearCache('*');
        } elseif ((Tools::isSubmit('submitRow') || Tools::isSubmit('delete_col') || Tools::isSubmit('delete_row')) && Tools::isSubmit('id_dfxmenu') ||
        Tools::isSubmit('changeStatusRow') || Tools::isSubmit('deleteMenuImage')) {
            $this->_postProcess();
            $this->_html .= $this->renderMenuForm();

            $this->_clearCache('*');
        } elseif (Tools::isSubmit('submitCol') || Tools::isSubmit('changeStatusColumn')|| Tools::isSubmit('deleteBannerImage')) {
            if ($this->_postValidation()) {
                $this->_postProcess();
                $this->_html .= $this->renderMenuForm();
            } else {
                $this->_html .= $this->renderAddColumn();
            }
            $this->_clearCache('*');
        } elseif (Tools::isSubmit('addCol') || Tools::isSubmit('id_dfxmenucolumn')) {
            $this->_html .= $this->renderAddColumn();
            $this->_clearCache('*');
        } elseif (Tools::isSubmit('addRow') || Tools::isSubmit('id_dfxmenurow')) {
            $this->_postProcess();
            $this->_html .= $this->renderMenuForm();
            $this->_clearCache('*');
        } elseif (Tools::isSubmit('addMenu') || (Tools::isSubmit('id_dfxmenu') && $this->menuExists((int)Tools::getValue('id_dfxmenu')))) {
            if (Tools::isSubmit('addMenu')) {
                $mode = 'add';
            } else {
                $mode = 'edit';
            }

            if ($mode == 'add') {
                if (Shop::getContext() != Shop::CONTEXT_GROUP && Shop::getContext() != Shop::CONTEXT_ALL) {
                    $this->_html .= $this->renderMenuForm();
                } else {
                    $this->_html .= $this->getShopContextError(null, $mode);
                }
            } else {
                $associated_shop_ids = DfxmenuClass::getAssociatedIdsShop((int)Tools::getValue('id_dfxmenu'));
                $context_shop_id = (int)Shop::getContextShopID();

                if ($associated_shop_ids === false) {
                    $this->_html .= $this->getShopAssociationError((int)Tools::getValue('id_dfxmenu'));
                } elseif (Shop::getContext() != Shop::CONTEXT_GROUP && Shop::getContext() != Shop::CONTEXT_ALL && in_array($context_shop_id, $associated_shop_ids)) {
                    if (count($associated_shop_ids) > 1) {
                        $this->_html = $this->getSharedMenuWarning();
                    }
                    $this->_html .= $this->renderMenuForm();
                } else {
                    $shops_name_list = array();
                    foreach ($associated_shop_ids as $shop_id) {
                        $associated_shop = new Shop((int)$shop_id);
                        $shops_name_list[] = $associated_shop->name;
                    }
                    $this->_html .= $this->getShopContextError($shops_name_list, $mode);
                }
            }
        } else {
            if (Shop::getContext() != Shop::CONTEXT_GROUP && Shop::getContext() != Shop::CONTEXT_ALL) {
                $this->_html .= $this->renderList();
            }
            $this->_html .= $this->getWarningMultishopHtml().$this->getCurrentShopInfoMsg();
        }

        return $this->_html;
    }

    public function _postValidation()
    {
        $errors = array();

        /* Validation for Slider configuration */
        if (Tools::isSubmit('changeStatus')) {
            if (!Validate::isInt(Tools::getValue('id_dfxmenu'))) {
                $errors[] = $this->trans('Invalid menu', array(), 'Modules.DfxMegamenu.Admin');
            }
        } elseif (Tools::isSubmit('submitMenu')) {
            /* Checks state (active) */
            if (!Validate::isInt(Tools::getValue('active_menu')) || (Tools::getValue('active_menu') != 0 && Tools::getValue('active_menu') != 1)) {
                $errors[] = $this->trans('Invalid menu state.', array(), 'Modules.DfxMegamenu.Admin');
            }
            /* Checks position */
            if (!Validate::isInt(Tools::getValue('position')) || (Tools::getValue('position') < 0)) {
                $errors[] = $this->trans('Invalid menu position.', array(), 'Modules.DfxMegamenu.Admin');
            }
            /* If edit : checks id_dfxmenu */
            if (Tools::isSubmit('id_dfxmenu')) {
                if (!Validate::isInt(Tools::getValue('id_dfxmenu')) && !$this->menuExists(Tools::getValue('id_dfxmenu'))) {
                    $errors[] = $this->trans('Invalid menu ID', array(), 'Modules.DfxMegamenu.Admin');
                }
            }
            /* Checks title/url/legend/description/image */
            $languages = Language::getLanguages(false);
            foreach ($languages as $language) {
                if (Tools::strlen(Tools::getValue('title_' . $language['id_lang'])) > 255) {
                    $errors[] = $this->trans('The title is too long.', array(), 'Modules.DfxMegamenu.Admin');
                }
                if (Tools::strlen(Tools::getValue('url_' . $language['id_lang'])) > 255) {
                    $errors[] = $this->trans('The URL is too long.', array(), 'Modules.DfxMegamenu.Admin');
                }
                if (Tools::strlen(Tools::getValue('url_' . $language['id_lang'])) > 0 && !Validate::isUrl(Tools::getValue('url_' . $language['id_lang']))) {
                    $errors[] = $this->trans('The URL format is not correct.', array(), 'Modules.DfxMegamenu.Admin');
                }
            }

            /* Checks title/url/legend/description for default lang */
            $id_lang_default = (int)Configuration::get('PS_LANG_DEFAULT');
            if (Tools::strlen(Tools::getValue('title_' . $id_lang_default)) == 0) {
                $errors[] = $this->trans('The Title is not set.', array(), 'Modules.DfxMegamenu.Admin');
            }
            
			switch(Tools::getValue('menu_type'))
            {
                case 'dfxcustom':
                   if (Tools::strlen(Tools::getValue('url_' . $id_lang_default)) == 0) {
                     $errors[] = $this->trans('The URL is not set.', array(), 'Modules.DfxMegamenu.Admin');
                   }
                    break;
                case 'dfxcms':
                    if(!(int)Tools::getValue('cmslink'))
                      $errors[] = $this->trans('CMS link is not set.', array(), 'Modules.DfxMegamenu.Admin');
                    break;
                case 'dfxcategory':
                    if(!(int)Tools::getValue('categorieslink'))
                      $errors[] = $this->trans('Category link is not set.', array(), 'Modules.DfxMegamenu.Admin');
                    break;
                case 'dfxmanufacturer':
                    if(!(int)Tools::getValue('manufacturerlink'))
                       $errors[] = $this->trans('Manufacturer link is not set.', array(), 'Modules.DfxMegamenu.Admin');
                    break;
                case 'dfxsupplier':
                    if(!(int)Tools::getValue('supplierlink'))
                      $errors[] = $this->trans('Supplier link is not set.', array(), 'Modules.DfxMegamenu.Admin');
                    break;
                case 'dfxcontact':
                    break;
                case 'dfxhome':
                    break;
                default:
                    $errors[] = $this->trans('link type is not set.', array(), 'Modules.DfxMegamenu.Admin');
                    break;
            }
        } elseif (Tools::isSubmit('submitCol')) {
            /* Checks title/url/legend/description/image */
            if (!Validate::isInt(Tools::getValue('active_column')) || (Tools::getValue('active_column') != 0 && Tools::getValue('active_column') != 1)) {
                $errors[] = $this->trans('Invalid menu state.', array(), 'Modules.DfxMegamenu.Admin');
            }
            $languages = Language::getLanguages(false);
            foreach ($languages as $language) {
                if (Tools::strlen(Tools::getValue('title_' . $language['id_lang'])) > 255) {
                    $errors[] = $this->trans('The title is too long.', array(), 'Modules.DfxMegamenu.Admin');
                }
                if (Tools::strlen(Tools::getValue('url_' . $language['id_lang'])) > 255) {
                    $errors[] = $this->trans('The URL is too long.', array(), 'Modules.DfxMegamenu.Admin');
                }
                if (Tools::strlen(Tools::getValue('url_' . $language['id_lang'])) > 0 && !Validate::isUrl(Tools::getValue('url_' . $language['id_lang']))) {
                    $errors[] = $this->trans('The URL format is not correct.', array(), 'Modules.DfxMegamenu.Admin');
                }
            }

            /* Checks title/url/legend/description for default lang */
            $id_lang_default = (int)Configuration::get('PS_LANG_DEFAULT');
            if (Tools::strlen(Tools::getValue('title_' . $id_lang_default)) == 0) {
                $errors[] = $this->trans('The Title is not set.', array(), 'Modules.DfxMegamenu.Admin');
            }
        } elseif (Tools::isSubmit('delete_col') && (!Validate::isInt(Tools::getValue('delete_col')))) {
            $errors[] = $this->trans('Invalid Column ID', array(), 'Modules.DfxMegamenu.Admin');
        } elseif (Tools::isSubmit('delete_id_menu') && (!Validate::isInt(Tools::getValue('delete_id_menu')) || !$this->menuExists((int)Tools::getValue('delete_id_menu')))) {
            $errors[] = $this->trans('Invalid menu ID', array(), 'Modules.DfxMegamenu.Admin');
        }

        /* Display errors if needed */
        if (count($errors)) {
            $this->_html .= $this->displayError(implode('<br />', $errors));

            return false;
        }

        /* Returns if validation is ok */

        return true;
    }

    public function _postProcess()
    {
        $errors = array();
        $languages = Language::getLanguages(false);
        $idlang_default = (int)Configuration::get('PS_LANG_DEFAULT');
        /* Processes Slider */
        if (Tools::isSubmit('submitMenu')) {
            /* Sets ID if needed */
            if (Tools::getValue('id_dfxmenu')) {
                $menu = new DfxmenuClass((int)Tools::getValue('id_dfxmenu'));
                    
                $position_menu = $menu->rePositions((int)Tools::getValue('id_dfxmenu'));
                $menu->position = $position_menu['position'];
                    
                if (!Validate::isLoadedObject($menu)) {
                    $this->_html .= $this->displayError($this->trans('Invalid menu ID', array(), 'Modules.DfxMegamenu.Admin'));
                    return false;
                }
            } else {
                $menu = new DfxmenuClass();
                $menu->position = (int)Tools::getValue('position', 0);
            }
			
            /* Sets active */
            $menu->active = (int)Tools::getValue('active_menu');
            $menu->new_window = (int)Tools::getValue('new_window');
            /* Sets each langue fields */
            $menu->background_position = Tools::getValue('background_position');
            $menu->background_stretch = Tools::getValue('background_stretch');
            $menu->dropwidth = Tools::getValue('dropwidth');
            $menu->custom_css = Tools::getValue('custom_css');
            
			
			$menu->menu_type = Tools::getValue('menu_type');
			$menu->categorieslink = (int)Tools::getValue('categorieslink');
			$menu->manufacturerlink = (int)Tools::getValue('manufacturerlink');
			$menu->supplierlink = (int)Tools::getValue('supplierlink');
			$menu->cmslink = (int)Tools::getValue('cmslink');

            /* Uploads image and sets slide */
            $type = Tools::strtolower(Tools::substr(strrchr($_FILES['menu_image']['name'], '.'), 1));
            $imagesize = @getimagesize($_FILES['menu_image']['tmp_name']);
            if (isset($_FILES['menu_image']) &&
                    isset($_FILES['menu_image']['tmp_name']) &&
                    !empty($_FILES['menu_image']['tmp_name']) &&
                    !empty($imagesize) &&
                    in_array(Tools::strtolower(Tools::substr(strrchr($imagesize['mime'], '/'), 1)), array( 'jpg', 'gif', 'jpeg', 'png')) &&
                    in_array($type, array('jpg', 'gif', 'jpeg', 'png'))
                ) {
                $temp_name = tempnam(_PS_TMP_IMG_DIR_, 'PS');
                $salt = sha1(microtime());
                if ($error = ImageManager::validateUpload($_FILES['menu_image'])) {
                    $errors[] = $error;
                } elseif (!$temp_name || !move_uploaded_file($_FILES['menu_image']['tmp_name'], $temp_name)) {
                    return false;
                } elseif (!ImageManager::resize($temp_name, dirname(__FILE__).'/views/img/'.$salt.'_'.$_FILES['menu_image']['name'], null, null, $type)) {
                    $errors[] = $this->displayError($this->trans('An error occurred during the image upload process.', array(), 'Admin.Notifications.Error'));
                }
                if (isset($temp_name)) {
                    @unlink($temp_name);
                }
                
                $menu->menu_image = $salt.'_'.$_FILES['menu_image']['name'];
            } elseif (Tools::getValue('menu_image_old') != '') {
                $menu->menu_image = Tools::getValue('menu_image_old');
            }
                
            $menu->menu_image_w = $imagesize['0'];
            $menu->menu_image_h = $imagesize['1'];
                
            foreach ($languages as $language) {
                $menu->title[$language['id_lang']] = Tools::getValue('title_'.$language['id_lang']);
                $menu->url[$language['id_lang']] = Tools::getValue('url_'.$language['id_lang']);
            }

            /* Processes if no errors  */
            if (!$errors) {
                /* Adds */
                if (!Tools::getValue('id_dfxmenu')) {
                    if (!$menu->add()) {
                        $errors[] = $this->displayError($this->trans('The menu could not be added.', array(), 'Modules.DfxMegamenu.Admin'));
                    }
                } elseif (!$menu->update()) {
                    $errors[] = $this->displayError($this->trans('The menu could not be updated.', array(), 'Modules.DfxMegamenu.Admin'));
                }
                $this->_clearCache('*');
            }
            
            $this->generateCss();
            Tools::redirectAdmin($this->context->link->getAdminLink('AdminModules', true).'&conf=4&configure='.$this->name.'&tab_module='.$this->tab.'&module_name='.$this->name.'&id_dfxmenu='.$menu->id);
        } elseif (Tools::isSubmit('changeStatus') && Tools::isSubmit('id_dfxmenu')) {
            $menu = new DfxmenuClass((int)Tools::getValue('id_dfxmenu'));
            if ($menu->active == 0) {
                $menu->active = 1;
            } else {
                $menu->active = 0;
            }
            if ($menu->new_window == 0) {
                $menu->new_window = 1;
            } else {
                $menu->new_window = 0;
            }
            
            $res = $menu->update();

            $this->_clearCache('*');
            $this->_html .= ($res ? $this->displayConfirmation($this->trans('Configuration updated', array(), 'Admin.Notifications.Success')) : $this->displayError($this->trans('The configuration could not be updated.', array(), 'Modules.DfxMegamenu.Admin')));
        } elseif (Tools::isSubmit('delete_id_menu')) {
            $menu = new DfxmenuClass((int)Tools::getValue('delete_id_menu'));
            $res = $menu->delete();
            $this->_clearCache('*');
            if (!$res) {
                $this->_html .= $this->displayError('Could not delete.');
            } else {
                Tools::redirectAdmin($this->context->link->getAdminLink('AdminModules', true) . '&conf=1&configure=' . $this->name . '&tab_module=' . $this->tab . '&module_name=' . $this->name);
            }
        } elseif (Tools::isSubmit('addRow')) {
            $id_dfxmenu = (int) Tools::getValue('id_dfxmenu');
            $id_dfxmenurow = (int) Tools::getValue(1);
            $row_item = new DfxmenuRow($id_dfxmenu);
            if ($id_dfxmenurow) {
                $row_item = new DfxmenuRow($id_dfxmenu, $id_dfxmenurow);
            }
                
                
            $row_item->id_dfxmenu = (int)Tools::getValue('id_dfxmenu');
            $row_item->active = (int)Tools::getValue('active_row', 1);
            $row_item->position = (int)Tools::getValue('position');
            if (!$errors) {
                if (!Tools::getValue('id_dfxmenurow')) {
                    if (!$row_item->add()) {
                        $errors[] = $this->displayError($this->trans('The row_item could not be added.', array(), 'Modules.DfxMegamenu.Admin'));
                    }
                } elseif (!$row_item->update()) {
                    $errors[] = $this->displayError($this->trans('The row_item could not be updated.', array(), 'Modules.DfxMegamenu.Admin'));
                }
                $this->_clearCache('*');
            }
            Tools::redirectAdmin($this->context->link->getAdminLink('AdminModules', true).'&conf=4&configure='.$this->name.'&tab_module='.$this->tab.'&module_name='.$this->name.'&id_dfxmenu='.Tools::getValue('id_dfxmenu'));
        } elseif (Tools::isSubmit('changeStatusRow') && Tools::isSubmit('id_dfxmenurow')) {
            $id_dfxmenu = (int) Tools::getValue('id_dfxmenu');
            $id_dfxmenurow = (int) Tools::getValue('id_dfxmenurow');
            $row = new DfxmenuRow($id_dfxmenu);
            if ($id_dfxmenurow) {
                $row = new DfxmenuRow($id_dfxmenu, $id_dfxmenurow);
            }
            
            if ($row->active == 0) {
                $row->active = 1;
            } else {
                $row->active = 0;
            }
            
            $res = $row->update();
            
            $this->_clearCache('*');
            $this->_html .= ($res ? $this->displayConfirmation($this->trans('Configuration updated')) : $this->displayError($this->trans('The configuration could not be updated.')));
            Tools::redirectAdmin($this->context->link->getAdminLink('AdminModules', true).'&conf=4&configure='.$this->name.'&tab_module='.$this->tab.'&module_name='.$this->name.'&id_dfxmenu='.Tools::getValue('id_dfxmenu'));
        } elseif (Tools::isSubmit('delete_row')) {
            $id_dfxmenu = (int) Tools::getValue('id_dfxmenu');
            $id_dfxmenurow = (int) Tools::getValue('id_dfxmenurow');
            $row_item = new DfxmenuRow($id_dfxmenu);
            if ($id_dfxmenurow) {
                $row_item = new DfxmenuRow($id_dfxmenu, $id_dfxmenurow);
            }
                
            $res = $row_item->delete();
            $this->_clearCache('*');
            if (!$res) {
                $this->_html .= $this->displayError('Could not delete.');
            } else {
                Tools::redirectAdmin($this->context->link->getAdminLink('AdminModules', true).'&conf=2&configure='.$this->name.'&tab_module='.$this->tab.'&module_name='.$this->name.'&id_dfxmenu='.Tools::getValue('id_dfxmenu'));
            }
        } elseif (Tools::isSubmit('submitCol')) {
            $id_dfxmenurow = (int) Tools::getValue('id_dfxmenurow');
            $id_dfxmenucolumn = (int) Tools::getValue('id_dfxmenucolumn');
            $column_block = new DfxmenuColumn($id_dfxmenurow);
            if ($id_dfxmenucolumn) {
                $column_block = new DfxmenuColumn($id_dfxmenurow, $id_dfxmenucolumn);
            }

            $column_block->id_dfxmenurow = $id_dfxmenurow;
            $column_block->position = (int)Tools::getValue('position');
            $column_block->active = (int)Tools::getValue('active_column');

            $column_block->prod_count = Tools::getValue('prod_count');
            $column_block->cat_thumb = (int)Tools::getValue('category_thumb');
            $column_block->col_title = (int)Tools::getValue('col_title');
            $column_block->width = Tools::getValue('width');
            $column_block->content_type = Tools::getValue('content_type');
            $column_block->category_list = Tools::getValue('categorylist_js');
            
            $column_block->products = Tools::getValue('customproducts_js');
        
            $column_block->manufacturers = Tools::getValue('manufacturers_js');
            $column_block->cmslinks = Tools::getValue('cmslink_js');
            $column_block->short_product = Tools::getValue('short_product');
                
            $id_lang_default = (int)Configuration::get('PS_LANG_DEFAULT');
            $static_content = array();
                
            foreach ($languages as $lang) {
                $column_block->title[$lang['id_lang']] = Tools::getValue('title_'.$lang['id_lang']);
                    
                $column_block->url[$lang['id_lang']] = Tools::getValue('url_'.$lang['id_lang']);
            
                    
                $column_block->banner_url[$lang['id_lang']] = Tools::getValue('banner_url_'.$lang['id_lang']);
                    
                
                $static_content[$lang['id_lang']] = Tools::getValue('static_content_' . $lang['id_lang']);
                if (!$static_content[$lang['id_lang']]) {
                    $static_content[$lang['id_lang']] = Tools::getValue('static_content_' . $id_lang_default);
                }
            }
                
            $column_block->static_content = $static_content;

            /* Uploads image and sets slide */
            $type = Tools::strtolower(Tools::substr(strrchr($_FILES['banner_image']['name'], '.'), 1));
            $imagesize = @getimagesize($_FILES['banner_image']['tmp_name']);
            if (isset($_FILES['banner_image']) &&
                    isset($_FILES['banner_image']['tmp_name']) &&
                    !empty($_FILES['banner_image']['tmp_name']) &&
                    !empty($imagesize) &&
                    in_array(
                        Tools::strtolower(Tools::substr(strrchr($imagesize['mime'], '/'), 1)),
                        array(
                            'jpg',
                            'gif',
                            'jpeg',
                            'png'
                        )
                    ) &&
                    in_array($type, array('jpg', 'gif', 'jpeg', 'png'))
                ) {
                $temp_name = tempnam(_PS_TMP_IMG_DIR_, 'PS');
                $salt = sha1(microtime());
                if ($error = ImageManager::validateUpload($_FILES['banner_image'])) {
                    $errors[] = $error;
                } elseif (!$temp_name || !move_uploaded_file($_FILES['banner_image']['tmp_name'], $temp_name)) {
                    return false;
                } elseif (!ImageManager::resize($temp_name, dirname(__FILE__).'/views/img/'.$salt.'_'.$_FILES['banner_image']['name'], null, null, $type)) {
                    $errors[] = $this->displayError($this->trans('An error occurred during the image upload process.', array(), 'Admin.Notifications.Error'));
                }
                if (isset($temp_name)) {
                    @unlink($temp_name);
                }
                $column_block->banner_image = $salt.'_'.$_FILES['banner_image']['name'];
            } elseif (Tools::getValue('banner_image') != '') {
                $column_block->banner_image = Tools::getValue('banner_image_old');
            }
                
            $column_block->banner_image_w = $imagesize['0'];
            $column_block->banner_image_h = $imagesize['1'];
                    
            $id_dfxmenu = (int)Tools::getValue('id_dfxmenu');

            if (!$errors) {
                if (!Tools::getValue('id_dfxmenucolumn')) {
                    if (!$column_block->add()) {
                        $errors[] = $this->displayError($this->trans('The column_block could not be added.'));
                    }
                } elseif (!$column_block->update()) {
                    $errors[] = $this->displayError($this->trans('The column_block could not be updated.'));
                }
                $this->_clearCache('*');
            }
            Tools::redirectAdmin($this->context->link->getAdminLink('AdminModules', true).'&conf=4&configure='.$this->name.'&tab_module='.$this->tab.'&module_name='.$this->name.'&id_dfxmenu='.Tools::getValue('id_dfxmenu'));
        } elseif (Tools::isSubmit('changeStatusColumn') && Tools::isSubmit('id_dfxmenucolumn')) {
            $id_dfxmenurow = (int) Tools::getValue('id_dfxmenurow');
            $id_dfxmenucolumn = (int) Tools::getValue('id_dfxmenucolumn');
            $column_block = new DfxmenuColumn($id_dfxmenurow);
            if ($id_dfxmenucolumn) {
                $column_block = new DfxmenuColumn($id_dfxmenurow, $id_dfxmenucolumn);
            }
            
            if ($column_block->active == 0) {
                $column_block->active = 1;
            } else {
                $column_block->active = 0;
            }
            
            $res = $column_block->update();
            
            $this->_clearCache('*');
            $this->_html .= ($res ? $this->displayConfirmation($this->trans('Configuration updated')) : $this->displayError($this->trans('The configuration could not be updated.')));
            Tools::redirectAdmin($this->context->link->getAdminLink('AdminModules', true).'&conf=4&configure='.$this->name.'&tab_module='.$this->tab.'&module_name='.$this->name.'&id_dfxmenu='.Tools::getValue('id_dfxmenu'));
        } elseif (Tools::isSubmit('deleteMenuImage')) {
            $id_menu = (int)Tools::getValue('id_dfxmenu');

            $menu = new DfxmenuClass($id_menu);
            if ($menu->menu_image) {
                $image_path = $this->local_path.$this->menu_img_folder.$menu->menu_image;

                if (file_exists($image_path)) {
                    unlink($image_path);
                }

                $menu->menu_image = null;
                $menu->menu_image_w = '150';
                $menu->menu_image_h = '150';
                $menu->update(false);
                $this->_clearCache('*');
                Tools::redirectAdmin($this->context->link->getAdminLink('AdminModules', true).'&conf=7&configure='.$this->name.'&tab_module='.$this->tab.'&module_name='.$this->name.'&id_dfxmenu='.$id_menu);
            }
        } elseif (Tools::isSubmit('deleteBannerImage')) {
            $id_menu = (int)Tools::getValue('id_dfxmenu');
            $id_dfxmenurow = (int) Tools::getValue('id_dfxmenurow');
            $id_dfxmenucolumn = (int) Tools::getValue('id_dfxmenucolumn');
            $column_block = new DfxmenuColumn($id_dfxmenurow);
            if ($id_dfxmenucolumn) {
                $column_block = new DfxmenuColumn($id_dfxmenurow, $id_dfxmenucolumn);
            }
            
            if ($column_block->banner_image) {
                $image_bannerpath = $this->local_path.$this->menu_img_folder.$column_block->banner_image;

                if (file_exists($image_bannerpath)) {
                    unlink($image_bannerpath);
                }

                $column_block->banner_image = null;
                $column_block->update();
                $this->_clearCache('*');
                Tools::redirectAdmin($this->context->link->getAdminLink('AdminModules', true).'&conf=2&configure='.$this->name.'&tab_module='.$this->tab.'&module_name='.$this->name.'&id_dfxmenu='.$id_menu.'&id_dfxmenurow='.$id_dfxmenurow.'&id_dfxmenucolumn='.$id_dfxmenucolumn);
            }
        } elseif (Tools::isSubmit('delete_col')) {
            $id_dfxmenurow = (int) Tools::getValue('id_dfxmenurow');
            $id_dfxmenucolumn = (int) Tools::getValue('id_dfxmenucolumn');
            $column_block = new DfxmenuColumn($id_dfxmenurow);
            if ($id_dfxmenucolumn) {
                $column_block = new DfxmenuColumn($id_dfxmenurow, $id_dfxmenucolumn);
            }

            $res = $column_block->delete();
            $this->_clearCache('*');
            if (!$res) {
                $this->_html .= $this->displayError('Could not delete.');
            } else {
                Tools::redirectAdmin($this->context->link->getAdminLink('AdminModules', true).'&conf=2&configure='.$this->name.'&tab_module='.$this->tab.'&module_name='.$this->name.'&id_dfxmenu='.Tools::getValue('id_dfxmenu'));
            }
        }
        

        /* Display errors if needed */
        if (count($errors)) {
            $this->_html .= $this->displayError(implode('<br />', $errors));
        } elseif (Tools::isSubmit('submitMenu') && Tools::getValue('id_dfxmenu')) {
            Tools::redirectAdmin($this->context->link->getAdminLink('AdminModules', true) . '&conf=4&configure=' . $this->name . '&tab_module=' . $this->tab . '&module_name=' . $this->name);
        } elseif (Tools::isSubmit('submitMenu')) {
            Tools::redirectAdmin($this->context->link->getAdminLink('AdminModules', true) . '&conf=3&configure=' . $this->name . '&tab_module=' . $this->tab . '&module_name=' . $this->name);
        } elseif (Tools::isSubmit('submitRow') && Tools::getValue('id_dfxmenurow')) {
            Tools::redirectAdmin($this->context->link->getAdminLink('AdminModules', true) . '&conf=4&configure=' . $this->name . '&tab_module=' . $this->tab . '&module_name=' . $this->name);
        } elseif (Tools::isSubmit('submitRow')) {
            Tools::redirectAdmin($this->context->link->getAdminLink('AdminModules', true) . '&conf=3&configure=' . $this->name . '&tab_module=' . $this->tab . '&module_name=' . $this->name);
        } elseif (Tools::isSubmit('submitCol') && Tools::getValue('id_dfxmenucolumn')) {
            Tools::redirectAdmin($this->context->link->getAdminLink('AdminModules', true) . '&conf=4&configure=' . $this->name . '&tab_module=' . $this->tab . '&module_name=' . $this->name);
        } elseif (Tools::isSubmit('submitCol')) {
            Tools::redirectAdmin($this->context->link->getAdminLink('AdminModules', true) . '&conf=3&configure=' . $this->name . '&tab_module=' . $this->tab . '&module_name=' . $this->name);
        }
    }
    
    public function hookDisplayHeader($params)
    {
        // js files for front page
        $this->context->controller->addCSS($this->_path.'views/css/front.css');
		$display_rtl = $this->context->language->is_rtl;

		if($display_rtl) {
			$this->context->controller->addCSS($this->_path.'views/css/menucustom_rtl.css');
		} else {
			$this->context->controller->addCSS($this->_path.'views/css/menucustom.css');
		}
    }

    public function hookActionShopDataDuplication($params)
    {
        Db::getInstance()->execute(
            '
            INSERT IGNORE INTO '._DB_PREFIX_.'dfxmenu_shop (id_dfxmenu, id_shop)
            SELECT id_dfxmenu, '.(int)$params['new_id_shop'].'
            FROM '._DB_PREFIX_.'dfxmenu_shop
            WHERE id_shop = '.(int)$params['old_id_shop']
        );
        
        Db::getInstance()->execute(
            '
			INSERT IGNORE INTO '._DB_PREFIX_.'dfxmenurow_shop (`id_dfxmenurow`, `id_dfxmenu`, `id_shop`)
			SELECT id_dfxmenurow, id_dfxmenu, '.(int)$params['new_id_shop'].'
			FROM '._DB_PREFIX_.'dfxmenurow_shop
			WHERE id_shop = '.(int)$params['old_id_shop']
        );
        
        Db::getInstance()->execute(
            '
			INSERT IGNORE INTO '._DB_PREFIX_.'dfxmenucolumn_shop (`id_dfxmenucolumn`, `id_dfxmenurow`, `id_shop`)
			SELECT id_dfxmenucolumn, id_dfxmenurow, '.(int)$params['new_id_shop'].'
			FROM '._DB_PREFIX_.'dfxmenucolumn_shop
			WHERE id_shop = '.(int)$params['old_id_shop']
        );
        
        $this->_clearCache('*');
    }

    

    public function getMenus($active = null)
    {
        $this->context = Context::getContext();
        $id_shop = $this->context->shop->id;
        $id_lang = $this->context->language->id;

        $menus = Db::getInstance(_PS_USE_SQL_SLAVE_)->executeS(
            '
            SELECT hs.`id_dfxmenu` as id_dfxmenu, hss.*, hssl.`title`,
            hssl.`url` FROM '._DB_PREFIX_.'dfxmenu_shop hs
            LEFT JOIN '._DB_PREFIX_.'dfxmenu hss ON (hs.id_dfxmenu = hss.id_dfxmenu)
            LEFT JOIN '._DB_PREFIX_.'dfxmenu_lang hssl ON (hss.id_dfxmenu = hssl.id_dfxmenu)
            WHERE id_shop = '.(int)$id_shop.'
            AND hssl.id_lang = '.(int)$id_lang.
            ($active ? ' AND hss.`active` = 1' : ' ').'
            ORDER BY hss.position ASC, hss.id_dfxmenu ASC'
        );
        if($menus) {
           foreach ($menus as &$menu) {
			  $menu['url'] = $this->getMenuLink($menu);
              $menu['image_url'] = $this->_path . $this->menu_img_folder . $menu['menu_image'];
           }
		}

        return $menus;
    }
	public function getMenuLink($menu)
    {
        if(isset($menu['menu_type']))
        {
            switch($menu['menu_type'])
            {
                case 'dfxcustom':
                    return $menu['url'];
				case 'dfxcontact':
                    return $this->context->link->getPageLink('contact', true);
                case 'dfxcms':
                    return $this->context->link->getCMSLink((int)$menu['cmslink']);
                case 'dfxcategory':
                    return $this->context->link->getCategoryLink((int)$menu['categorieslink']);
                case 'dfxmanufacturer':
                    $manufacturer = new Manufacturer((int)$menu['manufacturerlink'], (int)$this->context->language->id);
                    if(!is_null($manufacturer->id))
                    {
                        if ((int)Configuration::get('PS_REWRITING_SETTINGS'))
    						$manufacturer->link_rewrite = Tools::link_rewrite($manufacturer->name);
    					else
    						$manufacturer->link_rewrite = 0;
                        return $this->context->link->getManufacturerLink((int)$menu['manufacturerlink'], $manufacturer->link_rewrite);
                    }
                    return '#';
                case 'dfxsupplier':
                    $supplier= new Supplier((int)$menu['supplierlink'],(int)$this->context->language->id);
                    if(!is_null($supplier->id))
                    {
                        return $this->context->link->getSupplierLink($supplier->id);
                    }
                    return '#';
                case 'dfxhome':
                    return $this->context->link->getPageLink('index', true);
            }
        }
        return '#';
    }

    public function displayStatus($id_menu, $active)
    {
        $title = ((int)$active == 0 ? $this->trans('Disabled', array(), 'Admin.Global') : $this->trans('Enabled', array(), 'Admin.Global'));
        $icon = ((int)$active == 0 ? 'icon-remove' : 'icon-check');
        $class = ((int)$active == 0 ? 'btn-danger' : 'btn-success');
        $html = '<a class="btn '.$class.'" href="'.AdminController::$currentIndex.
            '&configure='.$this->name.
                '&token='.Tools::getAdminTokenLite('AdminModules').
                '&changeStatus&id_dfxmenu='.(int)$id_menu.'" title="'.$title.'"><i class="'.$icon.'"></i> '.$title.'</a>';

        return $html;
    }
    
    public function displayStatusRow($id_menu, $id_dfxmenurow, $active)
    {
        $title = ((int)$active == 0 ? $this->trans('Disabled') : $this->trans('Enabled'));
        $icon = ((int)$active == 0 ? 'icon-remove' : 'icon-check');
        $class = ((int)$active == 0 ? 'btn-danger' : 'btn-success');
        
        $html = '<a class="btn '.$class.'" href="'.AdminController::$currentIndex.
            '&configure='.$this->name.
                '&token='.Tools::getAdminTokenLite('AdminModules').
                '&changeStatusRow&id_dfxmenu='.(int)$id_menu.'&id_dfxmenurow='.(int)$id_dfxmenurow.'" title="'.$title.'"><i class="'.$icon.'"></i> '.$title.'</a>';

        return $html;
    }
    
    public function displayStatusColumn($id_dfxmenurow, $id_dfxmenucolumn, $active)
    {
        $id_menu = (int)Tools::getValue('id_dfxmenu');
        $title = ((int)$active == 0 ? $this->trans('Disabled') : $this->trans('Enabled'));
        $icon = ((int)$active == 0 ? 'icon-remove' : 'icon-check');
        $class = ((int)$active == 0 ? 'btn-danger' : 'btn-success');
        
        $html = '<a class="btn '.$class.'" href="'.AdminController::$currentIndex.
            '&configure='.$this->name.
                '&token='.Tools::getAdminTokenLite('AdminModules').
                '&changeStatusColumn&id_dfxmenu='.(int)$id_menu.'&id_dfxmenurow='.(int)$id_dfxmenurow.'&id_dfxmenucolumn='.(int)$id_dfxmenucolumn.'" title="'.$title.'"><i class="'.$icon.'"></i> '.$title.'</a>';

        return $html;
    }

    public function menuExists($id_menu)
    {
        $req = 'SELECT hs.`id_dfxmenu` as id_dfxmenu
                FROM `'._DB_PREFIX_.'dfxmenu_shop` hs
                WHERE hs.`id_dfxmenu` = '.(int)$id_menu;
        $row = Db::getInstance(_PS_USE_SQL_SLAVE_)->getRow($req);

        return ($row);
    }

    public function renderList()
    {
        $this->context->controller->addJqueryUI('ui.sortable');
        $menus = $this->getMenus();
        foreach ($menus as $key => $menu) {
            $menus[$key]['status'] = $this->displayStatus($menu['id_dfxmenu'], $menu['active']);
            $associated_shop_ids = DfxmenuClass::getAssociatedIdsShop((int)$menu['id_dfxmenu']);
            if ($associated_shop_ids && count($associated_shop_ids) > 1) {
                $menus[$key]['is_shared'] = true;
            } else {
                $menus[$key]['is_shared'] = false;
            }
        }

        $this->context->smarty->assign(
            array(
                'link' => $this->context->link,
                'menus' => $menus,
                'url_base' => $this->context->shop->physical_uri.$this->context->shop->virtual_uri,
                'secure_key' => $this->secure_key
            )
        );

        return $this->display(__FILE__, 'views/templates/admin/list.tpl');
    }

    public function renderMenuForm()
    {
		$id_lang = $this->context->language->id;
        $id_menu = (int)Tools::getValue('id_dfxmenu');
        $menu = new DfxmenuClass($id_menu);
        /* Image content */
        $image_url = false;
        $image_size = false;
        if ($id_menu) {
            if ($menu->menu_image) {
                $image_url = $this->_path . $this->menu_img_folder . $menu->menu_image;
                $image_size = filesize($this->local_path . $this->menu_img_folder . $menu->menu_image) / 1000;
            }
        }

        /* Choose the Category for Dropdown */
        $background_position = array(
            'query' => array(
                array('id' => 'left top', 'name' => $this->trans('Left Top')),
                array('id' => 'left center', 'name' => $this->trans('Left Center')),
                array('id' => 'left bottom', 'name' => $this->trans('Left Bottom')),
                array('id' => 'center top', 'name' => $this->trans('Center Top')),
                array('id' => 'center center', 'name' => $this->trans('Center Center')),
                array('id' => 'center bottom', 'name' => $this->trans('Center Bottom')),
                array('id' => 'right top', 'name' => $this->trans('Right Top')),
                array('id' => 'right center', 'name' => $this->trans('Right Center')),
                array('id' => 'right bottom', 'name' => $this->trans('Right Bottom')),
            ),
            'id' => 'id',
            'name' => 'name',
        );

        $background_stretch = array(
            'query' => array(
                array('id' => 'auto', 'name' => $this->trans('auto')),
                array('id' => 'contain', 'name' => $this->trans('contain')),
                array('id' => 'cover', 'name' => $this->trans('cover')),
                array('id' => 'inherit', 'name' => $this->trans('inherit')),
                array('id' => 'initial', 'name' => $this->trans('initial')),
                array('id' => 'revert', 'name' => $this->trans('revert')),
                array('id' => 'unset', 'name' => $this->trans('unset')),
                array('id' => $menu->menu_image_w.'px '.$menu->menu_image_h.'px', 'name' => $this->trans($menu->menu_image_w.'px '.$menu->menu_image_h.'px')),
            ),
            'id' => 'id',
            'name' => 'name',
        );

        $fields_form = array(
            'form' => array(
                'legend' => array(
                    'title' => $this->trans('Menu information', array(), 'Modules.DfxMegamenu.Admin'),
                    'icon' => 'icon-cogs'
                ),
                'input' => array(
                    array(
                        'type' => 'hidden',
                        'name' => 'id_dfxmenu',
                    ),
					array(
					'type' => 'select',
					'label' => $this->trans('Menu link type'),
					'name' => 'menu_type',
                    'id' => 'dfxmenu_type',
					'options' => array(
            			 'query' => array(
                                array(
                                    'id_option' => 'dfxcustom',
                                    'name' => $this->trans('Custom link')
                                ),
								array(
                                    'id_option' => 'dfxcontact',
                                    'name' => $this->trans('Contact us link')
                                ),
                                array(
                                    'id_option' => 'dfxcms',
                                    'name' => $this->trans('CMS page')
                                ),
                                array(
                                    'id_option' => 'dfxcategory',
                                    'name' => $this->trans('Category')
                                ),
                                array(
                                    'id_option' => 'dfxmanufacturer',
                                    'name' => $this->trans('Manufacturer')
                                ),
                                array(
                                    'id_option' => 'dfxsupplier',
                                    'name' => $this->trans('Supplier')
                                ),
                                array(
                                    'id_option' => 'dfxhome',
                                    'name' => $this->trans('Home')
                                )
                            ),
                         'id' => 'id_option',
            			 'name' => 'name'
                    ),
                    'default' => 'dfxcustom',
				),
                array(
                     'type' => 'text',
                     'label' => $this->trans('Title', array(), 'Modules.DfxMegamenu.Admin'),
                     'name' => 'title',
                     'required' => true,
                     'lang' => true,
                ),
                array(
                     'type' => 'text',
                     'label' => $this->trans('Target URL', array(), 'Modules.DfxMegamenu.Admin'),
                     'name' => 'url',
                     'lang' => true,
					 'form_group_class' => 'dfxcustom_link',
                ),
				array(
                    'label' => $this->trans('Link to Manufacturer'),
                    'type' => 'select',
                    'options' => array(
					  'query' => Manufacturer::getManufacturers(false, $this->context->language->id),
				      'id' => 'id_manufacturer',
					  'name' => 'name'
					 ),
					'name' => 'manufacturerlink',
					'form_group_class' => 'dfxmanufacturer_link',
                ),
                array(
                    'label' => $this->trans('Link to Supplier'),
                    'type' => 'select',
					'name' => 'supplierlink',
					'options' => array(
					  'query' => Supplier::getSuppliers((int)$this->context->language->id),
				      'id' => 'id_supplier',
					  'name' => 'name'
					 ),
					'form_group_class' => 'dfxsupplier_link',
                ),
               array(
					'type'  => 'selectcategory',
					'label' => $this->trans('Link to Category'),
					'name'  => 'categorieslink',
					'form_group_class' => 'dfxcategory_link',
				),
                array(
                    'label' => $this->trans('Link to CMS page'),
                    'type' => 'select',
					'name'  => 'cmslink',
					'options' => array(
					  'query' => CMS::getLinks((int)$this->context->language->id),
				      'id' => 'id_cms',
					  'name' => 'meta_title'
					 ),

					'form_group_class' => 'dfxcms_link',
                ),
                    array(
                        'type' => 'file',
                        'label' => $this->trans('Background Image for Dropdown', array(), 'Modules.DfxMegamenu.Admin'),
                        'name' => 'menu_image',
                        'desc' => $this->trans('Maximum image size: %s.', array(ini_get('upload_max_filesize')), 'Admin.Global'),
                        'display_image' => true,
                        'image' => $image_url ? '<img src="' . $image_url . '" alt="" class="img-thumbnail" style="max-width:410px;" />' : false,
                        'size' => $image_size,
                        'delete_url' => $this->currentIndex . '&token=' . Tools::getAdminTokenLite('AdminModules') . '&deleteMenuImage&id_dfxmenu=' . $id_menu,
                    ),
                    array(
                        'type' => 'select',
                        'label' => $this->trans('Background Position'),
                        'name' => 'background_position',
                        'id' => 'background_position_selectbox',
                        'options' => $background_position,
                        'hint' => $this->trans('Dropdown Image Positions.'),
                        'form_group_class' => 'menu_image_properties',
                    ),
                    array(
                        'type' => 'select',
                        'label' => $this->trans('Background Stretch'),
                        'name' => 'background_stretch',
                        'id' => 'background_stretch_selectbox',
                        'options' => $background_stretch,
                        'hint' => $this->trans('Dropdown Image Positions.'),
                        'form_group_class' => 'menu_image_properties',
                    ),
                    array(
                        'type' => 'text',
                        'label' => $this->trans('Dropdown width'),
                        'name' => 'dropwidth',
                        'default' => '100%',
                        'desc' => $this->trans('Use "px" or "%" or "vw". Eg: "100%" or "100px" or "80vw" '),
                    ),
                    array(
                        'type' => 'textarea',
                        'label' => $this->trans('Custom Css', array(), 'Modules.DfxMegamenu.Admin'),
                        'name' => 'custom_css',
                        'rows' => 5,
                        'cols' => 25,
                    ),
                    array(
                        'type' => 'switch',
                        'label' => $this->trans('Enabled', array(), 'Admin.Global'),
                        'name' => 'active_menu',
                        'is_bool' => true,
                        'values' => array(
                            array(
                                'id' => 'active_on',
                                'value' => 1,
                                'label' => $this->trans('Yes', array(), 'Admin.Global')
                            ),
                            array(
                                'id' => 'active_off',
                                'value' => 0,
                                'label' => $this->trans('No', array(), 'Admin.Global')
                            )
                        ),
                    ),
                    array(
                        'type' => 'switch',
                        'label' => $this->trans('New window', array(), 'Admin.Global'),
                        'name' => 'new_window',
                        'is_bool' => true,
                        'values' => array(
                            array(
                                'id' => 'active_on',
                                'value' => 1,
                                'label' => $this->trans('Yes', array(), 'Admin.Global')
                            ),
                            array(
                                'id' => 'active_off',
                                'value' => 0,
                                'label' => $this->trans('No', array(), 'Admin.Global')
                            )
                        ),
                    ),
                ),
                'submit' => array(
                    'title' => $this->trans('Save', array(), 'Admin.Actions'),
                ),
                'buttons' => array(
                    array(
                        'href' => $this->currentIndex . '&token=' . Tools::getAdminTokenLite('AdminModules'),
                        'title' => $this->trans('Back'),
                        'icon' => 'process-icon-back',
                    ),
                ),
            ),
        );
        $menucatlink = '';
        if (Tools::isSubmit('id_dfxmenu') && $this->menuExists((int)Tools::getValue('id_dfxmenu'))) {
            $menu = new DfxmenuClass((int)Tools::getValue('id_dfxmenu'));
            $fields_form['form']['input'][] = array('type' => 'hidden', 'name' => 'id_dfxmenu');
			$menucatlink = $menu->categorieslink;
        }
        
        $helper = new HelperForm();
        $helper->show_toolbar = false;
        $helper->table = $this->table;
        $lang = new Language((int)Configuration::get('PS_LANG_DEFAULT'));
        $helper->default_form_language = $lang->id;
        $helper->allow_employee_form_lang = Configuration::get('PS_BO_ALLOW_EMPLOYEE_FORM_LANG') ? Configuration::get('PS_BO_ALLOW_EMPLOYEE_FORM_LANG') : 0;
        $this->fields_form = array();
        $helper->module = $this;
        $helper->identifier = $this->identifier;
        $helper->submit_action = 'submitMenu';
        $helper->currentIndex = $this->context->link->getAdminLink('AdminModules', false).'&configure='.$this->name.'&tab_module='.$this->tab.'&module_name='.$this->name;
        $helper->token = Tools::getAdminTokenLite('AdminModules');
        $language = new Language((int)Configuration::get('PS_LANG_DEFAULT'));
        $helper->tpl_vars = array(
            'base_url' => $this->context->shop->getBaseURL(),
            'language' => array(
                'id_lang' => $language->id,
                'iso_code' => $language->iso_code
            ),
            'fields_value' => $this->getAddFieldsValues(),
			'cat_options' => $this->getCategoryOption(1, (int)$id_lang, (int)Shop::getContextShopID(), 'general'),
            'languages' => $this->context->controller->getLanguages(),
            'id_language' => $this->context->language->id,
			'menucatlink' => $menucatlink,
        );
         
        $helper->override_folder = '/';
    
        return $helper->generateForm(array($fields_form)).$this->renderBuildMenu();
    }

    public function getAddFieldsValues()
    {
        $fields = array();
        $id_menu = (int) Tools::getValue('id_dfxmenu');
       
        $menu = new DfxmenuClass($id_menu);
       
        $fields['id_dfxmenu'] = $id_menu;
        $fields['dropwidth'] = Tools::getValue('dropwidth', $menu->dropwidth);
        $fields['custom_css'] = Tools::getValue('custom_css', $menu->custom_css);
        $fields['active_menu'] = Tools::getValue('active_menu', $menu->active);
        $fields['new_window'] = Tools::getValue('new_window', $menu->new_window);
        $fields['background_position'] = Tools::getValue('background_position', $menu->background_position);
        $fields['background_stretch'] = Tools::getValue('background_stretch', $menu->background_stretch);
		
		$fields['menu_type'] = Tools::getValue('menu_type', $menu->menu_type);
        $fields['categorieslink'] = Tools::getValue('categorieslink', $menu->categorieslink);
		$fields['manufacturerlink'] = Tools::getValue('manufacturerlink', $menu->manufacturerlink);
		$fields['supplierlink'] = Tools::getValue('supplierlink', $menu->supplierlink);
		$fields['cmslink'] = Tools::getValue('cmslink', $menu->cmslink);
    
        $languages = Language::getLanguages(false);

        foreach ($languages as $lang) {
			$default_title = isset($menu->title[$lang['id_lang']]) ? $menu->title[$lang['id_lang']] : '';
            $fields['title'][$lang['id_lang']] = Tools::getValue('title_'.(int)$lang['id_lang'], $default_title);
			
			$default_url = isset($menu->url[$lang['id_lang']]) ? $menu->url[$lang['id_lang']] : '';
            $fields['url'][$lang['id_lang']] = Tools::getValue('url_'.(int)$lang['id_lang'], $default_url);
        }
                
        $fields['menu_image'] = Tools::getValue('menu_image', $menu->menu_image);
        
        return $fields;
    }
	
    public function renderAddColumn()
    {
        $id_menu = (int)Tools::getValue('id_dfxmenu');
        $id_lang = (int)$this->context->language->id;

        $id_dfxmenurow = (int) Tools::getValue('id_dfxmenurow');
        $id_dfxmenucolumn = (int) Tools::getValue('id_dfxmenucolumn');
        $dfxcolumn = new DfxmenuColumn($id_dfxmenurow);
        if ($id_dfxmenucolumn) {
            $dfxcolumn = new DfxmenuColumn($id_dfxmenurow, $id_dfxmenucolumn);
        }
            
            
        $colwidth = array(
            'query' => array(
                array('id' => '1', 'name' => $this->trans('1/12')),
                array('id' => '2', 'name' => $this->trans('2/12')),
                array('id' => '3', 'name' => $this->trans('3/12')),
                array('id' => '4', 'name' => $this->trans('4/12')),
                array('id' => '5', 'name' => $this->trans('5/12')),
                array('id' => '6', 'name' => $this->trans('6/12')),
                array('id' => '7', 'name' => $this->trans('7/12')),
                array('id' => '8', 'name' => $this->trans('8/12')),
                array('id' => '9', 'name' => $this->trans('9/12')),
                array('id' => '10', 'name' => $this->trans('10/12')),
                array('id' => '11', 'name' => $this->trans('11/12')),
                array('id' => '12', 'name' => $this->trans('12/12')),
            ),
            'id' => 'id',
            'name' => 'name',
        );
        /* Choose the Category for Dropdown */
        $content_list = array(
            'query' => array(
                array('id' => 'none', 'name' => ''),
                array('id' => 'category', 'name' => $this->trans('Category')),
                array('id' => 'product', 'name' => $this->trans('Product')),
                array('id' => 'html', 'name' => $this->trans('Custom HTML')),
                array('id' => 'manufacturer', 'name' => $this->trans('Manufacturer')),
                 array('id' => 'cms_link', 'name' => $this->trans('CMS Links')),
                 array('id' => 'bannerimage', 'name' => $this->trans('Banner Image')),
            ),
            'id' => 'id',
            'name' => 'name',
        );
        $shortproduct = array(
            'query' => array(
                array('id' => 'short', 'name' => $this->trans('Short Product')),
                array('id' => 'normal', 'name' => $this->trans('Normal Product')),
            ),
            'id' => 'id',
            'name' => 'name',
        );
        
        /* Custom Products */
        $prodList = array();
        $customproducts = array();
        $selectedproduct = array();
        $productObj = new Product();
        $res_all_prod = $productObj->getProducts($id_lang, 0, 0, 'id_product', 'ASC');

        foreach ($res_all_prod as $prod) {
            $prodList[$prod['id_product']] = $prod['name'];
        }

        /* Banner Image */
        /* Image content */
        $image_url = false;
        $image_size = false;
        if ($id_dfxmenucolumn) {
            if ($dfxcolumn->banner_image) {
                $image_url = $this->_path . $this->menu_img_folder . $dfxcolumn->banner_image;
                $image_size = filesize($this->local_path . $this->menu_img_folder . $dfxcolumn->banner_image) / 1000;
            }
        }
    
        $fields_form = array(
            'form' => array(
                'legend' => array(
                    'title' => $this->trans('Column'),
                    'icon' => 'icon-cogs'
                ),
                'input' => array(
                    array(
                        'type' => 'hidden',
                        'name' => 'id_dfxmenucolumn',
                    ),
                    array(
                        'type' => 'select',
                        'label' => $this->trans('Content Type', array(), 'Modules.DfxMegamenu.Admin'),
                        'name' => 'content_type',
                        'id' => 'content_list_selection',
                        'options' => $content_list,
                        'hint' => $this->trans('Dropdown Content.', array(), 'Modules.DfxMegamenu.Admin'),
                    ),
                    array(
                        'type' => 'text',
                        'label' => $this->trans('Title', array(), 'Admin.Global'),
                        'name' => 'title',
                        'lang' => true,
                        'required' => true,
                    ),
                    array(
                        'type' => 'text',
                        'label' => $this->trans('Target URL', array(), 'Modules.DfxMegamenu.Admin'),
                        'name' => 'url',
                        'lang' => true,
                    ),
                    array(
                         'type' => 'link_choice',
                         'label' => $this->trans('Select the Parent Categories', array(), 'Modules.DfxMegamenu.Admin'),
                         'name' => 'link',
                         'lang' => true,
                         'form_group_class' => 'content_type_category',
                        ),
                    array(
                        'type' => 'products',
                        'label' => $this->trans('Select the Products', array(), 'Modules.DfxMegamenu.Admin'),
                        'name' => 'products',
                        'multiple' => 'multiple',
                        'form_group_class' => 'content_type_product',
                    ),
                    array(
                        'type' => 'textarea',
                        'label' => $this->trans('Custom HTML Content', array(), 'Modules.DfxMegamenu.Admin'),
                        'name' => 'static_content',
                        'autoload_rte' => true,
                        'lang' => true,
                        'rows' => 10,
                        'cols' => 100,
                        'form_group_class' => 'content_type_html',
                    ),
                    array(
                        'type' => 'file',
                        'label' => $this->trans('Banner Image', array(), 'Modules.DfxMegamenu.Admin'),
                        'name' => 'banner_image',
                        'desc' => $this->trans('Maximum image size: %s.', array(ini_get('upload_max_filesize')), 'Admin.Global'),
                        'display_image' => true,
                        'image' => $image_url ? '<img src="' . $image_url . '" alt="" class="img-thumbnail" style="max-width:410px;" />' : false,
                        'size' => $image_size,
                        'delete_url' => $this->currentIndex . '&token=' . Tools::getAdminTokenLite('AdminModules') . '&deleteBannerImage&id_dfxmenu='.$id_menu.'&id_dfxmenurow='.$id_dfxmenurow.'&id_dfxmenucolumn='.$id_dfxmenucolumn,
                        'form_group_class' => 'content_type_image',
                    ),
                    array(
                        'type' => 'text',
                        'label' => $this->trans('Banner Link', array(), 'Modules.DfxMegamenu.Admin'),
                        'name' => 'banner_url',
                        'lang' => true,
                        'form_group_class' => 'content_type_image',
                    ),
                    array(
                        'type' => 'manufacturer',
                        'label' => $this->trans('Choose the Manufacturers', array(), 'Modules.DfxMegamenu.Admin'),
                        'name' => 'manufacturers',
                        'lang' => true,
                        'form_group_class' => 'content_type_manufacturer',
                    ),
                    array(
                        'type' => 'cmslinks',
                        'label' => $this->trans('Select the CMS', array(), 'Modules.DfxMegamenu.Admin'),
                        'name' => 'cmslinks',
                        'lang' => true,
                        'form_group_class' => 'content_type_cms',
                    ),
                    array(
                        'type' => 'select',
                        'label' => $this->trans('Dropdown width', array(), 'Modules.DfxMegamenu.Admin'),
                        'name' => 'width',
                        'id' => 'colwidth',
                        'options' => $colwidth,
                        'hint' => $this->trans('Select Dropdown width', array(), 'Modules.DfxMegamenu.Admin'),
                    ),
                    array(
                        'type' => 'select',
                        'label' => $this->trans('Short Product / Normal Product', array(), 'Modules.DfxMegamenu.Admin'),
                        'name' => 'short_product',
                        'id' => 'shortproduct',
                        'options' => $shortproduct,
                        'hint' => $this->trans('Select Product Style', array(), 'Modules.DfxMegamenu.Admin'),
                        'form_group_class' => 'content_type_product',
                    ),
                    array(
                        'type' => 'switch',
                        'label' => $this->trans('Column Title', array(), 'Modules.DfxMegamenu.Admin'),
                        'name' => 'col_title',
                        'is_bool' => true,
                        'values' => array(
                            array(
                                'id' => 'active_on',
                                'value' => 1,
                                'label' => $this->trans('Yes', array(), 'Modules.DfxMegamenu.Admin'),
                            ),
                            array(
                                'id' => 'active_off',
                                'value' => 0,
                                'label' => $this->trans('No', array(), 'Modules.DfxMegamenu.Admin'),
                            )
                        ),
                    ),
                    array(
                        'type' => 'switch',
                        'label' => $this->trans('Category Menu Thumb', array(), 'Modules.DfxMegamenu.Admin'),
                        'name' => 'category_thumb',
                        'is_bool' => true,
                        'values' => array(
                            array(
                                'id' => 'active_on',
                                'value' => 1,
                                'label' => $this->trans('Yes', array(), 'Modules.DfxMegamenu.Admin'),
                            ),
                            array(
                                'id' => 'active_off',
                                'value' => 0,
                                'label' => $this->trans('No', array(), 'Modules.DfxMegamenu.Admin'),
                            )
                        ),
                        'form_group_class' => 'content_type_category',
                    ),
                    array(
                        'type' => 'switch',
                        'label' => $this->trans('Display', array(), 'Modules.DfxMegamenu.Admin'),
                        'name' => 'active_column',
                        'is_bool' => true,
                        'values' => array(
                            array(
                                'id' => 'active_on',
                                'value' => 1,
                                'label' => $this->trans('Yes', array(), 'Modules.DfxMegamenu.Admin'),
                            ),
                            array(
                                'id' => 'active_off',
                                'value' => 0,
                                'label' => $this->trans('No', array(), 'Modules.DfxMegamenu.Admin'),
                            )
                        ),
                    ),
                ),
                'submit' => array(
                    'title' => $this->trans('Save', array(), 'Modules.DfxMegamenu.Admin'),
                    'class' => 'btn btn-default pull-right submitCol'
                ),
                'buttons' => array(
                    array(
                     'href' => $this->currentIndex . '&token=' . Tools::getAdminTokenLite('AdminModules').'&id_dfxmenu='.$id_menu,
                    'title' => $this->trans('Cancel', array(), 'Modules.DfxMegamenu.Admin'),
                    'icon' => 'process-icon-back'
                    )
                )
            ),
        );
        $selectedproduct = array();
        $id_dfxmenucolumn = (int)Tools::getValue('id_dfxmenucolumn');
        
        $id_dfxmenurow = (int) Tools::getValue('id_dfxmenurow');
        
            
        if (Tools::isSubmit('id_dfxmenucolumn')) {
            $columnclass = new DfxmenuColumn($id_dfxmenurow, $id_dfxmenucolumn);
            $fields_form['form']['input'][] = array('type' => 'hidden', 'name' => 'id_dfxmenucolumn');
            
            $customproducts = $columnclass->products;
        }
        
        
        if ($customproducts) {
            $selectedproduct[$id_dfxmenucolumn] = explode(',', $customproducts);
        }
        
        $fields_form['form']['input'][] = array('type' => 'hidden', 'name' => 'id_dfxmenurow');
        
        $helper = new HelperForm();
        $helper->show_toolbar = false;
        $helper->table = $this->table;
        $lang = new Language((int)Configuration::get('PS_LANG_DEFAULT'));
        $helper->default_form_language = $lang->id;
        $helper->allow_employee_form_lang = Configuration::get('PS_BO_ALLOW_EMPLOYEE_FORM_LANG') ? Configuration::get('PS_BO_ALLOW_EMPLOYEE_FORM_LANG') : 0;
        $this->fields_form = array();
        $helper->module = $this;
        $helper->identifier = $this->identifier;
        $helper->submit_action = 'submitCol';
        $helper->currentIndex = $this->context->link->getAdminLink('AdminModules', false).'&configure='.$this->name.'&tab_module='.$this->tab.'&module_name='.$this->name.
        '&id_dfxmenu='.$id_menu.'&id_dfxmenurow=' . $id_dfxmenurow;
        $helper->token = Tools::getAdminTokenLite('AdminModules');
        $language = new Language((int)Configuration::get('PS_LANG_DEFAULT'));
        $helper->tpl_vars = array(
            'base_url' => $this->context->shop->getBaseURL(),
            'language' => array(
                'id_lang' => $language->id,
                'iso_code' => $language->iso_code
            ),
            'fields_value' => $this->getAddFieldsValuesCol(),
            'languages' => $this->context->controller->getLanguages(),
            'id_language' => $this->context->language->id,
            'id_dfxmenucolumn' => $id_dfxmenucolumn,
            'choices' => $this->renderCategoriesSelect(),
            'selectedCategories' => $this->makeCategoryOption(),
            'selectedManufacturer' => $this->makeManufacturerOption(),
            'res_all_prod' => $res_all_prod,
            'prodList' => $prodList,
            'selectedproduct' => $selectedproduct,
            'manufacturer_choice' => $this->renderManufacturerSelect(),
            'cms_links' => $this->renderCMSSelect(),
            'selectedCMS' => $this->makeCmsOption(),
        );
        
        $helper->override_folder = '/';

        return $helper->generateForm(array($fields_form));
    }
    
    
    public function getAddFieldsValuesCol()
    {
        $fields = array();
        $languages = Language::getLanguages(false);
        
        
        $id_dfxmenurow = (int) Tools::getValue('id_dfxmenurow');
        $id_dfxmenucolumn = (int) Tools::getValue('id_dfxmenucolumn');
        $col = new DfxmenuColumn($id_dfxmenurow);
        if ($id_dfxmenucolumn) {
            $col = new DfxmenuColumn($id_dfxmenurow, $id_dfxmenucolumn);
            $fields['col_title'] = Tools::getValue('col_title', $col->col_title);
                
            $fields['width'] = Tools::getValue('width', $col->width);
            $fields['prod_count'] = Tools::getValue('prod_count', $col->prod_count);
            $fields['active_column'] = Tools::getValue('active_column', $col->active);
        } else {
            $fields['width'] = Tools::getValue('width', '3');
            $fields['col_title'] = Tools::getValue('col_title', 0);
            $fields['prod_count'] = Tools::getValue('prod_count', '8');
            $fields['active_column'] = Tools::getValue('active_column', 1);
        }
            
    
        $fields['id_dfxmenurow'] = $id_dfxmenurow;
        $fields['id_dfxmenucolumn'] = Tools::getValue('id_dfxmenucolumn', $col->id_dfxmenucolumn);
        
        $fields['content_type'] = Tools::getValue('content_type', $col->content_type);
        $fields['category_thumb'] = Tools::getValue('category_thumb', $col->cat_thumb);
        $fields['short_product'] = Tools::getValue('short_product', $col->short_product);
        $fields['categorylist_js'] = Tools::getValue('categorylist_js', array());
        $fields['customproducts_js'] = Tools::getValue('customproducts_js', array());
        $fields['manufacturers_js'] = Tools::getValue('manufacturers_js', array());
        $fields['cmslink_js'] = Tools::getValue('cmslink_js', array());

        
        foreach ($languages as $lang) {
			$default_title = isset($col->title[$lang['id_lang']]) ? $col->title[$lang['id_lang']] : '';
            $fields['title'][$lang['id_lang']] = Tools::getValue('title_'.(int)$lang['id_lang'], $default_title);
			$default_url = isset($col->url[$lang['id_lang']]) ? $col->url[$lang['id_lang']] : '';
            $fields['url'][$lang['id_lang']] = Tools::getValue('url_'.(int)$lang['id_lang'], $default_url);
			$default_banner_url = isset($col->banner_url[$lang['id_lang']]) ? $col->banner_url[$lang['id_lang']] : '';
            $fields['banner_url'][$lang['id_lang']] = Tools::getValue('banner_url_'.(int)$lang['id_lang'], $default_banner_url);
            
            $default_static_content = isset($col->static_content[$lang['id_lang']]) ? $col->static_content[$lang['id_lang']] : '';
            $fields['static_content'][$lang['id_lang']] = Tools::getValue('static_content_' . (int)$lang['id_lang'], $default_static_content);
        }
        
        $fields['banner_image'] = Tools::getValue('banner_image', $col->banner_image);
        
        return $fields;
    }
    public function selectedCategories()
    {
        $id_dfxmenurow = (int) Tools::getValue('id_dfxmenurow');
        $id_dfxmenucolumn = (int) Tools::getValue('id_dfxmenucolumn');
        $col = new DfxmenuColumn($id_dfxmenurow);
        if ($id_dfxmenucolumn) {
            $col = new DfxmenuColumn($id_dfxmenurow, $id_dfxmenucolumn);
        }
    
        $items = $col->category_list;

        if (Tools::strlen($items)) {
            return explode(',', $items);
        } else {
            return array();
        }
    }
    public function makeCategoryOption()
    {
        $html = '';
        $id_dfxmenucolumn = (int)Tools::getValue('id_dfxmenucolumn');
        
        $menu_item = $this->selectedCategories();
        
        $id_lang = $this->context->language->id;
        $html = '<select multiple="multiple" name="items_'.$id_dfxmenucolumn.'[]" id="items_'.$id_dfxmenucolumn.'" style="width: 300px; height: 160px;">';
        foreach ($menu_item as $item) {
            preg_match($this->pattern, $item, $values);
            $id = (int)Tools::substr($item, Tools::strlen($values[1]), Tools::strlen($item));
            
        
            $category = new Category((int)$id, (int)$id_lang);
            if (Validate::isLoadedObject($category)) {
                $html .= '<option selected="selected" value="'.$id.'">'.$category->name.'</option>'.PHP_EOL;
            }
        }
         
        return $html.'</select>';
    }
    public function renderCategoriesSelect()
    {
        $id_dfxmenucolumn = (int)Tools::getValue('id_dfxmenucolumn');
        $id_lang = $this->context->language->id;
        
        $html = '<select multiple="multiple" id="availableItems_'.$id_dfxmenucolumn.'" name="availableItems_'.$id_dfxmenucolumn.'" style="width: 300px; height: 160px;">';
     
        $html .= '<optgroup label="'.$this->trans('Categories', array(), 'Admin.Global').'">';
        $html .= $this->getCategoryOption(1, (int)$id_lang, (int)Shop::getContextShopID(), 'general');
       
        $html .= '</optgroup>';
        $html .= '</select>';
        return $html;
    }
    public function getCategoryOption($id_category = 1, $id_lang = false, $id_shop = false, $recursive = true, $link = false)
    {
        $html = '';
        $id_lang = $id_lang ? (int)$id_lang : (int)Context::getContext()->language->id;
        $id_shop = $id_shop ? (int)$id_shop : (int)Context::getContext()->shop->id;
        $category = new Category((int)$id_category, (int)$id_lang, (int)$id_shop);
        if (is_null($category->id)) {
            return;
        }
        if ($recursive) {
            $children = Category::getChildren((int)$id_category, (int)$id_lang, true, (int)$id_shop);
            $spacer = str_repeat('&nbsp;', 3 * (int)$category->level_depth);
        }

        if (!in_array($category->id, array(Configuration::get('PS_HOME_CATEGORY'), Configuration::get('PS_ROOT_CATEGORY')))) {
            $html .= '<option value="'.(int)$category->id.'">'.(isset($spacer) ? $spacer : '').$category->name.'</option>';

        }
        
        if (isset($children) && count($children)) {
            foreach ($children as $child) {
                $html .= $this->getCategoryOption(
                    (int)$child['id_category'],
                    (int)$id_lang,
                    (int)$child['id_shop'],
                    $recursive,
                    $link
                );
            }
        }
        return $html;
    }

    public function selectedManufacturer()
    {
        $id_dfxmenurow = (int) Tools::getValue('id_dfxmenurow');
        $id_dfxmenucolumn = (int) Tools::getValue('id_dfxmenucolumn');
        $col = new DfxmenuColumn($id_dfxmenurow);
        if ($id_dfxmenucolumn) {
            $col = new DfxmenuColumn($id_dfxmenurow, $id_dfxmenucolumn);
        }
        
        $items = $col->manufacturers;
        
        if (Tools::strlen($items)) {
            return explode(',', $items);
        } else {
            return array();
        }
    }
    public function makeManufacturerOption()
    {
        $html = '';
        $id_dfxmenucolumn = (int) Tools::getValue('id_dfxmenucolumn');
        
        $menu_item = $this->selectedManufacturer();
        
        $id_lang = $this->context->language->id;
        $html = '<select multiple="multiple" name="manufact_'.$id_dfxmenucolumn.'[]" id="manufact_'.$id_dfxmenucolumn.'" style="width: 300px; height: 160px;">';
        foreach ($menu_item as $item) {
            preg_match($this->pattern, $item, $values);
            $id = (int)Tools::substr($item, Tools::strlen($values[1]), Tools::strlen($item));
            
        
            $manufacturer = new Manufacturer((int)$id, (int)$id_lang);
            if (Validate::isLoadedObject($manufacturer)) {
                $html .= '<option selected="selected" value="'.$id.'">'.$manufacturer->name.'</option>'.PHP_EOL;
            }
        }
         
        return $html.'</select>';
    }
    public function renderManufacturerSelect()
    {
        $spacer = str_repeat('&nbsp;', $this->spacer_size);
        $id_dfxmenucolumn = (int)Tools::getValue('id_dfxmenucolumn');
        
        $html = '<select multiple="multiple" id="manufactureItems_'.$id_dfxmenucolumn.'" name="manufactureItems_'.$id_dfxmenucolumn.'" style="width: 300px; height: 160px;">';
        // BEGIN Manufacturer
        $html .= '<optgroup label="'.$this->trans('Brand', array(), 'Admin.Global').'">';
        // Option to show all Manufacturers
        $html .= '<option value="ALLMAN0">'.$this->trans('All brands', array(), 'Modules.DfxMegamenu.Admin').'</option>';
        $manufacturers = Manufacturer::getManufacturers(false, $this->context->language->id);
        foreach ($manufacturers as $manufacturer) {
            $html .= '<option value="'.$manufacturer['id_manufacturer'].'">'.$spacer.$manufacturer['name'].'</option>';
        }
        $html .= '</optgroup>';
        $html .= '</select>';
        return $html;
    }
    public function selectedCMSlinks()
    {
        $id_dfxmenurow = (int) Tools::getValue('id_dfxmenurow');
        $id_dfxmenucolumn = (int) Tools::getValue('id_dfxmenucolumn');
        $col = new DfxmenuColumn($id_dfxmenurow);
        if ($id_dfxmenucolumn) {
            $col = new DfxmenuColumn($id_dfxmenurow, $id_dfxmenucolumn);
        }

        $items = $col->cmslinks;
        
        if (Tools::strlen($items)) {
            return explode(',', $items);
        } else {
            return array();
        }
    }
    public function makeCmsOption()
    {
        $html = '';
        
        $id_dfxmenurow = (int) Tools::getValue('id_dfxmenurow');
        $id_dfxmenucolumn = (int) Tools::getValue('id_dfxmenucolumn');
        
        $menu_item = $this->selectedCMSlinks();
        
        $id_lang = $this->context->language->id;
        
        $html = '<select multiple="multiple" name="cmslink_'.$id_dfxmenucolumn.'[]" id="cmslink_'.$id_dfxmenucolumn.'" style="width: 300px; height: 160px;">';
        foreach ($menu_item as $item) {
            preg_match($this->pattern, $item, $values);
            $id = (int)Tools::substr($item, Tools::strlen($values[1]), Tools::strlen($item));
            
            $cms = new CMS((int)$id, (int)$id_lang);
            if (Validate::isLoadedObject($cms)) {
                $html .= '<option selected="selected" value="'.$id.'">'.$cms->meta_title.'</option>'.PHP_EOL;
            }
        }
         
        return $html.'</select>';
    }
    public function renderCMSSelect()
    {
        $spacer = str_repeat('&nbsp;', $this->spacer_size);
        $id_dfxmenucolumn = (int)Tools::getValue('id_dfxmenucolumn');
        $id_lang = $this->context->language->id;
        $html = '<select multiple="multiple" id="CMSItems_'.$id_dfxmenucolumn.'" name="CMSItems_'.$id_dfxmenucolumn.'" style="width: 300px; height: 160px;">';
        // BEGIN Manufacturer
        $html .= '<optgroup label="'.$this->trans('CMS Links', array(), 'Admin.Global').'">';
        // Option to show all Manufacturers
        $cmslinks = CMS::getLinks((int)$id_lang);
        if ($cmslinks) {
            foreach ($cmslinks as $cms) {
                $html .= '<option value="'.$cms['id_cms'].'">'.$spacer.$cms['meta_title'].'</option>';
            }
        }
        $html .= '</optgroup>';
        $html .= '</select>';
        return $html;
    }

    public function renderBuildMenu()
    {
        $this->context->controller->addJqueryUI('ui.sortable');
        $id_menu = (int) Tools::getValue('id_dfxmenu');
        $megamenu = new DfxmenuClass($id_menu);
        
        $row_lists = $this->getRowslist($id_menu);
        
        
        foreach ($row_lists as $key => $row_list) {
            $row_lists[$key]['status'] = $this->displayStatusRow($id_menu, $row_list['id_dfxmenurow'], $row_list['active']);
            $row_lists[$key]['list_col'] = $this->getColumns($row_list['id_dfxmenurow']);
        }

        foreach ($row_lists as $key => &$rowlist) {
            $menucolumns = $rowlist['list_col'];
            
            foreach ($menucolumns as &$menucolumn) {
                $menucolumn['status'] = $this->displayStatusColumn($rowlist['id_dfxmenurow'], $menucolumn['id_dfxmenucolumn'], $menucolumn['active']);
                $menucolumn = $this->getMenucolumnitems($menucolumn);
            }
            $rowlist['menucolumns'] = $menucolumns;
        }

        $this->context->smarty->assign(
            array(
                'id_dfxmenu' => $id_menu,
                'token' => Tools::getAdminTokenLite('AdminModules'),
                'dropwidth' => $megamenu->dropwidth,
                'row_lists' => $row_lists,
                'link' => $this->context->link,
                'url_base' => $this->context->shop->physical_uri.$this->context->shop->virtual_uri,
                'secure_key' => $this->secure_key,
                
            )
        );
        if ((int)$megamenu->id < 1) {
            return $this->display(__FILE__, 'views/templates/admin/build-message.tpl');
        } else {
            return $this->display(__FILE__, 'views/templates/admin/build_menu.tpl');
        }
    }
    
    public function getRowslist($id_menu, $active = null)
    {
        $id_shop = (int)$this->context->shop->id;
        $row_infos = array();
        $row_infos_rs = Db::getInstance(_PS_USE_SQL_SLAVE_)->executeS(
            '
			SELECT hs.`id_dfxmenurow` as id_dfxmenurow, hss.`active`
			FROM '._DB_PREFIX_.'dfxmenurow_shop hs
			LEFT JOIN '._DB_PREFIX_.'dfxmenurow hss ON (hs.id_dfxmenurow = hss.id_dfxmenurow)
			WHERE id_shop = '.$id_shop.' AND hss.id_dfxmenu = '.$id_menu.($active ? ' AND hss.`active` = 1' : '')
        );
        if (is_array($row_infos_rs) && count($row_infos_rs) > 0) {
            $row_infos = $row_infos_rs;
        }
        return $row_infos;
    }

    public function getColumns($id_dfxmenurow, $active = null)
    {
        $this->context = Context::getContext();
        $id_shop = (int)$this->context->shop->id;
        $id_lang = $this->context->language->id;
        
        $columns = Db::getInstance(_PS_USE_SQL_SLAVE_)->executeS(
            '
			SELECT hs.*, hss.*, hssl.*
            FROM '._DB_PREFIX_.'dfxmenucolumn_shop hs
            LEFT JOIN '._DB_PREFIX_.'dfxmenucolumn hss ON (hs.id_dfxmenucolumn = hss.id_dfxmenucolumn)
			LEFT JOIN '._DB_PREFIX_.'dfxmenucolumn_lang hssl ON (hss.id_dfxmenucolumn = hssl.id_dfxmenucolumn)
            WHERE id_shop = '.$id_shop.' AND hssl.id_lang = '.(int)$id_lang.' AND hss.id_dfxmenurow = '.$id_dfxmenurow.($active ? ' AND hss.`active` = 1' : '').'
            ORDER BY hss.position ASC, hss.id_dfxmenucolumn ASC'
        );
        
        foreach ($columns as &$column) {
            $column['url'] = $column['url'];
            $column['banner_url'] = $column['banner_url'];
        }
        return $columns;
    }
    
    public function getWarningMultishopHtml()
    {
        if (Shop::getContext() == Shop::CONTEXT_GROUP || Shop::getContext() == Shop::CONTEXT_ALL) {
            return '<p class="alert alert-warning">' .
            $this->trans('You cannot manage slides items from a "All Shops" or a "Group Shop" context, select directly the shop you want to edit', array(), 'Modules.DfxMegamenu.Admin') .
            '</p>';
        } else {
            return '';
        }
    }

    public function getShopContextError($shop_contextualized_name, $mode)
    {
        if (is_array($shop_contextualized_name)) {
            $shop_contextualized_name = implode('<br/>', $shop_contextualized_name);
        }

        if ($mode == 'edit') {
            return '<p class="alert alert-danger">' .
            $this->trans('You can only edit this menu from the shop(s) context: %s', array($shop_contextualized_name), 'Modules.DfxMegamenu.Admin') .
            '</p>';
        } else {
            return '<p class="alert alert-danger">' .
            $this->trans('You cannot add menus from a "All Shops" or a "Group Shop" context', array(), 'Modules.DfxMegamenu.Admin') .
            '</p>';
        }
    }

    public function getShopAssociationError($id_menu)
    {
        return '<p class="alert alert-danger">'.
                        $this->trans('Unable to get menu shop association information (id_menu: %d)', array((int)$id_menu), 'Modules.DfxMegamenu.Admin') .
                '</p>';
    }


    protected function getCurrentShopInfoMsg()
    {
        $shop_info = null;

        if (Shop::isFeatureActive()) {
            if (Shop::getContext() == Shop::CONTEXT_SHOP) {
                $shop_info = $this->trans('The modifications will be applied to shop: %s', array($this->context->shop->name), 'Modules.DfxMegamenu.Admin');
            } elseif (Shop::getContext() == Shop::CONTEXT_GROUP) {
                $shop_info = $this->trans('The modifications will be applied to this group: %s', array(Shop::getContextShopGroup()->name), 'Modules.DfxMegamenu.Admin');
            } else {
                $shop_info = $this->trans('The modifications will be applied to all shops and shop groups', array(), 'Modules.DfxMegamenu.Admin');
            }

            return '<div class="alert alert-info">'.
                        $shop_info.
                    '</div>';
        } else {
            return '';
        }
    }

    public function getSharedMenuWarning()
    {
        return '<p class="alert alert-warning">'.
                    $this->trans('This slide is shared with other shops! All shops associated to this slide will apply modifications made here', array(), 'Modules.DfxMegamenu.Admin').
                '</p>';
    }
    
    /* Front Office Features */
    public function hookAddProduct($params)
    {
        $this->_clearCache('*');
    }

    public function hookUpdateProduct($params)
    {
        $this->_clearCache('*');
    }

    public function hookDeleteProduct($params)
    {
        $this->_clearCache('*');
    }

    public function hookActionCategoryAdd($params)
    {
        $this->_clearCache('*');
    }

    public function hookActionCategoryUpdate($params)
    {
        $this->_clearCache('*');
    }

    public function hookActionCategoryDelete($params)
    {
        $this->_clearCache('*');
    }

    public function hookUpdateOrderStatus($params)
    {
        $this->_clearCache('*');
    }
	
	public function hookActionObjectManufacturerAddAfter()
    {
        $this->_clearCache('*');
    }

    public function hookActionObjectManufacturerUpdateAfter()
    {
        $this->_clearCache('*');
    }

    public function hookActionObjectManufacturerDeleteAfter()
    {
        $this->_clearCache('*');
    }
	
	public function _clearCache($template, $cache_id = null, $compile_id = null)
    {
        parent::_clearCache($this->templateFile);
    }
	
	public function renderWidget($hookName = null, array $configuration = [])
    {
        if (preg_match('/^displayMegamenuMobile\d*$/', $hookName)) {
			$this->templateFile = 'module:dfxmegamenu/views/templates/hook/megamenu_mobile.tpl';
			if (!$this->isCached($this->templateFile, $this->getCacheId('dfxmegamenu'))) {
				$this->smarty->assign($this->getWidgetVariables($hookName, $configuration));
			}
			return $this->fetch($this->templateFile, $this->getCacheId('dfxmegamenu'));
		} elseif (preg_match('/^displayNavFullWidth\d*$/', $hookName)) {
			$this->templateFile = 'module:dfxmegamenu/views/templates/hook/megamenu.tpl';
			if (!$this->isCached($this->templateFile, $this->getCacheId('dfxmegamenu'))) {
				$this->smarty->assign($this->getWidgetVariables($hookName, $configuration));
			}
			return $this->fetch($this->templateFile, $this->getCacheId('dfxmegamenu'));
		}
    }

    public function getWidgetVariables($hookName = null, array $configuration = [])
    {
        $menus = $this->getMenus(true);
        
        foreach ($menus as &$menu) {
            $menurows = $this->getRowslist($menu['id_dfxmenu'], true);
            
            foreach ($menurows as &$menurow) {
                $menurow['menucolumns'] = $this->getColumns($menurow['id_dfxmenurow'], true);

                foreach ($menurow['menucolumns'] as &$menucolumn) {
                    $menucolumn = $this->getMenucolumnitems($menucolumn);
                }
            }
            $menu['menurows'] = $menurows;
        }

         return [
            'menus' => $menus,
        ];
    }

    public function getMenucolumnitems($menucolumn)
    {
        $id_lang = (int)$this->context->language->id;
        $catlist = array();

                    /* category links */
            if ($menucolumn['content_type'] == 'category') {
                $cat_ids = explode(',', $menucolumn['category_list']);
                
                if ($menucolumn['cat_thumb'] == 1) {
                    foreach ($cat_ids as $key => $catid) {
                        $catlist[$key] = $this->getCategoryBigMenuThumb($catid, $id_lang);
                    }
                } else {
                    foreach ($cat_ids as $key => $catid) {
                        $catlist[$key] = $this->getCategoryBigMenu($catid, $id_lang);
                    }
                }
                
                $menucolumn['categories'] = implode(" ", $catlist);
			} elseif ($menucolumn['content_type'] == 'product') {
            /* products */
                $productbyIds = explode(',', $menucolumn['products']);
                $menucolumn['products'] = false;
                $menucolumn['products'] = $this->getProductsByIds($productbyIds, $id_lang);
                
			} elseif ($menucolumn['content_type'] == 'html') {
            /* html content */
                $menucolumn['htmlcontent'] = $menucolumn['static_content'];
			} elseif ($menucolumn['content_type'] == 'manufacturer') {
            /* manufacturer logo */
                $menucolumn['manufacture'] = $this->getManufacturersByIds($menucolumn['manufacturers'], $id_lang);
			} elseif ($menucolumn['content_type'] == 'cms_link') {
                /* cms link */
                $cms_ids = explode(',', $menucolumn['cmslinks']);
                $menucolumn['cmsHtml'] = $this->renderCMShtml($cms_ids, $id_lang);
			} elseif ($menucolumn['content_type'] == 'bannerimage') {
            /* Banner Image */
                $menucolumn['banner_image'] = $this->_path . $this->menu_img_folder . $menucolumn['banner_image'];
                $menucolumn['bannerlink'] = $menucolumn['banner_url'];
                $menucolumn['banner_w'] = $menucolumn['banner_image_w'];
                $menucolumn['banner_h'] = $menucolumn['banner_image_h'];
			}
                        /* content type close*/

        return $menucolumn;
    }
    public function categoryThumb($id_category = 1, $id_lang = false, $id_shop = false)
    {
        $id_lang = $this->context->language->id;

        $html = '';
        $category = new Category((int)$id_category, (int)$id_lang, (int)$id_shop);
        $level = $category->level_depth;
        
        if ($level > 1) {
            $category_link = $category->getLink();
        } else {
            $category_link = $this->context->link->getPageLink('index');
        }
        $imagesize = array();
        if ($level == 2) {
            $files = scandir(_PS_CAT_IMG_DIR_);
        
            if (count(preg_grep('/^'.$category->id.'-([0-9])?_thumb.jpg/i', $files)) > 0) {
                foreach ($files as $file) {
                    if (preg_match('/^'.$category->id.'-([0-9])?_thumb.jpg/i', $file) === 1) {
                        $image_url = _THEME_CAT_DIR_.$file;
                        $imagesize = @getimagesize(_THEME_CAT_DIR_.$file);
						$image_width = isset($imagesize['0']) ? $imagesize['0'] : '200';
                        $image_height = isset($imagesize['1']) ? $imagesize['1'] : '100';
                        $html .= '<div class="cat-img"><a href="'.Tools::HtmlEntitiesUTF8($category_link).'">';
                        $html .= '<img src="'.$image_url.'" loading="lazy" width="'.$image_width.'" height="'.$image_height.'"/>';
                        $html .= '</a></div>';
                    }
                }
            }
        }
        return $html;
    }
    
    public function getCategoryBigMenuThumb($id_category = 1, $id_lang = false, $id_shop = false, $recursive = true)
    {
        $html = '';
        $id_lang = $id_lang ? (int)$id_lang : (int)Context::getContext()->language->id;
        
        $category = new Category((int)$id_category, (int)$id_lang, (int)$id_shop);
        $level = $category->level_depth;
        
        if ($level > 1) {
            $category_link = $category->getLink();
        } else {
            $category_link = $this->context->link->getPageLink('index');
        }
            
        if (is_null($category->id)) {
            return;
        }
            
        if ($recursive) {
            $children = Category::getChildren((int)$id_category, (int)$id_lang, true, (int)$id_shop);
        }
    
        if (count($children)) {
            $html .= '<li class="has-sub">';
             $html .= $this->categoryThumb($category->id, (int)$id_lang);
        } else {
            $html .= '<li>';
        }
            
        if ($this->context->controller->controller_type != 'admin') {
            $html .= '<a href="'.Tools::HtmlEntitiesUTF8($category_link).'" class="mega-with-ul">'.$category->name.'</a>';
        } else {
            $html .= '<a href="'.Tools::HtmlEntitiesUTF8($category_link).'" class="mega-with-ul" target="_blank">'.$category->name.'</a>';
        }

        
        if ($level == 2) {
            $html .= '<div class="subcatlink">';
        }
                
        if (count($children)) {
            $html .= '<ul class="child-category depth-'.$level.'">';
                            
            foreach ($children as $child) {
                $html .= $this->getCategoryBigMenuThumb((int)$child['id_category'], (int)$id_lang, (int)$child['id_shop'], $recursive);
            }
                    
            $html .= '</ul>';
        }
        
        if ($level == 2) {
            $html .= '</div>';
        }
        $html .='</li>';
        return $html;
    }
    
    public function getCategoryBigMenu($id_category = 1, $id_lang = false, $id_shop = false, $recursive = true)
    {
        $html = '';
        $id_lang = $id_lang ? (int)$id_lang : (int)Context::getContext()->language->id;
        $category = new Category((int)$id_category, (int)$id_lang, (int)$id_shop);
		
		if (is_null($category->id)) {
            return;
        }
		
        $level = $category->level_depth;
        
        if ($level > 1) {
            $category_link = $category->getLink();
        } else {
            $category_link = $this->context->link->getPageLink('index');
        }

        if (is_null($category->id)) {
            return;
        }
            
        if ($recursive) {
            $children = Category::getChildren((int)$id_category, (int)$id_lang, true, (int)$id_shop);
        }
        if (count($children)) {
            $html .= '<li class="has-sub">';
        } else {
            $html .= '<li>';
        }
            
        if ($this->context->controller->controller_type != 'admin') {
            $html .= '<a href="'.Tools::HtmlEntitiesUTF8($category_link).'" class="mega-with-ul">'.$category->name.'</a>';
        } else {
            $html .= '<a href="'.Tools::HtmlEntitiesUTF8($category_link).'" class="mega-with-ul" target="_blank">'.$category->name.'</a>';
        }

        
        if ($level == 2) {
            $html .= '<div class="subcatlink">';
        }
        if (count($children)) {
            $html .= '<ul class="child-category depth-'.$level.'">';
            foreach ($children as $child) {
                $html .= $this->getCategoryBigMenu((int)$child['id_category'], (int)$id_lang, (int)$child['id_shop'], $recursive);
            }
            $html .= '</ul>';
        }
        
        if ($level == 2) {
            $html .= '</div>';
        }
        $html .='</li>';
        return $html;
    }

    public function renderCMShtml($cms_ids, $id_lang)
    {
        $html = '';
        $cms_links = CMS::getLinks((int)$id_lang, $cms_ids);
        $html .= '<ul>';
        if (count($cms_links)) {
            foreach ($cms_links as $cms) {
                $html .= '<li><a href="'.$cms['link'].'">'.$cms['meta_title'].'</a></li>';
            }
        }
        $html .= '</ul>';

        return $html;
    }
    
    public static function getManufacturersByIds($manufacturer_id = null, $id_lang = null)
    {
        if (empty($manufacturer_id)) {
            return false;
        }

        $context = Context::getContext();
        $id_lang = is_null($id_lang) ? $context->language->id : $id_lang;

        $query = 'SELECT m.*, ml.`description`, ml.`short_description`
            FROM `' . _DB_PREFIX_ . 'manufacturer` m
            ' . Shop::addSqlAssociation('manufacturer', 'm') . '
            LEFT JOIN `' . _DB_PREFIX_ . 'manufacturer_lang` ml ON (m.`id_manufacturer` = ml.`id_manufacturer` AND ml.`id_lang` = ' . (int)$id_lang . ')
            WHERE m.`id_manufacturer` IN (' . $manufacturer_id. ')
            AND m.`active` = 1
            GROUP BY m.`id_manufacturer`
            ORDER BY m.`name`';

        $manufacturers = Db::getInstance(_PS_USE_SQL_SLAVE_)->executeS($query);

        if (!$manufacturers) {
            return false;
        }

        foreach ($manufacturers as &$item) {
            $item['image'] = $context->link->getManufacturerImageLink($item['id_manufacturer']);
            $item['url'] = $context->link->getManufacturerLink($item['id_manufacturer']);
        }

        return $manufacturers;
    }

    public function getProductsByIds($productId = null, $id_lang = null)
    {
		 if (empty($productId)) {
            return false;
        }

        $products = $this->getProductsInfoByIds($productId, $id_lang);

        $presenterFactory = new ProductPresenterFactory($this->context);
        $presentationSettings = $presenterFactory->getPresentationSettings();

	    $presenter = new ProductListingPresenter(
            new ImageRetriever(
                $this->context->link
            ),
			
            $this->context->link,
            new PriceFormatter(),
            new ProductColorsRetriever(),
            $this->context->getTranslator()
        );


        if (is_array($products)) {
            foreach ($products as &$product) {
                $product = $presenter->present(
                    $presentationSettings,
                    Product::getProductProperties($id_lang, $product, $this->context),
                    $this->context->language
                );
            }
            unset($product);
        }

        return $products;
    }
    
    public function getProductsInfoByIds($productId, $id_lang, $active = true)
    {
        $product_ids = join(',', $productId);
    
        $id_shop = (int) $this->context->shop->id;
    
        $sql = 'SELECT p.*, product_shop.*, stock.out_of_stock, IFNULL(stock.quantity, 0) as quantity, pl.`description`, pl.`description_short`, pl.`link_rewrite`,
						pl.`meta_description`, pl.`meta_keywords`, pl.`meta_title`, pl.`name`, pl.`available_now`, pl.`available_later`,
						image_shop.`id_image` id_image, il.`legend`, m.`name` as manufacturer_name, cl.`name` AS category_default, IFNULL(product_attribute_shop.id_product_attribute, 0) id_product_attribute,
						DATEDIFF(
							p.`date_add`,
							DATE_SUB(
								"'.date('Y-m-d').' 00:00:00",
								INTERVAL '.(Validate::isUnsignedInt(Configuration::get('PS_NB_DAYS_NEW_PRODUCT')) ? Configuration::get('PS_NB_DAYS_NEW_PRODUCT') : 20).' DAY
							)
						) > 0 AS new
					FROM  `'._DB_PREFIX_.'product` p 
					'.Shop::addSqlAssociation('product', 'p').'
					LEFT JOIN `'._DB_PREFIX_.'product_attribute_shop` product_attribute_shop
						ON (p.`id_product` = product_attribute_shop.`id_product` AND product_attribute_shop.`default_on` = 1 AND product_attribute_shop.id_shop='.(int)$id_shop.')
					LEFT JOIN `'._DB_PREFIX_.'product_lang` pl ON (
						p.`id_product` = pl.`id_product`
						AND pl.`id_lang` = '.(int)$id_lang.Shop::addSqlRestrictionOnLang('pl').'
					)
					LEFT JOIN `'._DB_PREFIX_.'category_lang` cl ON (
						product_shop.`id_category_default` = cl.`id_category`
						AND cl.`id_lang` = '.(int)$id_lang.Shop::addSqlRestrictionOnLang('cl').'
					)
					LEFT JOIN `'._DB_PREFIX_.'image_shop` image_shop
						ON (image_shop.`id_product` = p.`id_product` AND image_shop.cover=1 AND image_shop.id_shop='.(int)$id_shop.')
					LEFT JOIN `'._DB_PREFIX_.'image_lang` il ON (image_shop.`id_image` = il.`id_image` AND il.`id_lang` = '.(int)$id_lang.')
					LEFT JOIN `'._DB_PREFIX_.'manufacturer` m ON (p.`id_manufacturer`= m.`id_manufacturer`)
					'.Product::sqlStock('p', 0).'
					WHERE p.id_product IN ('.$product_ids.')'.
                ($active ? ' AND product_shop.`active` = 1 AND product_shop.`visibility` != \'none\'' : '').'
					ORDER BY FIELD(product_shop.id_product, '.$product_ids.')
					';
        if (!$result = Db::getInstance(_PS_USE_SQL_SLAVE_)->executeS($sql)) {
            return false;
        }
        foreach ($result as &$row) {
            $row['id_product_attribute'] = Product::getDefaultAttribute((int)$row['id_product']);
        }
        return Product::getProductsProperties($id_lang, $result);
    }
        
    public function generateCss()
    {
        $menus = $this->getMenus();
        $display_rtl = $this->context->language->is_rtl;
        $css = '';
		$css_rtl = '';
        $cssgroup = array();
		$cssgroup_rtl = array();
        foreach ($menus as $key => $menu) {
			if($menu['custom_css']) {
            $cssgroup[$key] = ' .menulink-'.$menu['id_dfxmenu'].' '.$menu['custom_css'];
			 if (strpos($menu['custom_css'], 'right')) {
			  $cssgroup_rtl[$key] .= ' .menulink-'.$menu['id_dfxmenu'].' '.str_replace('right', 'left', $menu['custom_css']);
			 }
			 if (strpos($menu['custom_css'], 'left')) {
			  $cssgroup_rtl[$key] .= ' .menulink-'.$menu['id_dfxmenu'].' '.str_replace('left', 'right', $menu['custom_css']);
			 }
			}
			if($menu['custom_css'] == " ") {
				$cssgroup[$key] = '#customcss {}';
				$cssgroup_rtl[$key] = '#customcss {}';
			}
			
        }
		foreach ($cssgroup_rtl as $k => $customcss) {
			$css_rtl .= $customcss;
        }
        foreach ($cssgroup as $k => $customcss) {
			$css .= $customcss;
        }

        $my_owncss_rtl = $this->local_path.'views/css/menucustom_rtl.css';
		$handle_rtl = fopen($my_owncss_rtl, 'w+') or die("can't Load the css file");
        fwrite($handle_rtl, $css_rtl);
        fclose($handle_rtl);
		
		$my_owncss = $this->local_path.'views/css/menucustom.css';
        $handle = fopen($my_owncss, 'w+') or die("can't Load the css file");
        fwrite($handle, $css);
        fclose($handle);
    }
}
