<?php
/**
* Copyright since 2007 PrestaShop SA and Contributors
* PrestaShop is an International Registered Trademark & Property of PrestaShop SA
*
* NOTICE OF LICENSE
*
* This source file is subject to the Academic Free License 3.0 (AFL-3.0)
* that is bundled with this package in the file LICENSE.md.
* It is also available through the world-wide-web at this URL:
* https://opensource.org/licenses/AFL-3.0
* If you did not receive a copy of the license and are unable to
* obtain it through the world-wide-web, please send an email
* to license@prestashop.com so we can send you a copy immediately.
*
* DISCLAIMER
*
* Do not edit or add to this file if you wish to upgrade PrestaShop to newer
* versions in the future. If you wish to customize PrestaShop for your
* needs please refer to https://devdocs.prestashop.com/ for more information.
*
* @author    PrestaShop SA and Contributors <contact@prestashop.com>
* @copyright Since 2007 PrestaShop SA and Contributors
* @license   https://opensource.org/licenses/AFL-3.0 Academic Free License 3.0 (AFL-3.0)
*/

class Popletter extends ObjectModel
{
    public $id_dfxpopup;

    public $image;
    public $description;
    public $id_shop;
    /**
     * @see ObjectModel::$definition
     */
    public static $definition = array(
        'table' => 'dfxpopup',
        'primary' => 'id_dfxpopup',
        'multilang' => true,
        'fields' => array(
            'id_dfxpopup' => array('type' => self::TYPE_NOTHING, 'validate' => 'isUnsignedId'),
            // Lang fields
            'image' => array('type' => self::TYPE_STRING, 'lang' => true, 'validate' => 'isCleanHtml', 'size' => 255),
            'description' => array('type' => self::TYPE_HTML, 'lang' => true, 'validate' => 'isCleanHtml'),
        )
    );

    public function __construct($id_dfxpopup = null, $id_lang = null)
    {
        parent::__construct($id_dfxpopup, $id_lang);

        if (!$this->id_shop) {
            $this->id_shop = Context::getContext()->shop->id;
        }
    }
    /**
     * Return the popletter ID By shop ID
     *
     * @param int $shopId
     * @return bool|int
     */
    public static function getdfxpopupIdByShop($idlang = null)
    {
        $id_shop = Context::getContext()->shop->id;
		$id_lang = Context::getContext()->language->id;

        $sql = 'SELECT i.`id_dfxpopup` FROM `' . _DB_PREFIX_ . 'dfxpopup` i
		LEFT JOIN '._DB_PREFIX_.'dfxpopup_lang il ON (i.id_dfxpopup = il.id_dfxpopup)
        WHERE i.`id_shop` = ' . (int)$id_shop.' AND il.id_lang = '.(int)$id_lang;
        
        $id_dfxpopup = (int) Db::getInstance()->getValue($sql);

        if ($id_dfxpopup) {
            return new self($id_dfxpopup, $idlang);
        } else {
            return new self();
        }

        return false;
    }
}
