<?php
/**
 * Copyright since 2007 PrestaShop SA and Contributors
 * PrestaShop is an International Registered Trademark & Property of PrestaShop SA
 *
 * NOTICE OF LICENSE
 *
 * This source file is subject to the Academic Free License 3.0 (AFL-3.0)
 * that is bundled with this package in the file LICENSE.md.
 * It is also available through the world-wide-web at this URL:
 * https://opensource.org/licenses/AFL-3.0
 * If you did not receive a copy of the license and are unable to
 * obtain it through the world-wide-web, please send an email
 * to license@prestashop.com so we can send you a copy immediately.
 *
 * DISCLAIMER
 *
 * Do not edit or add to this file if you wish to upgrade PrestaShop to newer
 * versions in the future. If you wish to customize PrestaShop for your
 * needs please refer to https://devdocs.prestashop.com/ for more information.
 *
 * @author    PrestaShop SA and Contributors <contact@prestashop.com>
 * @copyright Since 2007 PrestaShop SA and Contributors
 * @license   https://opensource.org/licenses/AFL-3.0 Academic Free License 3.0 (AFL-3.0)
 */

if (!defined('_PS_VERSION_')) {
    exit;
}

use PrestaShop\PrestaShop\Core\Module\WidgetInterface;

class DfxRedfoxsocial extends Module implements WidgetInterface
{
    protected $templateFile;
    
    public function __construct()
    {
        $this->name = 'dfxredfoxsocial';
        $this->tab = 'front_office_features';
        $this->version = '1.0.0';
        $this->author = 'DigitalRedfox';
        $this->need_instance = 0;
        $this->bootstrap = true;

        parent::__construct();

        $this->displayName = $this->getTranslator()->trans('Redfox Social & Service Links', array(), 'Modules.DfxRedfoxsocial.Admin');
        $this->description = $this->getTranslator()->trans('Allows you to add extra information about social networks', array(), 'Modules.DfxRedfoxsocial.Admin');
        $this->ps_versions_compliancy = array('min' => '1.7.4.0', 'max' => _PS_VERSION_);

        $this->templateFile = 'module:dfxredfoxsocial/views/templates/front/dfxredfoxsocial.tpl';
    }
    
    
    public function install()
    {
        return (parent::install()
        && $this->registerHook('displayHeader')
        && $this->registerHook('displayFootTab')
        && $this->installFixture()
        );
    }
    
    protected function installFixture()
    {
        $languages = Language::getLanguages(false);
        $values = array();
        foreach ($languages as $lang) {
            $values['DFXREDFOXSOCIAL_TITLE'][$lang['id_lang']] = 'Connect with us on';
            $values['DFXREDFOXSOCIAL_LINK2'][$lang['id_lang']] = '#';
            $values['DFXREDFOXSOCIAL_TITLE2'][$lang['id_lang']] = 'Free Shipping on orders over $99';
            $values['DFXREDFOXSOCIAL_CONTENT2'][$lang['id_lang']] = 'Sed do eiusmod tempor incididunt ut labore et dolore magna aliqua.';
            
            $values['DFXREDFOXSOCIAL_LINK3'][$lang['id_lang']] = '#';
            $values['DFXREDFOXSOCIAL_TITLE3'][$lang['id_lang']] = 'Online Support';
            $values['DFXREDFOXSOCIAL_CONTENT3'][$lang['id_lang']] = 'Lorem ipsum dolor sit amet, consectetur adipisicing elit, sed do eiusmod tempor incididunt.';
            
            $values['dfxredfoxsocial_facebook'][$lang['id_lang']] = '#';
            $values['dfxredfoxsocial_googleplus'][$lang['id_lang']] = '#';
            $values['dfxredfoxsocial_pinterest'][$lang['id_lang']] = '#';
            $values['dfxredfoxsocial_youtube'][$lang['id_lang']] = '#';
            $values['dfxredfoxsocial_twitter'][$lang['id_lang']] = '#';
            $values['dfxredfoxsocial_in'][$lang['id_lang']] = '#';
        }
		Configuration::updateValue('new_window', 0);
        Configuration::updateValue('DFXREDFOXSOCIAL_TITLE', $values['DFXREDFOXSOCIAL_TITLE']);
        Configuration::updateValue('DFXREDFOXSOCIAL_TITLE2', $values['DFXREDFOXSOCIAL_TITLE2']);
        Configuration::updateValue('DFXREDFOXSOCIAL_TITLE3', $values['DFXREDFOXSOCIAL_TITLE3']);
        
        Configuration::updateValue('DFXREDFOXSOCIAL_CONTENT2', $values['DFXREDFOXSOCIAL_CONTENT2']);
        Configuration::updateValue('DFXREDFOXSOCIAL_CONTENT3', $values['DFXREDFOXSOCIAL_CONTENT3']);
        
        Configuration::updateValue('DFXREDFOXSOCIAL_LINK2', $values['DFXREDFOXSOCIAL_LINK2']);
        Configuration::updateValue('DFXREDFOXSOCIAL_LINK3', $values['DFXREDFOXSOCIAL_LINK3']);

        Configuration::updateValue('dfxredfoxsocial_facebook', $values['dfxredfoxsocial_facebook']);
        Configuration::updateValue('dfxredfoxsocial_googleplus', $values['dfxredfoxsocial_googleplus']);
        Configuration::updateValue('dfxredfoxsocial_pinterest', $values['dfxredfoxsocial_pinterest']);
        Configuration::updateValue('dfxredfoxsocial_youtube', $values['dfxredfoxsocial_youtube']);
        Configuration::updateValue('dfxredfoxsocial_twitter', $values['dfxredfoxsocial_twitter']);
        Configuration::updateValue('dfxredfoxsocial_in', $values['dfxredfoxsocial_in']);

        return true;
    }

    public function uninstall()
    {
        Configuration::deleteByName('dfxredfoxsocial_facebook');
        Configuration::deleteByName('dfxredfoxsocial_googleplus');
        Configuration::deleteByName('dfxredfoxsocial_pinterest');
        Configuration::deleteByName('dfxredfoxsocial_youtube');
        Configuration::deleteByName('dfxredfoxsocial_twitter');
        Configuration::deleteByName('dfxredfoxsocial_in');
        Configuration::deleteByName('DFXREDFOXSOCIAL_TITLE');
        
        Configuration::deleteByName('DFXREDFOXSOCIAL_LINK2');
        Configuration::deleteByName('DFXREDFOXSOCIAL_TITLE2');
        Configuration::deleteByName('DFXREDFOXSOCIAL_CONTENT2');
        
        Configuration::deleteByName('DFXREDFOXSOCIAL_LINK3');
        Configuration::deleteByName('DFXREDFOXSOCIAL_TITLE3');
        Configuration::deleteByName('DFXREDFOXSOCIAL_CONTENT3');
		
		Configuration::deleteByName('new_window');
        
        return parent::uninstall();
    }
    
    public function getContent()
    {
        return $this->postProcess().$this->renderForm();
    }
    public function _clearCache($template, $cache_id = null, $compile_id = null)
    {
        parent::_clearCache($this->templateFile);
    }
    protected function renderForm()
    {
        $fields_form = array(
        'form' => array(
            'legend' => array(
                    'title' => $this->getTranslator()->trans('Settings', array(), 'Admin.Global'),
                    'icon' => 'icon-cogs'
                ),
                'input' => array(
                    array(
                        'col' => 7,
                        'type' => 'text',
                        'label' => $this->getTranslator()->trans('Title', array(), 'Modules.DfxRedfoxsocial.Admin'),
                        'name' => 'DFXREDFOXSOCIAL_TITLE',
                        'desc' => $this->getTranslator()->trans('you can add here of your Common Title for Socialmedia links', array(), 'Modules.DfxRedfoxsocial.Admin'),
                        'lang' => true,
                    ),
                    array(
                        'col' => 7,
                        'type' => 'text',
                        'label' => $this->getTranslator()->trans('Facebook Link', array(), 'Modules.DfxRedfoxsocial.Admin'),
                        'name' => 'dfxredfoxsocial_facebook',
                           'lang' => true,
                        'desc' => $this->getTranslator()->trans('you can add here of your facebook URL', array(), 'Modules.DfxRedfoxsocial.Admin')
                    ),
                    array(
                        'col' => 7,
                        'type' => 'text',
                        'label' => $this->getTranslator()->trans('Google Plus Link', array(), 'Modules.DfxRedfoxsocial.Admin'),
                        'name' => 'dfxredfoxsocial_googleplus',
                        'lang' => true,
                        'desc' => $this->getTranslator()->trans('you can add here of your Googleplus URL', array(), 'Modules.DfxRedfoxsocial.Admin')
                    ),
                   array(
                        'col' => 7,
                        'type' => 'text',
                        'label' => $this->getTranslator()->trans('Pinterest Link', array(), 'Modules.DfxRedfoxsocial.Admin'),
                        'name' => 'dfxredfoxsocial_pinterest',
                        'lang' => true,
                        'desc' => $this->getTranslator()->trans('you can add here of your pinterest URL', array(), 'Modules.DfxRedfoxsocial.Admin')
                    ),
                   array(
                        'col' => 7,
                        'type' => 'text',
                        'label' => $this->getTranslator()->trans('Youtube Link', array(), 'Modules.DfxRedfoxsocial.Admin'),
                        'name' => 'dfxredfoxsocial_youtube',
                        'lang' => true,
                        'desc' => $this->getTranslator()->trans('you can add here of your youtube URL', array(), 'Modules.DfxRedfoxsocial.Admin')
                    ),
                   array(
                        'col' => 7,
                        'type' => 'text',
                        'label' => $this->getTranslator()->trans('Twitter Link', array(), 'Modules.DfxRedfoxsocial.Admin'),
                        'name' => 'dfxredfoxsocial_twitter',
                        'lang' => true,
                        'desc' => $this->getTranslator()->trans('you can add here of your Twitter URL', array(), 'Modules.DfxRedfoxsocial.Admin')
                    ),
                   array(
                        'col' => 7,
                        'type' => 'text',
                        'label' => $this->getTranslator()->trans('IN Link', array(), 'Modules.DfxRedfoxsocial.Admin'),
                        'name' => 'dfxredfoxsocial_in',
                        'lang' => true,
                        'desc' => $this->getTranslator()->trans('you can add here of your IN URL', array(), 'Modules.DfxRedfoxsocial.Admin')
                    ),
                   array(
                        'col' => 7,
                        'type' => 'text',
                        'label' => $this->getTranslator()->trans('Title 2', array(), 'Modules.DfxRedfoxsocial.Admin'),
                        'name' => 'DFXREDFOXSOCIAL_TITLE2',
                        'lang' => true,
                        'desc' => $this->getTranslator()->trans('Add your Title 2', array(), 'Modules.DfxRedfoxsocial.Admin')
                    ),
                      array(
                        'col' => 7,
                        'type' => 'text',
                        'label' => $this->getTranslator()->trans('Content 2', array(), 'Modules.DfxRedfoxsocial.Admin'),
                        'name' => 'DFXREDFOXSOCIAL_CONTENT2',
                        'lang' => true,
                        'desc' => $this->getTranslator()->trans('Add your Content 2', array(), 'Modules.DfxRedfoxsocial.Admin')
                    ),
                   array(
                        'col' => 7,
                        'type' => 'text',
                        'label' => $this->getTranslator()->trans('Link 2', array(), 'Modules.DfxRedfoxsocial.Admin'),
                        'name' => 'DFXREDFOXSOCIAL_LINK2',
                        'lang' => true,
                        'desc' => $this->getTranslator()->trans('Add URL for second block', array(), 'Modules.DfxRedfoxsocial.Admin')
                    ),
                   array(
                        'col' => 7,
                        'type' => 'text',
                        'label' => $this->getTranslator()->trans('Title 3', array(), 'Modules.DfxRedfoxsocial.Admin'),
                        'name' => 'DFXREDFOXSOCIAL_TITLE3',
                        'lang' => true,
                        'desc' => $this->getTranslator()->trans('Add your Title 3', array(), 'Modules.DfxRedfoxsocial.Admin')
                    ),
                   array(
                        'col' => 7,
                        'type' => 'text',
                        'label' => $this->getTranslator()->trans('Content 3', array(), 'Modules.DfxRedfoxsocial.Admin'),
                        'name' => 'DFXREDFOXSOCIAL_CONTENT3',
                        'lang' => true,
                        'desc' => $this->getTranslator()->trans('Add your Content 3', array(), 'Modules.DfxRedfoxsocial.Admin')
                    ),
                   array(
                        'col' => 7,
                        'type' => 'text',
                        'label' => $this->getTranslator()->trans('Link 3', array(), 'Modules.DfxRedfoxsocial.Admin'),
                        'name' => 'DFXREDFOXSOCIAL_LINK3',
                        'lang' => true,
                        'desc' => $this->getTranslator()->trans('Add URL for Third block', array(), 'Modules.DfxRedfoxsocial.Admin')
                    ),
					 array(
                        'type' => 'switch',
                        'label' => $this->getTranslator()->trans('All the Link open in New window', array(), 'Modules.DfxRedfoxsocial.Admin'),
                        'name' => 'new_window',
                        'is_bool' => true,
                        'values' => array(
                            array(
                                'id' => 'active_on',
                                'value' => 1,
                                'label' => $this->getTranslator()->trans('Yes', array(), 'Modules.DfxRedfoxsocial.Admin')
                            ),
                            array(
                                'id' => 'active_off',
                                'value' => 0,
                                'label' => $this->getTranslator()->trans('No', array(), 'Modules.DfxRedfoxsocial.Admin')
                            )
                        ),
                    ),
                 ),
                'submit' => array(
                    'title' => $this->getTranslator()->trans('Save', array(), 'Admin.Actions'),
                    'name' => 'submitAdvConf',
                )
            ),
        );
        $helper = new HelperForm();
        $helper->show_toolbar = false;
        $helper->table = $this->table;
        $lang = new Language((int)Configuration::get('PS_LANG_DEFAULT'));
        $helper->default_form_language = $lang->id;
        $helper->allow_employee_form_lang = Configuration::get('PS_BO_ALLOW_EMPLOYEE_FORM_LANG') ? Configuration::get('PS_BO_ALLOW_EMPLOYEE_FORM_LANG') : 0;
        $this->fields_form = array();

        $helper->identifier = $this->identifier;
        $helper->submit_action = 'submitAdvConf';
        $helper->currentIndex = $this->context->link->getAdminLink('AdminModules', false).'&configure='.$this->name.'&tab_module='.$this->tab.'&module_name='.$this->name;
        $helper->token = Tools::getAdminTokenLite('AdminModules');
        $helper->tpl_vars = array(
            'fields_value' => $this->getConfigFieldsValues(),
            'languages' => $this->context->controller->getLanguages(),
            'id_language' => $this->context->language->id
        );

        return $helper->generateForm(array($fields_form));
    }
    public function postProcess()
    {
        $languages = Language::getLanguages();
        $result = array();
        if (Tools::isSubmit('submitAdvConf')) {
            foreach ($languages as $lang) {
                $result['DFXREDFOXSOCIAL_LINK2'][$lang['id_lang']] = Tools::getValue('DFXREDFOXSOCIAL_LINK2_'.$lang['id_lang']);
                    
                
                $result['DFXREDFOXSOCIAL_LINK3'][$lang['id_lang']] = Tools::getValue('DFXREDFOXSOCIAL_LINK3_'.$lang['id_lang']);
                
                $result['DFXREDFOXSOCIAL_TITLE'][$lang['id_lang']] = Tools::getValue('DFXREDFOXSOCIAL_TITLE_'.$lang['id_lang']);
                    
                $result['DFXREDFOXSOCIAL_TITLE2'][$lang['id_lang']] = Tools::getValue('DFXREDFOXSOCIAL_TITLE2_'.$lang['id_lang']);
                
                $result['DFXREDFOXSOCIAL_TITLE3'][$lang['id_lang']] = Tools::getValue('DFXREDFOXSOCIAL_TITLE3_'.$lang['id_lang']);
                
                $result['DFXREDFOXSOCIAL_CONTENT2'][$lang['id_lang']] = Tools::getValue('DFXREDFOXSOCIAL_CONTENT2_'.$lang['id_lang']);
                    
                $result['DFXREDFOXSOCIAL_CONTENT3'][$lang['id_lang']] = Tools::getValue('DFXREDFOXSOCIAL_CONTENT3_'.$lang['id_lang']);
            
                $result['dfxredfoxsocial_facebook'][$lang['id_lang']] = Tools::getValue('dfxredfoxsocial_facebook_'.$lang['id_lang']);
                    
                $result['dfxredfoxsocial_googleplus'][$lang['id_lang']] = Tools::getValue('dfxredfoxsocial_googleplus_'.$lang['id_lang']);
                    
                $result['dfxredfoxsocial_pinterest'][$lang['id_lang']] = Tools::getValue('dfxredfoxsocial_pinterest_'.$lang['id_lang']);
                    
                $result['dfxredfoxsocial_youtube'][$lang['id_lang']] = Tools::getValue('dfxredfoxsocial_youtube_'.$lang['id_lang']);
                
                $result['dfxredfoxsocial_twitter'][$lang['id_lang']] = Tools::getValue('dfxredfoxsocial_twitter_'.$lang['id_lang']);
                    
                $result['dfxredfoxsocial_in'][$lang['id_lang']] = Tools::getValue('dfxredfoxsocial_in_'.$lang['id_lang']);
            }
            Configuration::updateValue('new_window', Tools::getValue('new_window'));
            Configuration::updateValue('DFXREDFOXSOCIAL_LINK2', $result['DFXREDFOXSOCIAL_LINK2']);
            Configuration::updateValue('DFXREDFOXSOCIAL_LINK3', $result['DFXREDFOXSOCIAL_LINK3']);
            
            Configuration::updateValue('DFXREDFOXSOCIAL_TITLE', $result['DFXREDFOXSOCIAL_TITLE']);
            Configuration::updateValue('DFXREDFOXSOCIAL_TITLE2', $result['DFXREDFOXSOCIAL_TITLE2']);
            Configuration::updateValue('DFXREDFOXSOCIAL_TITLE3', $result['DFXREDFOXSOCIAL_TITLE3']);
            
            Configuration::updateValue('DFXREDFOXSOCIAL_CONTENT2', $result['DFXREDFOXSOCIAL_CONTENT2']);
            Configuration::updateValue('DFXREDFOXSOCIAL_CONTENT3', $result['DFXREDFOXSOCIAL_CONTENT3']);
            
            Configuration::updateValue('dfxredfoxsocial_facebook', $result['dfxredfoxsocial_facebook']);
            Configuration::updateValue('dfxredfoxsocial_googleplus', $result['dfxredfoxsocial_googleplus']);
            Configuration::updateValue('dfxredfoxsocial_pinterest', $result['dfxredfoxsocial_pinterest']);
            Configuration::updateValue('dfxredfoxsocial_youtube', $result['dfxredfoxsocial_youtube']);
            Configuration::updateValue('dfxredfoxsocial_twitter', $result['dfxredfoxsocial_twitter']);
            Configuration::updateValue('dfxredfoxsocial_in', $result['dfxredfoxsocial_in']);
            
            $this->_clearCache('*');

            return $this->displayConfirmation($this->trans('The settings have been updated.', array(), 'Admin.Notifications.Success'));
        }
        return '';
    }
    public function hookDisplayHeader($params)
    {
        $this->context->controller->addCSS($this->_path.'views/css/front.css');
    }
    
    public function getConfigFieldsValues()
    {
        $languages = Language::getLanguages(false);
        $fields = array();

        foreach ($languages as $lang) {
            $fields['DFXREDFOXSOCIAL_TITLE'][$lang['id_lang']] = Tools::getValue('DFXREDFOXSOCIAL_TITLE_'.$lang['id_lang'], Configuration::get('DFXREDFOXSOCIAL_TITLE', $lang['id_lang']));
            $fields['DFXREDFOXSOCIAL_TITLE2'][$lang['id_lang']] = Tools::getValue('DFXREDFOXSOCIAL_TITLE2_'.$lang['id_lang'], Configuration::get('DFXREDFOXSOCIAL_TITLE2', $lang['id_lang']));
            $fields['DFXREDFOXSOCIAL_TITLE3'][$lang['id_lang']] = Tools::getValue('DFXREDFOXSOCIAL_TITLE3_'.$lang['id_lang'], Configuration::get('DFXREDFOXSOCIAL_TITLE3', $lang['id_lang']));
            
            $fields['DFXREDFOXSOCIAL_CONTENT2'][$lang['id_lang']] = Tools::getValue('DFXREDFOXSOCIAL_CONTENT2_'.$lang['id_lang'], Configuration::get('DFXREDFOXSOCIAL_CONTENT2', $lang['id_lang']));
            $fields['DFXREDFOXSOCIAL_CONTENT3'][$lang['id_lang']] = Tools::getValue('DFXREDFOXSOCIAL_CONTENT3_'.$lang['id_lang'], Configuration::get('DFXREDFOXSOCIAL_CONTENT3', $lang['id_lang']));
            
            $fields['DFXREDFOXSOCIAL_LINK2'][$lang['id_lang']] = Tools::getValue('DFXREDFOXSOCIAL_LINK2_'.$lang['id_lang'], Configuration::get('DFXREDFOXSOCIAL_LINK2', $lang['id_lang']));
            $fields['DFXREDFOXSOCIAL_LINK3'][$lang['id_lang']] = Tools::getValue('DFXREDFOXSOCIAL_LINK3_'.$lang['id_lang'], Configuration::get('DFXREDFOXSOCIAL_LINK3', $lang['id_lang']));
            
            $fields['dfxredfoxsocial_facebook'][$lang['id_lang']] = Tools::getValue('dfxredfoxsocial_facebook_'.$lang['id_lang'], Configuration::get('dfxredfoxsocial_facebook', $lang['id_lang']));
            $fields['dfxredfoxsocial_googleplus'][$lang['id_lang']] = Tools::getValue('dfxredfoxsocial_googleplus_'.$lang['id_lang'], Configuration::get('dfxredfoxsocial_googleplus', $lang['id_lang']));
            $fields['dfxredfoxsocial_pinterest'][$lang['id_lang']] = Tools::getValue('dfxredfoxsocial_pinterest_'.$lang['id_lang'], Configuration::get('dfxredfoxsocial_pinterest', $lang['id_lang']));
            $fields['dfxredfoxsocial_youtube'][$lang['id_lang']] = Tools::getValue('dfxredfoxsocial_youtube_'.$lang['id_lang'], Configuration::get('dfxredfoxsocial_youtube', $lang['id_lang']));
            $fields['dfxredfoxsocial_twitter'][$lang['id_lang']] = Tools::getValue('dfxredfoxsocial_twitter_'.$lang['id_lang'], Configuration::get('dfxredfoxsocial_twitter', $lang['id_lang']));
            $fields['dfxredfoxsocial_in'][$lang['id_lang']] = Tools::getValue('dfxredfoxsocial_in_'.$lang['id_lang'], Configuration::get('dfxredfoxsocial_in', $lang['id_lang']));
        }
        $fields['new_window'] = Tools::getValue('new_window', Configuration::get('new_window'));
        return $fields;
    }

    public function renderWidget($hookName = null, array $configuration = [])
    {
	  if (!preg_match('/^displayHeader\d*$/', $hookName)) {
        if (!$this->isCached($this->templateFile, $this->getCacheId('dfxredfoxsocial'))) {
            $this->smarty->assign($this->getWidgetVariables($hookName, $configuration));
        }

        return $this->fetch($this->templateFile, $this->getCacheId('dfxredfoxsocial'));
	  } 
    }

    public function getWidgetVariables($hookName = null, array $configuration = [])
    {
        return array(
            'DFXREDFOXSOCIAL_TITLE' => Configuration::get('DFXREDFOXSOCIAL_TITLE', $this->context->language->id),
            'DFXREDFOXSOCIAL_TITLE2' => Configuration::get('DFXREDFOXSOCIAL_TITLE2', $this->context->language->id),
            'DFXREDFOXSOCIAL_TITLE3' => Configuration::get('DFXREDFOXSOCIAL_TITLE3', $this->context->language->id),
            'DFXREDFOXSOCIAL_CONTENT2' => Configuration::get('DFXREDFOXSOCIAL_CONTENT2', $this->context->language->id),
            'DFXREDFOXSOCIAL_CONTENT3' => Configuration::get('DFXREDFOXSOCIAL_CONTENT3', $this->context->language->id),
            'DFXREDFOXSOCIAL_LINK2' => Configuration::get('DFXREDFOXSOCIAL_LINK2', $this->context->language->id),
            'DFXREDFOXSOCIAL_LINK3' => Configuration::get('DFXREDFOXSOCIAL_LINK3', $this->context->language->id),
            
            'dfxredfoxsocial_facebook' => Configuration::get('dfxredfoxsocial_facebook', $this->context->language->id),
            'dfxredfoxsocial_googleplus' => Configuration::get('dfxredfoxsocial_googleplus', $this->context->language->id),
            'dfxredfoxsocial_pinterest' => Configuration::get('dfxredfoxsocial_pinterest', $this->context->language->id),
            'dfxredfoxsocial_youtube' => Configuration::get('dfxredfoxsocial_youtube', $this->context->language->id),
            'dfxredfoxsocial_twitter' => Configuration::get('dfxredfoxsocial_twitter', $this->context->language->id),
            'dfxredfoxsocial_in' => Configuration::get('dfxredfoxsocial_in', $this->context->language->id),
            'new_window' => Configuration::get('new_window'),
        );
    }

}
