<?php
/**
 * 2007-2021 PrestaShop
 *
 * NOTICE OF LICENSE
 *
 * This source file is subject to the Academic Free License (AFL 3.0)
 * that is bundled with this package in the file LICENSE.txt.
 * It is also available through the world-wide-web at this URL:
 * http://opensource.org/licenses/afl-3.0.php
 * If you did not receive a copy of the license and are unable to
 * obtain it through the world-wide-web, please send an email
 * to license@prestashop.com so we can send you a copy immediately.
 *
 * DISCLAIMER
 *
 * Do not edit or add to this file if you wish to upgrade PrestaShop to newer
 * versions in the future. If you wish to customize PrestaShop for your
 * needs please refer to http://www.prestashop.com for more information.
 *
 *  @author    PrestaShop SA <contact@prestashop.com>
 *  @copyright 2007-2021 PrestaShop SA
 *  @license   http://opensource.org/licenses/afl-3.0.php  Academic Free License (AFL 3.0)
 *  International Registered Trademark & Property of PrestaShop SA
 */


if (!defined('_PS_VERSION_')) {
    exit;
}

use PrestaShop\PrestaShop\Core\Module\WidgetInterface;

class DfxThemeOptions extends Module implements WidgetInterface
{
    protected $templateFile;
	private $themeName;
	private $color_settings;
	private $fonts;
	private $themelayout;
	public $_html = '';
	protected $currentIndex;

    public function __construct()
    {
        $this->name = 'dfxthemeoptions';
        $this->tab = 'front_office_features';
        $this->version = '5.2.0';
        $this->author = 'DigitalRedfox';
        $this->need_instance = 0;
        $this->bootstrap = true;
        parent::__construct();
        $this->displayName = $this->trans('Theme Controller', array(), 'Modules.DfxThemeoptions.Admin');
        $this->description = $this->trans('Theme Customizer front panel, change Template color fonts etc', array(), 'Modules.DfxThemeoptions.Admin');
        
        $this->dfx_color_settings = array();
        $this->dfx_color_custom = array();
        $this->theme_name = array();
		$this->is_mobile = ($this->context->isMobile() && $this->context->isTablet());
		$this->currentIndex = AdminController::$currentIndex.'&configure='.$this->name;
    }

    public function install()
    {
        if (parent::install() &&
        $this->registerHook('displayHeader') &&
        $this->registerHook('displayListImage') &&
        $this->registerHook('displayBrandImage') &&
        $this->registerHook('actionProductAdd') &&
        $this->registerHook('actionProductUpdate') &&
        $this->registerHook('displayThemeControll') &&
        $this->registerHook('actionProductDelete')) {
		include(dirname(__FILE__) . '/themes_style.php');
		$defaults = Configuration::updateValue('dfx_themelayout', serialize($themelayout));
		
		$defaults = Configuration::updateValue('dfx_layout', 0);
		$defaults = Configuration::updateValue('custom_css', '');
		$defaults = Configuration::updateValue('custom_font_path', '');
		$defaults = Configuration::updateValue('custom_font_name', '');
		$defaults = Configuration::updateValue('custom_contentfont_path', '');
		$defaults = Configuration::updateValue('custom_contentfont_name', '');
		
		// Colors
		$defaults = Configuration::updateValue('dfx_color_settings', serialize($color_settings));
		$defaults = Configuration::updateValue('dfx_themename', serialize($themeName));
		$defaults = Configuration::updateValue('dfx_color_custom', serialize($color_settings));
		
		// fonts
		$defaults = Configuration::updateValue('fonts', serialize($fonts));

		// theme 01
		$defaults = Configuration::updateValue('layout01_body_width', 0);
		$defaults = Configuration::updateValue('layout01_product_grid_list', "def_grid");
		$defaults = Configuration::updateValue('layout01_header_layout', "left_logo");
		$defaults = Configuration::updateValue('layout01_sticky_header', "no");
		$defaults = Configuration::updateValue('layout01_show_thumbnail', 1);
		$defaults = Configuration::updateValue('layout01_top_layout', 0);
		
		$defaults = Configuration::updateValue('layout01_body_bg_ext', '');
		$defaults = Configuration::updateValue('layout01_body_bg_repeat', 0);
		$defaults = Configuration::updateValue('layout01_body_bg_position', 0);
		$defaults = Configuration::updateValue('layout01_body_bg_fixed', '0');
		$defaults = Configuration::updateValue('layout01_body_bg_pattern', 0);
		$defaults = Configuration::updateValue('layout01_DRX_THEME_COLOR', 0);
		$defaults = Configuration::updateValue('layout01_title_font', 'font1');
		$defaults = Configuration::updateValue('layout01_content_font', 'font1');
		
		return $defaults;
        }
        return false;
    }

    public function uninstall()
    {
        if (!parent::uninstall()
        ) {
            return false;
        }
        return true;
    }

    public function getContent()
    {
        $this->context->controller->addCSS(($this->_path) . 'views/css/admin.css');

        $errors = '';
        $output = '';
        
        $this->dfx_color_settings = Tools::unSerialize(Configuration::get('dfx_color_settings'));
        $this->dfx_color_custom = Tools::unSerialize(Configuration::get('dfx_color_custom'));
        $this->theme_name = Tools::unSerialize(Configuration::get('dfx_themename'));
        
        $dfx_layout = Configuration::get('dfx_layout');

        $tlayouts = Tools::unSerialize(Configuration::get('dfx_themelayout'));
        
        $tlayout = $tlayouts[0];
        
        $output = '<h2>' . ucwords($this->displayName) . '</h2>';
        if (Tools::isSubmit($tlayout.'_body_upload_background')) {
            if (isset($_FILES[$tlayout.'_body_background_image_field']) && isset($_FILES[$tlayout.'_body_background_image_field']['tmp_name']) && !empty($_FILES[$tlayout.'_body_background_image_field']['tmp_name'])) {
                if ($error = ImageManager::validateUpload($_FILES[$tlayout.'_body_background_image_field'], Tools::convertBytes(ini_get('upload_max_filesize')))) {
                    $errors.= $error;
                } else {
                    Configuration::updateValue($tlayout.'_body_bg_ext', Tools::strtolower(Tools::substr(strrchr($_FILES[$tlayout.'_body_background_image_field']['name'], '.'), 1)));
                    
                    
                    $adv_imgname = $tlayout.'_body_background';
                    if (!move_uploaded_file($_FILES[$tlayout.'_body_background_image_field']['tmp_name'], _PS_MODULE_DIR_ . $this->name . '/views/img/' . $adv_imgname . '.' . Configuration::get($tlayout.'_body_bg_ext'))) {
                        $errors.= $this->trans('Error move uploaded file', array(), 'Modules.DfxThemeoptions.Admin');
                    }
                    $output.= '<div class="conf confirm">' . $this->trans('Image uploaded', array(), 'Modules.DfxThemeoptions.Admin') . '</div>';
                }
            }
			$this->_clearCache('*');
        } elseif (Tools::isSubmit($tlayout.'_body_background_image_delete')) {
            if (file_exists(_PS_MODULE_DIR_ . $this->name . '/views/img/'.$tlayout.'_body_background.' . Configuration::get($tlayout.'_body_bg_ext'))) {
                unlink(_PS_MODULE_DIR_ . $this->name . '/views/img/'.$tlayout.'_body_background.' . Configuration::get($tlayout.'_body_bg_ext'));
            }
            Configuration::updateValue($tlayout.'_body_bg_ext', "");
			$this->_clearCache('*');
            $output = $this->displayConfirmation($this->trans('Image removed', array(), 'Modules.DfxThemeoptions.Admin'));
        } elseif (Tools::isSubmit('submitSettings')) {
            Configuration::updateValue($tlayout.'_DRX_THEME_COLOR', Tools::getValue('theme_color'));
            
        
            $theme_number =  Tools::getValue('theme_color', Configuration::get($tlayout.'_DRX_THEME_COLOR'));
            
            $allcustom = $this->dfx_color_custom[$theme_number];
            
            $allcustom['body_bg'] = Tools::getValue($theme_number.'_body_bg');
            $allcustom['topnav_bg'] = Tools::getValue($theme_number.'_topnav_bg');
            $allcustom['template_bg'] = Tools::getValue($theme_number.'_template_bg');
            $allcustom['header_bg'] = Tools::getValue($theme_number.'_header_bg');
            $allcustom['menu_bg'] = Tools::getValue($theme_number.'_menu_bg');
            $allcustom['primary'] = Tools::getValue($theme_number.'_primary');
            $allcustom['secondary'] = Tools::getValue($theme_number.'_secondary');
            $allcustom['sale_bg'] = Tools::getValue($theme_number.'_sale_bg');
            $allcustom['new_bg'] = Tools::getValue($theme_number.'_new_bg');
            $allcustom['footer_bg'] = Tools::getValue($theme_number.'_footer_bg');

            $this->dfx_color_custom[$theme_number] = $allcustom;
            Configuration::updateValue('dfx_color_custom', serialize($this->dfx_color_custom));
            
            
            Configuration::updateValue('custom_font_path', Tools::getValue('custom_font_path'));
            Configuration::updateValue('custom_font_name', Tools::getValue('custom_font_name'));
            Configuration::updateValue('custom_contentfont_path', Tools::getValue('custom_contentfont_path'));
            Configuration::updateValue('custom_contentfont_name', Tools::getValue('custom_contentfont_name'));
            
            // Default Templates
            Configuration::updateValue('dfx_layout', Tools::getValue('dfx_layout'));
            Configuration::updateValue('custom_css', Tools::getValue('custom_css'));
            // General settings

            Configuration::updateValue($tlayout.'_body_width', Tools::getValue($tlayout.'_body_width'));
            Configuration::updateValue($tlayout.'_product_grid_list', Tools::getValue($tlayout.'_product_grid_list'));
            Configuration::updateValue($tlayout.'_body_bg_repeat', Tools::getValue($tlayout.'_body_bg_repeat'));
            Configuration::updateValue($tlayout.'_body_bg_position', Tools::getValue($tlayout.'_body_bg_position'));
            Configuration::updateValue($tlayout.'_body_bg_fixed', Tools::getValue($tlayout.'_body_bg_fixed'));
            Configuration::updateValue($tlayout.'_body_bg_pattern', Tools::getValue($tlayout.'_body_bg_pattern'));
            Configuration::updateValue($tlayout.'_show_thumbnail', Tools::getValue($tlayout.'_show_thumbnail'));
            // Header settings
            Configuration::updateValue($tlayout.'_header_layout', Tools::getValue($tlayout.'_header_layout'));
            Configuration::updateValue($tlayout.'_sticky_header', Tools::getValue($tlayout.'_sticky_header'));
            // Page Top Settings
            Configuration::updateValue($tlayout.'_top_layout', Tools::getValue($tlayout.'_top_layout'));
            Configuration::updateValue($tlayout.'_title_font', Tools::getValue($tlayout.'_title_font'));
            Configuration::updateValue($tlayout.'_content_font', Tools::getValue($tlayout.'_content_font'));
            
            $this->createStyleSheet();
			$this->_clearCache('*');
            $output = $this->displayConfirmation($this->trans('Your settings have been updated.', array(), 'Modules.DfxThemeoptions.Admin'));
        } elseif (Tools::isSubmit('reset_btn')) {
            $output = $this->resetSettings();
        } 

		if (Tools::isSubmit('submitLiveEnable')) {
            Configuration::updateValue('DFX_TC_ACTIVE', (int)Tools::getValue('live_conf'));
        }
		
        $output .= $this->renderConfigurationForm();
        $output .= $this->renderForm();
        return $output;
    }
    
    public function resetSettings()
    {
        $defaults = Configuration::updateValue('dfx_layout', 0);
        $defaults .= Configuration::updateValue('custom_css', '');
        // theme 01
        $defaults .= Configuration::updateValue('layout01_body_width', 0);
        $defaults .= Configuration::updateValue('layout01_product_grid_list', "def_grid");
        $defaults .= Configuration::updateValue('layout01_header_layout', "left_logo");
        $defaults .= Configuration::updateValue('layout01_sticky_header', "no");
        $defaults .= Configuration::updateValue('layout01_show_thumbnail', 1);
        $defaults .= Configuration::updateValue('layout01_top_layout', 0);
            
        $defaults .= Configuration::updateValue('layout01_body_bg_ext', '');
        $defaults .= Configuration::updateValue('layout01_body_bg_repeat', 0);
        $defaults .= Configuration::updateValue('layout01_body_bg_position', 0);
        $defaults .= Configuration::updateValue('layout01_body_bg_fixed', '0');
        $defaults .= Configuration::updateValue('layout01_body_bg_pattern', 0);
        $defaults .= Configuration::updateValue('layout01_DRX_THEME_COLOR', 0);

        $theme_number = Tools::getValue('theme_color', Configuration::get('layout01_DRX_THEME_COLOR'));
        $defSetting = $this->dfx_color_settings;
        $resetSetting = $defSetting[$theme_number];
        $this->dfx_color_custom[$theme_number] = $resetSetting;
        Configuration::updateValue('dfx_color_custom', serialize($this->dfx_color_custom));
            
        $this->createStyleSheet();
        $defaults = $this->displayConfirmation($this->trans('Your settings have been Reset successfully.', array(), 'Modules.DfxThemeoptions.Admin'));
            
        return $defaults;
    }
        
    public function renderForm()
    {
        $this->context->controller->addJS(($this->_path) . 'views/js/tabs.js');
        $this->context->controller->addCSS(($this->_path).'views/css/jquery.minicolors.css');
        $this->context->controller->addJS(($this->_path) . 'views/js/jquery.minicolors.js');
        
        // Default Templates
        $dfx_layout = Tools::getValue('dfx_layout', Configuration::get('dfx_layout'));
        $tlayouts = Tools::unSerialize(Configuration::get('dfx_themelayout'));
        
        $tlayout = $tlayouts[0];// $dfx_layout need to be replace to 0 <-
        // Colors Settings

        $theme_name = Tools::unSerialize(Configuration::get('dfx_themename'));
        $allcustom = $this->dfx_color_custom;

        // General settings
        $body_width = $tlayout.'_body_width';
        $body_bg_ext = Tools::getValue($tlayout.'_body_bg_ext', Configuration::get($tlayout.'_body_bg_ext'));
        $body_bg_repeat = $tlayout.'_body_bg_repeat';
        $body_bg_position = $tlayout.'_body_bg_position';
        $body_bg_fixed = $tlayout.'_body_bg_fixed';
        $body_bg_pattern = $tlayout.'_body_bg_pattern';
        $product_grid_list = $tlayout.'_product_grid_list';
        $show_thumbnail = $tlayout.'_show_thumbnail';
        
        // Header settings
        $header_layout = $tlayout.'_header_layout';
        $sticky_header = $tlayout.'_sticky_header';
        
        // Page Top settings
        $top_layout = $tlayout.'_top_layout';
        
        // font settings
        $title_font = $tlayout.'_title_font';
        $content_font = $tlayout.'_content_font';

        $custom_css = Tools::getValue('custom_css', Configuration::get('custom_css'));
        
        $custom_font_path = Tools::getValue('custom_font_path', Configuration::get('custom_font_path'));
        $custom_font_name = Tools::getValue('custom_font_name', Configuration::get('custom_font_name'));
        $custom_contentfont_path = Tools::getValue('custom_contentfont_path', Configuration::get('custom_contentfont_path'));
        $custom_contentfont_name = Tools::getValue('custom_contentfont_name', Configuration::get('custom_contentfont_name'));

        $allfonts = Tools::unserialize(Configuration::get('fonts'));

        $this->smarty->assign(array(
            'postAction' => 'index.php?tab=AdminModules&configure=' . $this->name . '&token=' . Tools::getAdminTokenLite('AdminModules') . '&tab_module=other&module_name=' . $this->name . '',
            'tlayouts' => $tlayouts,
            'tlayout' => $tlayout,
            'product_grid_list' => $product_grid_list,
            'body_width' => $body_width,
            'body_bg_repeat' => $body_bg_repeat,
            'body_bg_position' => $body_bg_position,
            'body_bg_fixed' => $body_bg_fixed,
            'body_bg_ext' => $body_bg_ext,
            'body_bg_pattern' => $body_bg_pattern,
            'header_layout' => $header_layout,
            'sticky_header' => $sticky_header,
            'show_thumbnail' => $show_thumbnail,
            'top_layout' => $top_layout,
            'allcustom' => $allcustom,
            'theme_name' => $theme_name,
            'title_font' => $title_font,
            'content_font' => $content_font,
            'allfonts' => $allfonts,
            'custom_font_path' => $custom_font_path,
            'custom_font_name' => $custom_font_name,
            'custom_contentfont_path' => $custom_contentfont_path,
            'custom_contentfont_name' => $custom_contentfont_name,
            'custom_css' => $custom_css,
            'dfx_color_custom' => $this->dfx_color_custom,
        
        ));
        return $this->display(__FILE__, 'views/templates/admin/adminform.tpl');
    }
    
        
    public function hookDisplayHeader()
    {
        $this->context->controller->addCSS(($this->_path).'views/css/theme_color.css', 'all');
        
        $dfx_layout = Configuration::get('dfx_layout');
        $tlayouts = Tools::unSerialize(Configuration::get('dfx_themelayout'));
        $tlayout = $tlayouts[0];
    
        $body_width =  Tools::getValue($tlayout.'_body_width', Configuration::get($tlayout.'_body_width'));
        $product_grid_list =  Tools::getValue($tlayout.'_product_grid_list', Configuration::get($tlayout.'_product_grid_list'));
        $header_layout =  Tools::getValue($tlayout.'_header_layout', Configuration::get($tlayout.'_header_layout'));
        $sticky_header =  Tools::getValue($tlayout.'_sticky_header', Configuration::get($tlayout.'_sticky_header'));
        $top_layout = Tools::getValue($tlayout.'_top_layout', Configuration::get($tlayout.'_top_layout'));
        $show_thumbnail =  Tools::getValue($tlayout.'_show_thumbnail', Configuration::get($tlayout.'_show_thumbnail'));

        $allfonts = Tools::unserialize(Configuration::get('fonts'));
        $title_fonts = Tools::getValue($tlayout.'_title_font', Configuration::get($tlayout.'_title_font'));
        $contentfonts = Tools::getValue($tlayout.'_content_font', Configuration::get($tlayout.'_content_font'));
            
        $header_font_link = "";
        $content_font_link = "";
                
        if (Configuration::get($tlayout.'_title_font')) {
            foreach ($allfonts as $fonts => $value) {
                if ($title_fonts == 'customfont') {
                    $header_font_link =  Tools::getValue('custom_font_path', Configuration::get('custom_font_path'));
                } elseif ($title_fonts == $fonts) {
                    $header_font_link =  $value['link'];
                }
            }
        }
            
        if (Configuration::get($tlayout.'_content_font')) {
            foreach ($allfonts as $fonts => $value) {
                if ($contentfonts == 'customfont') {
                    $content_font_link =  Tools::getValue('custom_contentfont_path', Configuration::get('custom_contentfont_path'));
                } elseif ($contentfonts == $fonts) {
                    $content_font_link =  $value['link'];
                }
            }
        }
        $display_rtl = $this->context->language->is_rtl;
        if ($this->is_mobile == false) {
            if (Configuration::get('DFX_TC_ACTIVE') == 1) {

             if($display_rtl) {
				 $this->context->controller->addCSS(($this->_path).'views/css/live_configurator_rtl.css');
			 } else {
				$this->context->controller->addCSS(($this->_path).'views/css/live_configurator.css');
			 }
                $this->context->controller->addCSS(($this->_path).'views/css/jquery.minicolors.css');
                $this->context->controller->addJS(($this->_path).'views/js/live_configurator.js');
                $this->context->controller->addJS(($this->_path).'views/js/jquery.minicolors.min.js');
                
                $jsFile = 'modules/'.$this->name.'/views/js/jquery.storageapi'.(_PS_MODE_DEV_ ? '' : '.min').'.js';
    
                $attributes = array(
                    'position' => 'bottom',
                    'priority' => 160
                );
                
                $this->context->controller->registerJavascript($this->name, $jsFile, $attributes);

                $this->themecontrollerURL = $this->renderWidget('displayThemeControll');
                
                Media::addJsDef(['themecontrollerURL' => $this->themecontrollerURL, 'pattern_url' => $this->_path.'views/img/patterns/', 'bodywidth' => $body_width]);
            }
            if ('product' === $this->context->controller->php_self) {
                $this->context->controller->addJS(($this->_path).'views/js/jqzoom.js');
                $this->context->controller->addJS(($this->_path).'views/js/jquery.elevatezoom.min.js');
            }
        }
	 
        $this->context->smarty->assign(array(
            'body_width' => $body_width,
            'product_grid_list' => $product_grid_list,
            'header_layout' => $header_layout,
            'sticky_header' => $sticky_header,
            'top_layout' => $top_layout,
            'show_thumbnail' => $show_thumbnail,
            'tlayout' => $tlayout,
            'header_font_link' => $header_font_link,
            'content_font_link' => $content_font_link,
            'themeoption' => 1,
			'title_font' => $title_fonts,
			'content_font' => $contentfonts,
            'imglazyload' => (!$this->context->isMobile() && !$this->context->controller->ajax),
        ));
    }

    public function renderConfigurationForm()
    {
        if ($this->context->shop->getBaseURL()) {
            $url = $this->context->link->getPageLink('index', null, $id_lang = null);
            
            $desc = '<a class="btn btn-default" href="'.$url.'" onclick="return !window.open($(this).attr(\'href\'));" id="live_conf_button">'
                .$this->trans('View', array(), 'Modules.DfxThemeoptions.Admin').' <i class="icon-external-link"></i></a><br />'
                .$this->trans('Only you can see this on your front office - your visitors will not see this tool.', array(), 'Modules.DfxThemeoptions.Admin');
        } else {
            $desc = $this->trans('Only you can see this on your front office - your visitors will not see this tool.', array(), 'Modules.DfxThemeoptions.Admin');
        }
            
        $fields_form = array(
            'form' => array(
                'legend' => array(
                    'title' => $this->trans('Settings'),
                    'icon' => 'icon-cogs'
                ),
                'description' => $this->trans('Configure Settings.', array(), 'Modules.DfxThemeoptions.Admin'),
                'input' => array(
                    array(
                        'type' => 'switch',
                        'label' => $this->trans('Enable Front Controll', array(), 'Modules.DfxThemeoptions.Admin'),
                        'name' => 'live_conf',
                        'desc' => $desc,
                        'values' => array(
                                    array(
                                        'id' => 'active_on',
                                        'value' => 1,
                                        'label' => $this->trans('Enabled')
                                    ),
                                    array(
                                        'id' => 'active_off',
                                        'value' => 0,
                                        'label' => $this->trans('Disabled')
                                    )
                                ),
                    ),
                ),
                'submit' => array(
                    'title' => $this->trans('Save'),
                )
            ),
        );

        $helper = new HelperForm();
        $helper->show_toolbar = false;
        $helper->table = $this->table;
        $lang = new Language((int)Configuration::get('PS_LANG_DEFAULT'));
        $helper->default_form_language = $lang->id;
        $this->fields_form = array();
        $helper->id = (int)Tools::getValue('id_carrier');
        $helper->identifier = $this->identifier;
        $helper->submit_action = 'submitLiveEnable';
        $helper->currentIndex = $this->context->link->getAdminLink('AdminModules', false).'&configure='.$this->name.'&tab_module='.$this->tab.'&module_name='.$this->name;
        $helper->token = Tools::getAdminTokenLite('AdminModules');
        $helper->tpl_vars = array(
            'fields_value' => $this->getConfigFieldsValuesSettings(),
            'languages' => $this->context->controller->getLanguages(),
            'id_language' => $this->context->language->id
        );

        return $helper->generateForm(array($fields_form));
    }
    
    public function getConfigFieldsValuesSettings()
    {
        return array(
            'live_conf' => Configuration::get('DFX_TC_ACTIVE'),
        );
    }
    
    public function readableColour($hex)
    {
        if ($hex == "transparent" || empty($hex)) {
            return '#000000';
        } else {
            $r = hexdec(Tools::substr($hex, 1, 2));
            $g = hexdec(Tools::substr($hex, 3, 2));
            $b = hexdec(Tools::substr($hex, 5, 2));
        
            $squared_contrast = (
                $r * $r * .299 +
                $g * $g * .587 +
                $b * $b * .114
            );

            if ($squared_contrast > pow(200, 2)) {
                return '#000000';
            } else {
                return '#FFFFFF';
            }
        }
    }
    public function readableColour2($hex)
    {
        if ($hex == "transparent" || empty($hex)) {
            return '#000000';
        } else {
            $r = hexdec(Tools::substr($hex, 1, 2));
            $g = hexdec(Tools::substr($hex, 3, 2));
            $b = hexdec(Tools::substr($hex, 5, 2));
        
            $squared_contrast = (
                $r * $r * .299 +
                $g * $g * .587 +
                $b * $b * .114
            );
            if ($squared_contrast > pow(200, 2)) {
                return '#000000';
            } else {
                return $hex;
            }
        }
    }
    
    public function luminance($hexcolor, $percent)
    {
    
    // validate hex string
        if ($hexcolor == "transparent" || empty($hexcolor)) {
            return 'transparent';
        } else {
            if (Tools::strlen($hexcolor) < 6) {
                $hexcolor = $hexcolor[0] . $hexcolor[0] . $hexcolor[1] . $hexcolor[1] . $hexcolor[2] . $hexcolor[2];
            }
            $hexcolor = array_map('hexdec', str_split(str_pad(str_replace('#', '', $hexcolor), 6, '0'), 2));
        
            foreach ($hexcolor as $i => $color) {
                $from = $percent < 0 ? 0 : $color;
                $to = $percent < 0 ? $color : 255;
                $pvalue = ceil(($to - $from) * $percent);
                $hexcolor[$i] = str_pad(dechex($color + $pvalue), 2, '0', STR_PAD_LEFT);
            }
        
            return '#' . implode($hexcolor);
        }
    }
    
    public function createStyleSheet()
    {
        $dfx_layout = Tools::getValue('dfx_layout', Configuration::get('dfx_layout'));
        $tlayouts = Tools::unSerialize(Configuration::get('dfx_themelayout'));
        $tlayout = $tlayouts[0];
            
        $theme_number = Configuration::get($tlayout.'_DRX_THEME_COLOR');
        $allcustom = $this->dfx_color_custom[$theme_number];
            
        $allfonts = Tools::unserialize(Configuration::get('fonts'));
        $title_fonts = Tools::getValue($tlayout.'_title_font', Configuration::get($tlayout.'_title_font'));
        $contentfonts = Tools::getValue($tlayout.'_content_font', Configuration::get($tlayout.'_content_font'));
            
        $header_fonts = "#header_fonts";
            
        $header_font = "";
        $content_font = "";
                
        if (Configuration::get($tlayout.'_title_font')) {
            foreach ($allfonts as $fonts => $value) {
                if ($title_fonts == 'customfont') {
                    $header_font =  Tools::getValue('custom_font_name', Configuration::get('custom_font_name'));
                } elseif ($title_fonts == $fonts) {
                    $header_font =  $value['name'];
                }
            }
        }
                        
        $content_fonts = "#content_fonts";
            
        if (Configuration::get($tlayout.'_content_font')) {
            foreach ($allfonts as $fonts => $value) {
                if ($contentfonts == 'customfont') {
                    $content_font =  Tools::getValue('custom_contentfont_name', Configuration::get('custom_contentfont_name'));
                } elseif ($contentfonts == $fonts) {
                    $content_font =  $value['name'];
                }
            }
        }
            
        // this is Color
        $color_replace_0 = "#template_bg";
        $color_0 = $allcustom['template_bg'];

        $color_replace_1 = "#body_bg";
        $color_1 = $allcustom['body_bg'];
            
        $color_replace_2 = "#menu_bg";
        $color_2 = $allcustom['menu_bg'];

        $color_replace_3 = "#menu_font";
        $color_3 = $this->readableColour($allcustom['menu_bg']);
            
        $color_replace_4 = "#button_hover_bg";
        $color_4 = $this->luminance($allcustom['primary'], -0.20);
                
        $color_replace_5 = "#button_hover_font";
        $color_5 = $this->readableColour($color_4);
            
            
        $color_replace_6 = "#primary_bg";
        $color_6 = $allcustom['primary'];
            
        $color_replace_7 = "#primary_color_font";
        $color_7 = $this->readableColour2($allcustom['primary']);
            
        $color_replace_8 = "#primary_font";
        $color_8 = $this->readableColour($allcustom['primary']);
            
        $color_replace_9 = "#secondary_bg";
        $color_9 = $allcustom['secondary'];
            
        $color_replace_10 = "#secondary_font";
        $color_10 = $this->readableColour($allcustom['secondary']);
            
        $color_replace_11 = "#sale_bg";
        $color_11 = $allcustom['sale_bg'];
            
        $color_replace_12 = "#sale_font";
        $color_12 = $this->readableColour($allcustom['sale_bg']);
            
        $color_replace_13 = "#new_bg";
        $color_13 = $allcustom['new_bg'];
            
        $color_replace_14 = "#new_font";
        $color_14 = $this->readableColour($allcustom['new_bg']);
            
        $color_replace_15 = "#footer_bg";
        $color_15 = $allcustom['footer_bg'];
            
        $color_replace_16 = "#footer_font";
        $color_16 = $this->readableColour($allcustom['footer_bg']);
            
        $color_replace_17 = "#topnav_bg";
        $color_17 = $allcustom['topnav_bg'];
            
        $color_replace_18 = "#topnav_font";
        $color_18 = $this->readableColour($allcustom['topnav_bg']);
            
        $color_replace_19 = "#header_bg";
        $color_19 = $allcustom['header_bg'];
            
        $color_replace_20 = "#headerfont";
        $color_20 = $this->readableColour($allcustom['header_bg']);
            
        $color_replace_21 = "#circle_font";
        $color_21 = $this->readableColour($this->readableColour($allcustom['header_bg']));
        
		$color_replace_22 = "#primary_opacity";
        $color_22 = $this->luminance($allcustom['primary'], 0.90);
		
        $body_pattern = "#body_pattern{}";
            

        $custom_css = Tools::getValue('custom_css', Configuration::get('custom_css'));

        $css = $custom_css;

        if ((Configuration::get($tlayout.'_body_bg_pattern')) && (Configuration::get($tlayout.'_body_bg_ext') == "")) {
            $css .= 'body.'.$tlayout.' {background-image: url(../img/patterns/' . Configuration::get($tlayout.'_body_bg_pattern') . '.png);}
		';
        }
        if (Configuration::get($tlayout.'_body_bg_ext')) {
            $css .= 'body.'.$tlayout.' {background-image: url(../img/'.$tlayout.'_body_background.' . Configuration::get($tlayout.'_body_bg_ext') . ');}
		';
            if (Configuration::get($tlayout.'_body_bg_fixed')) {
                $css .= '
			
			body.'.$tlayout.' {background-attachment:fixed;}
			';
            }
        }

        if (Configuration::get($tlayout.'_body_bg_repeat')) {
            switch (Configuration::get($tlayout.'_body_bg_repeat')) {
                case 1:
                    $repeat_option = 'background-repeat:repeat-x';
                    break;

                case 2:
                    $repeat_option = 'background-repeat:repeat-y';
                    break;

                case 3:
                    $repeat_option = 'background-repeat:no-repeat';
                    break;

                case 4:
                    $repeat_option = 'background-position:top center;background-repeat:repeat;background-attachment:fixed';
                    break;

                default:
                    $repeat_option = 'background-repeat:repeat';
            }

            $css .= 'body.'.$tlayout.' {' . $repeat_option . ';}
			';
        }

        if (Configuration::get($tlayout.'_body_bg_position')) {
            switch (Configuration::get($tlayout.'_body_bg_position')) {
                case 1:
                    $position_option = 'center top';
                    break;

                case 2:
                    $position_option = 'right top';
                    break;

                default:
                    $position_option = 'left top';
            }

            $css .= 'body.'.$tlayout.' {background-position: ' . $position_option . ';}
					';
        }

        $ftpThemeDir = $this->local_path."views/css";
        $oldfilename = $ftpThemeDir."/theme-custom-color.css";
        $themeCssPath = '/'."theme_color.css";
        $newfilename= $ftpThemeDir.$themeCssPath;

        $this->createCustomThemeCss(
            $oldfilename,
            $newfilename,
            $header_fonts,
            $header_font,
            $content_fonts,
            $content_font,
            $color_replace_0,
            $color_0,
            $color_replace_1,
            $color_1,
            $color_replace_2,
            $color_2,
            $color_replace_3,
            $color_3,
            $color_replace_4,
            $color_4,
            $color_replace_5,
            $color_5,
            $color_replace_6,
            $color_6,
            $color_replace_7,
            $color_7,
            $color_replace_8,
            $color_8,
            $color_replace_9,
            $color_9,
            $color_replace_10,
            $color_10,
            $color_replace_11,
            $color_11,
            $color_replace_12,
            $color_12,
            $color_replace_13,
            $color_13,
            $color_replace_14,
            $color_14,
            $color_replace_15,
            $color_15,
            $color_replace_16,
            $color_16,
            $color_replace_17,
            $color_17,
            $color_replace_18,
            $color_18,
            $color_replace_19,
            $color_19,
            $color_replace_20,
            $color_20,
            $color_replace_21,
            $color_21,
			$color_replace_22,
            $color_22,
            $body_pattern,
            $css
        );
		$this->_clearCache('*');
    }
    
    
    public function createCustomThemeCss(
        $oldfilename,
        $newfilename,
        $header_fonts,
        $header_font,
        $content_fonts,
        $content_font,
        $color_replace_0,
        $color_0,
        $color_replace_1,
        $color_1,
        $color_replace_2,
        $color_2,
        $color_replace_3,
        $color_3,
        $color_replace_4,
        $color_4,
        $color_replace_5,
        $color_5,
        $color_replace_6,
        $color_6,
        $color_replace_7,
        $color_7,
        $color_replace_8,
        $color_8,
        $color_replace_9,
        $color_9,
        $color_replace_10,
        $color_10,
        $color_replace_11,
        $color_11,
        $color_replace_12,
        $color_12,
        $color_replace_13,
        $color_13,
        $color_replace_14,
        $color_14,
        $color_replace_15,
        $color_15,
        $color_replace_16,
        $color_16,
        $color_replace_17,
        $color_17,
        $color_replace_18,
        $color_18,
        $color_replace_19,
        $color_19,
        $color_replace_20,
        $color_20,
        $color_replace_21,
        $color_21,
		$color_replace_22,
        $color_22,
        $body_pattern,
        $css
    ) {
        $content= Tools::file_get_contents($oldfilename);
        
        $content_separate=explode($header_fonts, $content);
        $content=implode($header_font, $content_separate);
        
        $content_separate=explode($content_fonts, $content);
        $content=implode($content_font, $content_separate);
        
        $content_separate=explode($color_replace_0, $content);
        $content=implode($color_0, $content_separate);
            
        $content_separate=explode($color_replace_1, $content);
        $content=implode($color_1, $content_separate);
            
        $content_separate=explode($color_replace_2, $content);
        $content=implode($color_2, $content_separate);
            
        $content_separate=explode($color_replace_3, $content);
        $content=implode($color_3, $content_separate);
            
        $content_separate=explode($color_replace_4, $content);
        $content=implode($color_4, $content_separate);
            
        $content_separate=explode($color_replace_5, $content);
        $content=implode($color_5, $content_separate);
            
        $content_separate=explode($color_replace_6, $content);
        $content=implode($color_6, $content_separate);
            
        $content_separate=explode($color_replace_7, $content);
        $content=implode($color_7, $content_separate);
            
        $content_separate=explode($color_replace_8, $content);
        $content=implode($color_8, $content_separate);
            
        $content_separate=explode($color_replace_9, $content);
        $content=implode($color_9, $content_separate);
            
        $content_separate=explode($color_replace_10, $content);
        $content=implode($color_10, $content_separate);
            
        $content_separate=explode($color_replace_11, $content);
        $content=implode($color_11, $content_separate);
            
        $content_separate=explode($color_replace_12, $content);
        $content=implode($color_12, $content_separate);
            
        $content_separate=explode($color_replace_13, $content);
        $content=implode($color_13, $content_separate);
            
        $content_separate=explode($color_replace_14, $content);
        $content=implode($color_14, $content_separate);
            
        $content_separate=explode($color_replace_15, $content);
        $content=implode($color_15, $content_separate);
            
        $content_separate=explode($color_replace_16, $content);
        $content=implode($color_16, $content_separate);
            
        $content_separate=explode($color_replace_17, $content);
        $content=implode($color_17, $content_separate);
            
        $content_separate=explode($color_replace_18, $content);
        $content=implode($color_18, $content_separate);
            
        $content_separate=explode($color_replace_19, $content);
        $content=implode($color_19, $content_separate);
            
        $content_separate=explode($color_replace_20, $content);
        $content=implode($color_20, $content_separate);
            
        $content_separate=explode($color_replace_21, $content);
        $content=implode($color_21, $content_separate);
		
		$content_separate=explode($color_replace_22, $content);
        $content=implode($color_22, $content_separate);
            
        $content_separate=explode($body_pattern, $content);
        $content=implode($css, $content_separate);

        
        file_put_contents($newfilename, $content);
    }
	
	public function hookActionProductAdd($params)
    {
        $this->_clearCache('*');
    }

    public function hookActionProductUpdate($params)
    {
        $this->_clearCache('*');
    }

    public function hookActionProductDelete($params)
    {
        $this->_clearCache('*');
    }

    public function _clearCache($template, $cache_id = null, $compile_id = null)
    {
        parent::_clearCache($this->templateFile);
    }
	
	public function renderWidget($hookName = null, array $configuration = [])
    {
        if (preg_match('/^displayThemeControll\d*$/', $hookName)) {
			$this->templateFile = 'module:dfxthemeoptions/views/templates/hook/live_configurator.tpl';
			if (!$this->isCached($this->templateFile, $this->getCacheId('dfxthemeoptions'))) {
				$this->smarty->assign($this->getWidgetVariables($hookName, $configuration));
			}
			return $this->fetch($this->templateFile, $this->getCacheId('dfxthemeoptions'));
		}  
    }

    public function getWidgetVariables($hookName = null, array $configuration = [])
    {
       if (preg_match('/^displayThemeControll\d*$/', $hookName)) {
          $themecontroll = $this->getThemeControll();
          if (!empty($themecontroll)) {
            return $themecontroll;
          }
       } 
       return false;
    }
	
	public function getThemeControll()
    {
        $theme_name = Tools::unSerialize(Configuration::get('dfx_themename'));
   
        $dfx_layout = Configuration::get('dfx_layout');
        $tlayouts = Tools::unSerialize(Configuration::get('dfx_themelayout'));
        $tlayout = $tlayouts[0];

        $body_bg_ext = Tools::getValue($tlayout.'_body_bg_ext', Configuration::get($tlayout.'_body_bg_ext'));
        $allcustom = Tools::unSerialize(Configuration::get('dfx_color_custom'));
   
        $body_bg_pattern = $tlayout.'_body_bg_pattern';
		$body_bgpattern = Tools::getValue($tlayout.'_body_bg_pattern', Configuration::get($tlayout.'_body_bg_pattern'));

        return array(
          'theme_name' => $theme_name,
          'allcustom' => $allcustom,
          'tlayout' => $tlayout,
          'body_bg_pattern' => $body_bg_pattern,
          'allfonts' => Tools::unserialize(Configuration::get('fonts')),
          'dfx_theme_color' => Tools::getValue($tlayout.'_DRX_THEME_COLOR', Configuration::get($tlayout.'_DRX_THEME_COLOR')),
          'title_font' => Tools::getValue($tlayout.'_title_font', Configuration::get($tlayout.'_title_font')),
          'content_font' => Tools::getValue($tlayout.'_content_font', Configuration::get($tlayout.'_content_font')),
          'body_bg_ext' => $body_bg_ext,
		  'body_bgpattern' => $body_bgpattern,
        );
    }
	
	public function getCacheId($id_product = null)
    {
        return parent::getCacheId().'|'.(int)$id_product;
    }
	
    public function hookDisplayListImage($params)
    {
	  $id_product = (int)$params['product']->id_product;

		  if (!$this->isCached('listthumb.tpl', $this->getCacheId($id_product))) {
           $prod = new Product($id_product);
           $images = $prod->getImages((int)$this->context->cookie->id_lang);
           $product_images = array();
           foreach ($images as $image) {
              $product_images[(int)$image['id_image']] = $image;
           }
           $this->smarty->assign(array(
             'prodImages' => $product_images,
             'product' => $params['product'],
			 'no_mobile' => (!$this->is_mobile ? true : false),
           ));
		  }
          return $this->display($this->_path, '/views/templates/front/listthumb.tpl', $this->getCacheId($id_product));

    }
	
	public function hookDisplayBrandImage($params)
    {
        $manufacturers = array();
        $image_size = array();
        $prod_brand = array();
        $id_product = (int)$params['product']->id_product;
       
	    if (!$this->isCached('brandimg.tpl', $this->getCacheId($id_product))) {
        $manufacturers = $this->getProductManufacture($id_product);
        foreach ($manufacturers as $manufacturer) {
            $manufacturer['image'] = $this->context->link->getManufacturerImageLink($manufacturer['id_manufacturer'], 'brand_default');
                    
            if (file_exists(_PS_MANU_IMG_DIR_.$manufacturer['id_manufacturer'].'.jpg')) {
                $prod_brand[] = $manufacturer;
            }
        }

        $image_size = ImageType::getByNameNType('brand_default', 'manufacturers');
        
			$this->smarty->assign(array(
			  'prod_brand' => $prod_brand,
			  'imageSize' => $image_size,
			));
		}
		return $this->display($this->_path, '/views/templates/front/brandimg.tpl', $this->getCacheId($id_product));
    }
    
    public static function getProductManufacture($id_product)
    {
        $row = Db::getInstance(_PS_USE_SQL_SLAVE_)->executeS(
            '
			SELECT `id_manufacturer` FROM `' . _DB_PREFIX_ . 'product`
			WHERE `id_product` = ' . (int) $id_product
        );

        return $row;
    }
}
